App.SecExp.unit = (function() {
	return {
		list,
		bulkUpgrade,
		humanLoyaltyChanges,
		fixBroken,
		gen:generate,
		deployMenu,
		humanUpgradeList,
		genID,
		replenish,
		replenishAll,
		isDeployed,
		humanSquads
	};

	function base(string, operation = 'print', value = 0) {
		let target, x = string.toLowerCase();

		if (x === "slaves") {
			target = V.menials;
		} else if (x === "militia") {
			target = V.SecExp.units.militia.free;
		} else if (x === "mercs") {
			target = V.SecExp.units.mercs.free;
		}

		if (operation === 'print') {
			return target;
		} else if (operation === 'add') {
			target += value;
		} else if (operation === 'remove') {
			target -= value;
		} else if (operation === 'set') {
			target = value;
		} else if (operation === 'can upgrade') {
			if (string === 'bots') {
				return V.cash >= 500;
			} else {
				return target > 0;
			}
		}
	}

	/** Lists all potential military units as an array.
	 *
	 */
	 function list() {
		return ['bots', 'slaves', 'militia', 'mercs'];
	 }

	/** Creates a bulk upgrade link for the unit that is passed.
	 * @param {object} [unit] the unit to be checked.
	 */
	function bulkUpgrade(unit) {
		unit = Array.isArray(unit) ? unit : [unit];
		let el = document.createElement("a");

		function upgradeUnit(x) {
			Object.assign(x, {
				maxTroops: 50, equip: 3, commissars: 2,
				cyber: 1, medics: 1
			});
			if (V.SF.Active >= 1) {
				x.SF = 1;
			} else {
				x.SF = 0;
			}
		}

		function getCost(x) {
			let cost = 0;
			const equipUpgradeCost = 250;
			if (x.maxTroops < 50) {
				cost += 5000 + (((50 - x.maxTroops) /10) * equipUpgradeCost * (x.equip + x.commissars + x.cyber + x.SF));
			}
			if (x.equip < 3) {
				cost += (equipUpgradeCost * x.maxTroops + 1000) * (3 - x.equip);
			}
			if (x.commissars < 2) {
				cost += (equipUpgradeCost * x.maxTroops + 1000) * (2 - x.commissars);
			}
			if ((V.prostheticsUpgrade >= 2 || V.researchLab.advCombatPLimb === 1) && x.cyber === 0) {
				cost += equipUpgradeCost * x.maxTroops + 2000;
			}
			if (x.medics === 0) {
				cost += equipUpgradeCost * x.maxTroops + 1000;
			}
			if (V.SF.Toggle && V.SF.Active >= 1 && x.SF === 0) {
				cost += equipUpgradeCost * x.maxTroops + 5000;
			}
			return Math.ceil(cost *= 1.1);
		}

		const price = unit.map(getCost).reduce((acc, cur) => acc + cur, 0);
		if (price > 0) {
			el.append(App.UI.DOM.link(`Bulk upgrade for ${cashFormat(price)}`, () => {
				unit.map(upgradeUnit).reduce((acc, cur) => acc + cur, 0);
				cashX(-price, "securityExpansion");
			},
			[], passage()
			));
		}
		return el;
	}

	/** Reports changes to the supplied unit's loyalty.
	 * @param {FC.SecExp.PlayerHumanUnitData} input the unit type to be checked.
	 * @param {FC.SecExp.PlayerHumanUnitType} type
	 * @returns {HTMLDivElement}
	 */
	function humanLoyaltyChanges(input, type) {
		let loyaltyChange = 0, el = document.createElement("div");

		el.append(`${input.platoonName}: `);
		if (V.SecExp.buildings.barracks && V.SecExp.buildings.barracks.loyaltyMod >= 1) {
			el.append("is periodically sent to the indoctrination facility in the barracks for thought correction therapy. ");
			loyaltyChange += 2 * V.SecExp.buildings.barracks.loyaltyMod;
		}
		if (input.commissars >= 1) {
			el.append("The commissars attached to the unit carefully monitor the officers and grunts for signs of insubordination. ");
			loyaltyChange += 2 * input.commissars;
		}
		if (V.SecExp.edicts.defense.soldierWages === 2) {
			if (type === 'slaves') {
				el.append("The slaves greatly appreciate the generous wage given to them for their service as soldiers. Occasions to earn money for a slave are scarce after all. ");
			} else if (type === 'militia') {
				el.append("The soldiers greatly appreciate the generous wage given to them for their service. They are proud to defend their homes while making a small fortune out of it. ");
			} else if (type === 'mercs') {
				el.append("The mercenaries greatly appreciate the generous wage given to them for their service. After all coin is the fastest way to reach their hearts. ");
			}
			loyaltyChange += random(5, 10);
		} else if (V.SecExp.edicts.defense.soldierWages === 1) {
			if (type === 'slaves') {
				el.append("The slaves appreciate the wage given to them for their service as soldiers, despite it being just adequate. Occasions to earn money for a slave are scarce after all. ");
			} else if (type === 'militia') {
				el.append("The soldiers appreciate the wage given to them for their service, despite it being just adequate. They are proud to defend their homes, though at the cost of possible financial gains. ");
			} else if (type === 'mercs') {
				el.append("The mercenaries do not appreciate the barely adequate wage given to them for their service. Still their professionalism keeps them determined to finish their contract. ");
			}
			loyaltyChange += random(-5, 5);
		} else {
			if (type === 'slaves') {
				el.append("The slaves do not appreciate the low wage given to them for their service as soldiers, but occasions to earn money for a slave are scarce, so they're not too affected by it. ");
			} else if (type === 'militia') {
				el.append("The soldiers do not appreciate the low wage given to them for their service. Their sense of duty keeps them proud of their role as defenders of the arcology, but many do feel its financial weight. ");
			} else if (type === 'mercs') {
				el.append("The mercenaries do not appreciate the low wage given to them for their service. Their skill would be better served by a better contract and this world does not lack demand for guns for hire. ");
			}
			loyaltyChange -= random(5, 10);
		}
		if (type === 'slaves' && V.SecExp.edicts.defense.privilege.slaveSoldier) {
			el.append("Allowing them to hold material possessions earns you their devotion and loyalty. ");
			loyaltyChange += random(1, 2);
		}
		if (type === 'militia' && V.SecExp.edicts.defense.privilege.militiaSoldier) {
			el.append("Allowing them to avoid rent payment for their military service earns you their happiness and loyalty. ");
			loyaltyChange += random(1, 2);
		}
		if (type === 'mercs' && V.SecExp.edicts.defense.privilege.mercSoldier) {
			el.append("Allowing them to keep part of the loot gained from your enemies earns you their trust and loyalty. ");
			loyaltyChange += random(1, 2);
		}

		el.append("This week, the loyalty of this unit ");
		if (loyaltyChange > 0) {
			 App.UI.DOM.appendNewElement("span", el, "increased.", "green");
		} else if (loyaltyChange === 0) {
			App.UI.DOM.appendNewElement("span", el, "did not change.", "yellow");
		} else {
			App.UI.DOM.appendNewElement("span", el, "decreased.", "red");
		}
		input.loyalty = Math.clamp(input.loyalty + loyaltyChange, 0, 100);
		if (input.training < 100 && V.SecExp.buildings && V.SecExp.buildings.barracks.training >= 1) {
			let options = document.createElement("div");
			options.append("The unit is able to make use of the training facilities to better prepare its soldiers, slowly increasing their experience level. ");
			el.append(options);
			input.training += random(2, 4) * 1.5 * V.SecExp.buildings.barracks.training;
		}

		return el;
	}

	/** Repairs unit if needed.
	 * @param {string} [unit] unit type
	 * @param {object} [squad] the unit to be checked.
	 */
	function fixBroken(unit, squad) {
		squad.SF = squad.SF || 0;
		if (!jsDef(squad.ID)) {
			squad.ID = genID();
		}
		squad.cyber = squad.cyber || 0;
		squad.commissars = squad.commissars || 0;
		squad.maxTroops = squad.maxTroops || 30;
		squad.troops = Math.clamp(squad.troops, 0, squad.maxTroops);
		squad.training = Math.clamp(squad.training, 0, 100);
		if (V.SF.Active < 1) {
			squad.SF = 0;
		}
		if (squad.platoonName.contains('undefined')) {
			if (unit === 'slaves') {
				squad.platoonName = squad.platoonName.replace('undefined', 'slave platoon');
			} else if (unit === 'militia') {
				squad.platoonName = squad.platoonName.replace('undefined', "citizens' platoon");
			} else if (unit === 'mercs') {
				squad.platoonName = squad.platoonName.replace('undefined', 'mercenary platoon');
			}
		}
	}

	/** Creates the requested unit object.
	 * @param {string} [type] the unit type to be created.
	 * @param {object} [updater] previous variables to check against.
	 */
	function generate(type, updater = {}) {
		if (!jsDef(V.SecExp.units[type]) || Object.entries(updater).length > 0) {
			V.SecExp.units[type] = V.SecExp.units[type] || {};
			if (type !== "bots") {
				V.SecExp.units[type].created = V.SecExp.units[type].created || updater.count || 0;
				V.SecExp.units[type].dead = V.SecExp.units[type].dead || updater.killed ||0;
				V.SecExp.units[type].squads = V.SecExp.units[type].squads || updater.squads ||[];
				if (type !== 'slaves') {
					V.SecExp.units[type].free = V.SecExp.units[type].free || updater.free || 0;
				}

				if (type === 'slaves') {
					V.SecExp.units.slaves.defaultName = V.SecExp.units.slaves.defaultName || "slave platoon";
				} else if (type === 'militia') {
					V.SecExp.units.militia.defaultName = V.SecExp.units.militia.defaultName || "citizens' platoon";
				} else if (type === 'mercs') {
					V.SecExp.units.mercs.defaultName = V.SecExp.units.mercs.defaultName || "mercenary platoon";
				}
			}
			return;
		}

		let newUnit = {
			ID: -1, equip: 0, active: 1, isDeployed: 0, maxTroops: 30, troops: 30
		};
		if (type !== "bots") {
			Object.assign(newUnit, {
				training: 0, cyber: 0, medics: 0,
				SF: 0, commissars: 0, battlesFought: 0,
				loyalty: jsRandom(40, 60),
				ID: genID(),
				platoonName: `${ordinalSuffix(++V.SecExp.units[type].created)} ` + V.SecExp.units[type].defaultName
			});

			if (type === "slaves") {
				newUnit.troops = Math.min(newUnit.maxTroops, V.menials);
				V.menials -= newUnit.troops;
			} else {
				newUnit.troops = Math.min(newUnit.maxTroops, V.SecExp.units[type].free);
				V.SecExp.units[type].free -= newUnit.troops;
			}
		}
		return newUnit;
	}

	/** Display's the deploy menu for the unit.
	 * @param {FC.SecExp.PlayerHumanUnitData} input the unit to be checked.
	 * @param {FC.SecExp.PlayerHumanUnitType} type
	 */
	function deployMenu(input, type, count = 0) {
		let el = document.createElement("div"), options = document.createElement("div");
		const canDeploy = input.isDeployed === 0 && App.SecExp.battle.deployableUnits() > 0;

		if (input.active === 1 && input.troops > 0) {
			if (type === "bots") {
				$(el).append(App.SecExp.getUnit("Bots").describe());
			} else {
				$(el).append(App.SecExp.getUnit(capFirstChar(type), count).describe());
			}
			options = document.createElement("div");
			options.append(App.UI.DOM.link(`${canDeploy ? 'Deploy' : 'Remove'} the unit`, () => {
				input.isDeployed = canDeploy ? 1 : 0;
				V.SecExp.war.saveValid = 0;
				App.UI.reload();
			}
			));
		}
		el.append(options);
		return el;
	}

	/** Prints a list of upgrades that can be applied to the passed human unit.
	 * @param {FC.SecExp.PlayerHumanUnitData} input the human unit to be checked.
	 */
	function humanUpgradeList(input) {
		const equipUpgradeCost = 250;
		let el = document.createElement("div"), options = document.createElement("div");

		if (input.maxTroops < 50) {
			options.append(`For ${cashFormat(5000 + 10 * equipUpgradeCost * (input.equip + input.commissars + input.cyber + input.SF))} provide this unit's `);
			options.append(App.UI.DOM.link("officers with intensive training", () => {
				input.maxTroops += 10;
				cashX(-(5000 + 10 * equipUpgradeCost * (input.equip + input.commissars + input.cyber + input.SF)), "securityExpansion");
			},
			[], passage()
			));
			options.append(` to increase the maximum number of soldiers in the unit by 10.`);
		} else {
			options.append(`Your officers reached their peak. Further training will have little impact on the number of troops they can effectively lead.`);
		}
		el.append(options);

		options = document.createElement("div");
		if (input.equip < 3) {
			options.append(`For ${cashFormat(equipUpgradeCost * input.maxTroops + 1000)} invest in `);
			options.append(App.UI.DOM.link("better equipment", () => {
				input.equip++;
				cashX(-(equipUpgradeCost * input.maxTroops + 1000), "securityExpansion");
			},
			[], passage()
			));
			options.append(` to increase this unit's attack and defense by 15% per investement.`);
		} else {
			options.append(`The unit is equipped with state of the art weaponry and equipment.`);
		}
		el.append(options);

		options = document.createElement("div");
		if (input.commissars < 2) {
			options.append(`For ${cashFormat(equipUpgradeCost * input.maxTroops + 1000)} attach `);
			options.append(App.UI.DOM.link("commissars", () => {
				input.commissars++;
				cashX(-(equipUpgradeCost * input.maxTroops + 1000), "securityExpansion");
			},
			[], passage()
			));
			options.append(` to slowly increase this unit's loyalty.`);
		}
		if (input.commissars === 1) {
			options.append(" The unit has a commissar detachment, keeping under control the ambitions of the unit's officers.");
		} else if (input.commissars === 2) {
			options.append(" The unit has a perfectly trained and loyal commissar detachment, keeping under control the ambitions of the unit's officers.");
		}
		el.append(options);

		options = document.createElement("div");
		if (V.prostheticsUpgrade >= 2 || V.researchLab.advCombatPLimb === 1) {
			if (input.cyber === 0) {
				options.append(`For ${cashFormat(equipUpgradeCost * input.maxTroops + 1000)} `);
				options.append(App.UI.DOM.link("augment all soldiers of the unit", () => {
					input.cyber++;
					cashX(-(equipUpgradeCost * input.maxTroops + 2000), "securityExpansion");
				},
				[], passage()
				));
				options.append(` with high tech cyber enhancements that will increase attack, defense and base hp values.`);
			} else {
				options.append('The unit is equipped with advanced cybernetic enhancements.');
			}
		}
		el.append(options);

		options = document.createElement("div");
		if (input.medics === 0) {
			options.append(`For ${cashFormat(equipUpgradeCost * input.maxTroops + 1000)} `);
			options.append(App.UI.DOM.link("attach trained medics to the unit", () => {
				input.medics++;
				cashX(-(equipUpgradeCost * input.maxTroops + 1000), "securityExpansion");
			},
			[], passage()
			));
			options.append(' which will decrease the number of casualties suffered during battle.');
		} else {
			options.append('The unit has a medic detachment following it into battle, decreasing the number of casualties the unit suffers');
		}
		el.append(options);

		if (V.SF.Toggle && V.SF.Active >= 1) {
			options = document.createElement("div");

			if (input.SF === 0) {
				options.append(`For ${cashFormat(equipUpgradeCost * input.maxTroops + 5000)} `);
				options.append(App.UI.DOM.link("attach Special Force advisors", () => {
					input.SF++;
					cashX(-(equipUpgradeCost * input.maxTroops + 5000), "securityExpansion");
				},
				[], passage()
				));
				options.append(' which will slightly increase the base stats of the unit.');
			} else {
				options.append(`The unit has attached advisors from ${V.SF.Lower} that will help the squad remain tactically aware and active.`);
			}
			el.append(options);
		}

		return el;
	}

	/** Generate a unit ID for a new unit
	 * @returns {number}
	 */
	function genID() {
		return Math.max(
			App.SecExp.unit.humanSquads().map(u => u.ID).reduce((acc, cur) => Math.max(acc, cur), 0)
		) + 1;
	}

	function replenish(squad, type) {
		const oldTroops = squad.troops;

		if (base(type) >= squad.maxTroops - squad.troops) {
			if (type === "slaves") {
				V.menials -= (squad.maxTroops - squad.troops);
			} else if (type === "militia") {
				V.SecExp.units.militia.free -= (squad.maxTroops - squad.troops);
			} else if (type === "mercs") {
				V.SecExp.units.mercs.free -= (squad.maxTroops - squad.troops);
			}
			squad.troops = squad.maxTroops;
		} else {
			if (type === "slaves") {
				squad.troops += V.menials;
				V.menials = 0;
			} else if (type === "militia") {
				squad.troops += V.SecExp.units.militia.free;
				V.SecExp.units.militia.free = 0;
			} else if (type === "mercs") {
				squad.troops += V.SecExp.units.mercs.free;
				V.SecExp.units.mercs.free = 0;
			}
		}
		const expLoss = (squad.troops - oldTroops) / squad.troops;
		squad.training -= squad.training * expLoss;
	}

	function replenishAll() {
		let el = document.createElement("div");
		let woundedUnit = [];

		if (V.SecExp.units.bots.troops < V.SecExp.units.bots.maxTroops && V.cash >= 500) {
			woundedUnit.push('bots');
		}

		for (const unit of list().slice(1)) {
			for (const squad of V.SecExp.units[unit].squads) {
				if (!woundedUnit.contains(unit) && squad.troops < squad.maxTroops && base(unit, 'can upgrade')) {
					woundedUnit.push(unit);
				}
			}
		}

		if (woundedUnit.length > 0) {
			el.append(App.UI.DOM.link("Replenish all units", () => {
				for (const unit of woundedUnit) {
					if (unit === 'bots') {
						cashX(-((V.SecExp.units.bots.maxTroops - V.SecExp.units.bots.troops) * 500), "securityExpansion");
						V.SecExp.units.bots.troops = V.SecExp.units.bots.maxTroops;
					} else {
						for (const squad of V.SecExp.units[unit].squads) {
							replenish(squad, unit);
						}
					}
				}
			},
			[], passage()
			));
			el.append(` Will replenish units as long as requirements are met.\n\n`);
		}

		return el;
	}

	function isDeployed(unit) {
		return (V.citizenRebellion === 0 && V.slaveRebellion === 0 && unit.isDeployed === 1) || ((V.citizenRebellion > 0 || V.slaveRebellion > 0) && unit.active === 1 && !V.SecExp.war.rebellingID.includes(unit.ID));
	}

	function humanSquads() {
		let array = [];
		for (const unit of list().slice(1)) {
			for (const squad of V.SecExp.units[unit].squads) {
				array.push(squad);
			}
		}
		return array;
	}
})();

/** Player unit factory - get a unit based on its type and index
 * @param {PlayerHumanUnitType} type - "Bots", "Militia", "Slaves", "Mercs", or "SF"
 * @param {number} [index] - must be supplied if type is not "Bots"
 * @returns {App.SecExp.Unit}
 */
App.SecExp.getUnit = function(type, index) {
	if (type === "Bots") {
		return new App.SecExp.DroneUnit(V.SecExp.units.bots, App.SecExp.BaseDroneUnit);
	} else if (type === "SF") {
		return new App.SecExp.SFUnit();
	} else if (typeof index !== "number") {
		throw `Bad index for unit type ${type}: ${index}`;
	}

	switch (type) {
		case "Militia":
			return new App.SecExp.HumanUnit(V.SecExp.units.militia.squads[index], App.SecExp.BaseMilitiaUnit, type);
		case "Slaves":
			return new App.SecExp.HumanUnit(V.SecExp.units.slaves.squads[index], App.SecExp.BaseSlaveUnit, type);
		case "Mercs":
			return new App.SecExp.HumanUnit(V.SecExp.units.mercs.squads[index], App.SecExp.BaseMercUnit, type);
		default:
			throw `Unknown unit type: ${type}`;
	}
};

/** Enemy unit factory - get a unit based on its type and basic data
 * @param {FC.SecExp.EnemyUnitType} type - "raiders", "free city", "old world", or "freedom fighters"
 * @param {number} troops
 * @param {number} equipment
 * @returns {App.SecExp.Unit}
 */
App.SecExp.getEnemyUnit = function(type, troops, equipment) {
	const baseUnitMap = new Map([
		["raiders", App.SecExp.BaseRaiderUnit],
		["free city", App.SecExp.BaseFreeCityUnit],
		["old world", App.SecExp.BaseOldWorldUnit],
		["freedom fighters", App.SecExp.BaseFreedomFighterUnit],
	]);
	const unitData = {
		troops: troops,
		maxTroops: troops,
		equip: equipment
	};
	return new App.SecExp.EnemyUnit(unitData, baseUnitMap.get(type));
};

/** Irregular unit factory - get an irregular unit (without organization/upgrade bonuses) based on its type and basic data
 * @param {string} type - "Militia", "Slaves", or "Mercs"
 * @param {number} troops
 * @param {number} equipment
 * @returns {App.SecExp.Unit}
 */
App.SecExp.getIrregularUnit = function(type, troops, equipment) {
	const baseUnitMap = new Map([
		["Militia", App.SecExp.BaseMilitiaUnit],
		["Slaves", App.SecExp.BaseSlaveUnit],
		["Mercs", App.SecExp.BaseMercUnit],
	]);
	const unitData = {
		troops: troops,
		maxTroops: troops,
		equip: equipment
	};

	return new App.SecExp.IrregularUnit(unitData, baseUnitMap.get(type));
};

/** Equipment multiplier (static balance variable) */
App.SecExp.equipMod = 0.15;

/** Turn a loyalty value into a corresponding bonus factor
 * @param {number} value range: [0-100]
 * @returns {number} bonus - range: [0.0-0.3], cap at input 67
 */
App.SecExp.loyaltyValueToBonusFactor = function(value) {
	return Math.min(value * 3 / 670, 0.3);
};

/** Turn a training value into a corresponding bonus factor
 * @param {number} value range: [0-100]
 * @returns {number} bonus - range: [0.0-0.5], cap at input 67
 */
App.SecExp.trainingValueToBonusFactor = function(value) {
	return Math.min(value * 3 / 400, 0.5);
};

/** Gets the bonus values provided for completing weapon manufacturing upgrades.
 * @param {string} type - unit type to check.
 * @returns {object} bouns values after checking for completed upgrades.
 */
App.SecExp.getAppliedUpgrades = function(type) {
	let hp = 0, morale = 0, def = 0, attack = 0;
	if (V.SecExp.buildings.weapManu) {
		if (type === 'drone') {
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(-3)) {
				hp++;
			}
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(-2)) {
				def++;
			}
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(-1)) {
				attack++;
			}
		} else if (type === 'human') {
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(0)) {
				attack++;
			}
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(1)) {
				def++;
			}
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(2)) {
				hp++;
			}
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(3)) {
				morale += 10;
			}
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(4)) {
				attack++; def++;
			}
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(5)) {
				hp++; morale += 10;
			}
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(6)) {
				attack++; def++;
			}
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(7)) {
				hp++; morale += 10;
			}
			if (V.SecExp.buildings.weapManu.upgrades.completed.includes(8)) {
				attack++; def++; hp++; morale += 10;
			}
			if (V.arcologies[0].FSNeoImperialistLaw1) {
				attack++;
			}
		}
	}
	return {
		attack: attack, defense: def, hp: hp, morale: morale
	};
};

App.SecExp.getEdictUpgradeVal = (function() {
	const data = {
		Militia: new Map([
			["legionTradition", {defense: 2, morale: 5, hp: 1}],
			["imperialTradition", {defense: 1, morale: 5, hp: 1}],
			["pharaonTradition", {attack: 2, defense: 2, morale: 10}],
			["sunTzu", {attack: 1, defense: 1, morale: 5}],
			["eliteOfficers", {morale: 5}],
			["lowerRequirements", {defense: -1, hp: -1}]
		]),
		Slave: new Map([
			["mamluks", {attack: 2, morale: 10, hp: 1}],
			["sunTzu", {attack: 1, defense: 1, morale: 5}],
			["eliteOfficers", {morale: -5}],
			["martialSchool", {morale: 5}]
		]),
		Merc: new Map([
			["eagleWarriors", {attack: 4, defense: -2, morale: 10}],
			["ronin", {attack: 2, defense: 2, morale: 10}],
			["sunTzu", {attack: 1, defense: 1, morale: 5}],
			["imperialTradition", {attack: 1, defense: 2, morale: 5}],
		])
	};

	/** Get the total edict upgrade effect on a particular stat for a particular unit
	 * @param {string} unitType
	 * @param {string} stat
	 * @returns {number}
	 */
	function getNetEffect(unitType, stat) {
		let retval = 0;
		for (const [key, val] of data[unitType]) {
			if (V.SecExp.edicts.defense[key] > 0 && val[stat]) {
				retval += val[stat];
			}
		}
		return retval;
	}

	return getNetEffect;
})();

/**
 * @interface
 * @typedef {object} BaseUnit
 * @property {number} attack
 * @property {number} defense
 * @property {number} morale
 * @property {number} hp
 */

/** @implements {BaseUnit} */
App.SecExp.BaseMilitiaUnit = class BaseMilitiaUnit {
	static get attack() {
		return 7 + App.SecExp.getAppliedUpgrades('human').attack + App.SecExp.getEdictUpgradeVal("Militia", "attack");
	}

	static get defense() {
		return 5 + App.SecExp.getAppliedUpgrades('human').defense + App.SecExp.getEdictUpgradeVal("Militia", "defense");
	}

	static get morale() {
		return 140 + App.SecExp.getAppliedUpgrades('human').morale + App.SecExp.getEdictUpgradeVal("Militia", "morale");
	}

	static get hp() {
		return 3 + App.SecExp.getAppliedUpgrades('human').hp + App.SecExp.getEdictUpgradeVal("Militia", "hp");
	}
};


/** @implements {BaseUnit} */
App.SecExp.BaseSlaveUnit = class BaseSlaveUnit {
	static get attack() {
		return 8 + App.SecExp.getAppliedUpgrades('human').attack + App.SecExp.getEdictUpgradeVal("Slave", "attack");
	}

	static get defense() {
		return 3 + App.SecExp.getAppliedUpgrades('human').defense + App.SecExp.getEdictUpgradeVal("Slave", "defense");
	}

	static get morale() {
		return 110 + App.SecExp.getAppliedUpgrades('human').morale + App.SecExp.getEdictUpgradeVal("Slave", "morale");
	}

	static get hp() {
		return 3 + App.SecExp.getAppliedUpgrades('human').hp + App.SecExp.getEdictUpgradeVal("Slave", "hp");
	}
};

/** @implements {BaseUnit} */
App.SecExp.BaseMercUnit = class BaseMercUnit {
	static get attack() {
		return 8 + App.SecExp.getAppliedUpgrades('human').attack + App.SecExp.getEdictUpgradeVal("Merc", "attack");
	}

	static get defense() {
		return 4 + App.SecExp.getAppliedUpgrades('human').defense + App.SecExp.getEdictUpgradeVal("Merc", "defense");
	}

	static get morale() {
		return 125 + App.SecExp.getAppliedUpgrades('human').morale + App.SecExp.getEdictUpgradeVal("Merc", "morale");
	}

	static get hp() {
		return 4 + App.SecExp.getAppliedUpgrades('human').hp + App.SecExp.getEdictUpgradeVal("Merc", "hp");
	}
};

/** @implements {BaseUnit} */
App.SecExp.BaseDroneUnit = class BaseDroneUnit {
	static get attack() {
		return 7 + App.SecExp.getAppliedUpgrades('drone').attack;
	}

	static get defense() {
		return 3 + App.SecExp.getAppliedUpgrades('drone').defense;
	}

	static get morale() {
		return 200;
	}

	static get hp() {
		return 3 + App.SecExp.getAppliedUpgrades('drone').hp;
	}
};

/** @implements {BaseUnit} */
App.SecExp.BaseRaiderUnit = class BaseRaiderUnit {
	static get attack() {
		return 7 + (V.SecExp.buildings.weapManu ? V.SecExp.buildings.weapManu.sellTo.raiders : 0);
	}

	static get defense() {
		return 2 + (V.SecExp.buildings.weapManu ? V.SecExp.buildings.weapManu.sellTo.raiders : 0);
	}

	static get morale() {
		return 100;
	}

	static get hp() {
		return 2;
	}
};

/** @implements {BaseUnit} */
App.SecExp.BaseFreeCityUnit = class BaseFreeCityUnit {
	static get attack() {
		return 6 + (V.SecExp.buildings.weapManu ? V.SecExp.buildings.weapManu.sellTo.FC : 0);
	}

	static get defense() {
		return 4 + (V.SecExp.buildings.weapManu ? V.SecExp.buildings.weapManu.sellTo.FC : 0);
	}

	static get morale() {
		return 130;
	}

	static get hp() {
		return 3;
	}
};

/** @implements {BaseUnit} */
App.SecExp.BaseOldWorldUnit = class BaseOldWorldUnit {
	static get attack() {
		return 8 + (V.SecExp.buildings.weapManu ? V.SecExp.buildings.weapManu.sellTo.oldWorld : 0);
	}

	static get defense() {
		return 4 + (V.SecExp.buildings.weapManu ? V.SecExp.buildings.weapManu.sellTo.oldWorld : 0);
	}

	static get morale() {
		return 110;
	}

	static get hp() {
		return 2;
	}
};

/** @implements {BaseUnit} */
App.SecExp.BaseFreedomFighterUnit = class BaseFreedomFighterUnit {
	static get attack() {
		return 9 + (V.SecExp.buildings.weapManu ? V.SecExp.buildings.weapManu.sellTo.oldWorld : 0);
	}

	static get defense() {
		return 2 + (V.SecExp.buildings.weapManu ? V.SecExp.buildings.weapManu.sellTo.oldWorld : 0);
	}

	static get morale() {
		return 160;
	}

	static get hp() {
		return 2;
	}
};

/** @implements {BaseUnit} */
App.SecExp.BaseSpecialForcesUnit = class BaseSpecialForcesUnit {
	static get attack() {
		return 8 + App.SecExp.getAppliedUpgrades('human').attack;
	}

	static get defense() {
		return 4 + App.SecExp.getAppliedUpgrades('human').defense;
	}

	static get morale() {
		return 140 + App.SecExp.getAppliedUpgrades('human').morale;
	}

	static get hp() {
		return 4 + App.SecExp.getAppliedUpgrades('human').hp;
	}
};

/** Unit base class */
App.SecExp.Unit = class SecExpUnit {
	/** @param {FC.SecExp.UnitData} data
	 * @param {BaseUnit} baseUnit
	 */
	constructor(data, baseUnit) {
		this._data = data;
		this._baseUnit = baseUnit;
	}

	/** @abstract
	 * @returns {number} */
	get attack() {
		throw "derive me";
	}

	/** @abstract
	 * @returns {number} */
	get defense() {
		throw "derive me";
	}

	/** @abstract
	 * @returns {number} */
	get morale() {
		return this._baseUnit.morale; // no morale modifiers
	}

	/** @abstract
	 * @returns {number} */
	get hp() {
		throw "derive me";
	}

	/** @abstract
	 * @returns {string} */
	describe() {
		throw "derive me";
	}

	/** @abstract
	 * @returns {string} */
	printStats() {
		throw "derive me";
	}
};

App.SecExp.DroneUnit = class SecExpDroneUnit extends App.SecExp.Unit {
	/** @param {FC.SecExp.PlayerUnitData} data
	 * @param {BaseUnit} baseUnit
	 */
	constructor(data, baseUnit) {
		super(data, baseUnit);
		this._data = data; // duplicate assignment, just for TypeScript
	}

	get attack() {
		const equipmentFactor = this._data.equip * App.SecExp.equipMod;
		return this._baseUnit.attack * (1 + equipmentFactor);
	}

	get defense() {
		const equipmentFactor = this._data.equip * App.SecExp.equipMod;
		return this._baseUnit.defense * (1 + equipmentFactor);
	}

	get hp() {
		return this._baseUnit.hp * this._data.troops;
	}

	describe() {
		return App.SecExp.describeUnit(this._data, "Bots");
	}

	printStats() {
		let r = [];
		r.push(`<br>Security drones base attack: ${this._baseUnit.attack} (After modifiers: ${Math.trunc(this.attack)})`);
		r.push(`<br>Security drones base defense: ${this._baseUnit.defense} (After modifiers: ${Math.trunc(this.defense)})`);
		r.push(`<br>Equipment bonus: +${this._data.equip * 15}%`);
		r.push(`<br>Security drones base hp: ${this._baseUnit.hp} (Total after modifiers for ${this._data.troops} drones: ${this.hp})`);
		r.push(`<br>Security drones base morale: ${this._baseUnit.morale}`);
		return r.join(` `);
	}
};

App.SecExp.HumanUnit = class SecExpHumanUnit extends App.SecExp.Unit {
	/** @param {FC.SecExp.PlayerHumanUnitData} data
	 * @param {BaseUnit} baseUnit
	 * @param {string} descriptionType
	 */
	constructor(data, baseUnit, descriptionType) {
		super(data, baseUnit);
		this._data = data; // duplicate assignment, just for TypeScript
		this._descType = descriptionType;
	}

	get attack() {
		const equipmentFactor = this._data.equip * App.SecExp.equipMod;
		const experienceFactor = App.SecExp.trainingValueToBonusFactor(this._data.training);
		const loyaltyFactor = App.SecExp.loyaltyValueToBonusFactor(this._data.loyalty);
		const SFFactor = 0.20 * this._data.SF;
		return this._baseUnit.attack * (1 + equipmentFactor + experienceFactor + loyaltyFactor + SFFactor) + this._data.cyber;
	}

	get defense() {
		const equipmentFactor = this._data.equip * App.SecExp.equipMod;
		const experienceFactor = App.SecExp.trainingValueToBonusFactor(this._data.training);
		const loyaltyFactor = App.SecExp.loyaltyValueToBonusFactor(this._data.loyalty);
		const SFFactor = 0.20 * this._data.SF;
		return this._baseUnit.defense * (1 + equipmentFactor + experienceFactor + loyaltyFactor + SFFactor) + this._data.cyber;
	}

	get hp() {
		const medicFactor = 0.25 * this._data.medics;
		const singleTroopHp = this._baseUnit.hp * (1 + medicFactor) + this._data.cyber;
		return singleTroopHp * this._data.troops;
	}

	describe() {
		return App.SecExp.describeUnit(this._data, this._descType);
	}

	printStats() {
		let r = [];
		r.push(`<br>${this._descType} base attack: ${this._baseUnit.attack} (After modifiers: ${Math.trunc(this.attack)})`);
		r.push(`<br>${this._descType} base defense: ${this._baseUnit.defense} (After modifiers: ${Math.trunc(this.defense)})`);
		if (this._data.equip > 0) {
			r.push(`<br>Equipment bonus: +${this._data.equip * 15}%`);
		}
		if (this._data.cyber > 0) {
			r.push(`<br>Cyber enhancements bonus: +1`);
		}
		if (this._data.training > 0) {
			r.push(`<br>Experience bonus: +${Math.trunc(App.SecExp.trainingValueToBonusFactor(this._data.training)*100)}%`);
		}
		if (this._data.loyalty > 0) {
			r.push(`<br>Loyalty bonus: +${Math.trunc(App.SecExp.loyaltyValueToBonusFactor(this._data.loyalty)*100)}%`);
		}
		if (this._data.SF > 0) {
			r.push(`<br>Special Force advisors bonus: +20%`);
		}
		r.push(`<br>${this._descType} base morale: ${this._baseUnit.morale} (After modifiers: ${this.morale})`);
		if (jsDef(V.SecExp.buildings.barracks) && V.SecExp.buildings.barracks.luxury > 0) {
			r.push(`<br>Barracks bonus: +${V.SecExp.buildings.barracks.luxury * 5}%`);
		}
		r.push(`<br>${this._descType} base hp: ${this._baseUnit.hp} (Total after modifiers for ${this._data.troops} troops: ${this.hp})`);
		if (this._data.medics > 0) {
			r.push(`<br>Medics detachment bonus: +25%`);
		}
		return r.join(` `);
	}
};

App.SecExp.troopsFromSF = function() {
	if (V.slaveRebellion === 0 && V.citizenRebellion === 0) { // attack: how many troops can we actually carry?
		const transportMax = Math.trunc(125 * (V.SF.Squad.GunS + (V.terrain !== "oceanic" ? ((V.SF.Squad.AV + V.SF.Squad.TV)/2) : 0)));
		return Math.min(transportMax, V.SF.ArmySize);
	} else {
		return V.SF.ArmySize; // rebellion: transport capabilities are irrelevant
	}
};

App.SecExp.SFUnit = class SFUnit extends App.SecExp.Unit {
	constructor() {
		super(null, App.SecExp.BaseSpecialForcesUnit);
		this._distancePenalty = (V.slaveRebellion === 0 && V.citizenRebellion === 0) ? 0.10 : 0.0;
	}

	get attack() {
		// ignores base attack? weird.
		const attackUpgrades = V.SF.Squad.Armoury + V.SF.Squad.Drugs + V.SF.Squad.AA + (V.terrain !== "oceanic" ? V.SF.Squad.AV : 0);
		return (0.75 - this._distancePenalty) * attackUpgrades;
	}

	get defense() {
		// ignores base defense? weird.
		const defenseUpgrades = V.SF.Squad.Armoury + V.SF.Squad.Drugs + (V.SF.Squad.AA + V.SF.Squad.TA) / 2 + (V.terrain !== "oceanic" ? (V.SF.Squad.AV + V.SF.Squad.TV) / 2 : 0);
		return (0.5 - this._distancePenalty) * defenseUpgrades;
	}

	get hp() {
		return this._baseUnit.hp * App.SecExp.troopsFromSF();
	}
};

App.SecExp.EnemyUnit = class SecExpEnemyUnit extends App.SecExp.Unit {
	/** @param {FC.SecExp.UnitData} data
	 * @param {BaseUnit} baseUnit
	 */
	constructor(data, baseUnit) {
		super(data, baseUnit);
	}

	get attack() {
		const equipmentFactor = this._data.equip * App.SecExp.equipMod;
		return this._baseUnit.attack * (1 + equipmentFactor);
	}

	get defense() {
		const equipmentFactor = this._data.equip * App.SecExp.equipMod;
		return this._baseUnit.defense * (1 + equipmentFactor);
	}

	get hp() {
		return this._baseUnit.hp * this._data.troops;
	}
};

App.SecExp.IrregularUnit = class SecExpEnemyUnit extends App.SecExp.Unit {
	/** @param {FC.SecExp.UnitData} data
	 * @param {BaseUnit} baseUnit
	 */
	constructor(data, baseUnit) {
		super(data, baseUnit);
	}

	get attack() {
		const equipmentFactor = this._data.equip * App.SecExp.equipMod;
		return (this._baseUnit.attack - App.SecExp.getAppliedUpgrades('human').attack) * (1 + equipmentFactor);
	}

	get defense() {
		const equipmentFactor = this._data.equip * App.SecExp.equipMod;
		return (this._baseUnit.defense - App.SecExp.getAppliedUpgrades('human').defense) * (1 + equipmentFactor);
	}

	get hp() {
		return (this._baseUnit.hp - App.SecExp.getAppliedUpgrades('human').hp) * this._data.troops;
	}
};

App.SecExp.describeUnit = (function() {
	return description;

	/**
	 * @param {FC.SecExp.PlayerUnitData} input
	 * @param {string} unitType
	 */
	function description(input, unitType = '') {
		let r = ``; const brief = V.SecExp.settings.unitDescriptions;

		if (unitType !== "Bots") {
			r += `<br><strong>${input.platoonName}</strong>${!brief ? ``:`. `} `;
			if (brief === 0) {
				if (input.battlesFought > 1) {
					r += `has participated in ${input.battlesFought} battles and is ready to face the enemy once more at your command. `;
				} else if (input.battlesFought === 1) {
					r += `is ready to face the enemy once more at your command. `;
				} else {
					r += `is ready to face the enemy in battle. `;
				}
				r += `<br>Its ${input.troops} men and women are `;

				if (unitType === "Militia") {
					r += `all proud citizens of your arcology, willing to put their lives on the line to protect their home. `;
				} else if (unitType === "Slaves") {
					r += `slaves in your possession, tasked with the protection of their owner and their arcology. `;
				} else if (unitType === "Mercs") {
					r += `mercenaries contracted to defend the arcology against external threats. `;
				}
			} else {
				r += `Battles fought: ${input.battlesFought}. `;
			}
		} else {
			if (brief === 0) {
				r += `<br>The drone unit is made up of ${input.troops} drones. All of which are assembled in an ordered formation in front of you, absolutely silent and ready to receive their orders.`;
			} else {
				r += `<br>Drone squad.`;
			}
		}

		if (brief === 0) {
			if (input.troops < input.maxTroops) {
				r += `The unit is not at its full strength of ${input.maxTroops} operatives. `;
			}
		} else {
			r += `Unit size: ${input.troops}/${input.maxTroops}.`;
		}

		if (brief === 0) {
			if (unitType !== "Bots") {
				if (input.equip === 0) {
					r += `They are issued with simple, yet effective equipment: firearms, a few explosives and standard uniforms, nothing more. `;
				} else if (input.equip === 1) {
					r += `They are issued with good, modern equipment: firearms, explosives and a few specialized weapons like sniper rifles and machine guns. They also carry simple body armor. `;
				} else if (input.equip === 2) {
					r += `They are issued with excellent, high tech equipment: modern firearms, explosives, specialized weaponry and modern body armor. They are also issued with modern instruments like night vision and portable radars. `;
				} else {
					r += `They are equipped with the best the modern world has to offer: modern firearms, explosives, specialized weaponry, experimental railguns, adaptive body armor and high tech recon equipment. `;
				}
			} else {
				if (input.equip === 0) {
					r += `They are equipped with light weaponry, mainly anti-riot nonlethal weapons. Not particularly effective in battle. `;
				} else if (input.equip === 1) {
					r += `They are equipped with light firearms, not an overwhelming amount of firepower, but with their mobility good enough to be effective. `;
				} else if (input.equip === 2) {
					r += `They are equipped with powerful, modern firearms and simple armor mounted around their frames. They do not make for a pretty sight, but on the battlefield they are a dangerous weapon. `;
				} else {
					r += `They are equipped with high energy railguns and adaptive armor. They are a formidable force on the battlefield, even for experienced soldiers. `;
				}
			}
		} else {
			r += `Equipment quality: `;
			if (input.equip === 0) {
				r += `basic. `;
			} else if (input.equip === 1) {
				r += `average. `;
			} else if (input.equip === 2) {
				r += `high. `;
			} else {
				r += `advanced. `;
			}
		}

		if (unitType !== "Bots") {
			if (brief === 0) {
				if (input.training <= 33) {
					r += `They lack the experience to be considered professionals, but `;
					if (unitType === "Militia") {
						r += `their eagerness to defend the arcology makes up for it. `;
					} else if (unitType === "Slaves") {
						r += `their eagerness to prove themselves makes up for it. `;
					} else if (unitType === "Mercs") {
						r += `they're trained more than enough to still be an effective unit. `;
					}
				} else if (input.training <= 66) {
					r += `They have trained `;
					if (input.battlesFought > 0) {
						r += `and fought `;
					}
					r += `enough to be considered disciplined, professional soldiers, ready to face the battlefield. `;
				} else {
					r += `They are consummate veterans, with a wealth of experience and perfectly trained. On the battlefield they are a well oiled war machine capable of facing pretty much anything. `;
				}

				if (input.loyalty < 10) {
					r += `The unit is extremely disloyal. Careful monitoring of their activities and relationships should be implemented. `;
				} else if (input.loyalty < 33) {
					r += `Their loyalty is low. Careful monitoring of their activities and relationships is advised. `;
				} else if (input.loyalty < 66) {
					r += `Their loyalty is not as high as it can be, but they are not actively working against their arcology owner. `;
				} else if (input.loyalty < 90) {
					r += `Their loyalty is high and strong. The likelihood of this unit betraying the arcology is low to non-existent. `;
				} else {
					r += `The unit is fanatically loyal. They would prefer death over betrayal. `;
				}

				if (input.cyber > 0) {
					r += `The soldiers of the unit have been enhanced with numerous cyberaugmentations which greatly increase their raw power. `;
				}
				if (input.medics > 0) {
					r += `The unit has a dedicated squad of medics that will follow them in battle. `;
				}
				if (V.SF.Toggle && V.SF.Active >= 1 && input.SF > 0) {
					r += `The unit has "advisors" from ${V.SF.Lower} that will help the squad remain tactically aware and active. `;
				}
			} else {
				r += `\nTraining: `;
				if (input.training <= 33) {
					r += `low. `;
				} else if(input.training <= 66) {
					r += `medium. `;
				} else {
					r += `high. `;
				}

				r += `Loyalty: `;
				if (input.loyalty < 10) {
					r += `extremely disloyal. `;
				} else if (input.loyalty < 33) {
					r += `low. `;
				} else if (input.loyalty < 66) {
					r += `medium. `;
				} else if (input.loyalty < 90) {
					r += `high. `;
				} else {
					r += `fanatical. `;
				}

				if (jsDef(input.cyber) && input.cyber > 0) {
					r += `\nCyberaugmentations applied. `;
				}
				if (jsDef(input.medics) && input.medics > 0) {
					r += `Medical squad attached. `;
				}
				if (V.SF.Toggle && V.SF.Active >= 1 && jsDef(input.SF) && input.SF > 0) {
					r += `${capFirstChar(V.SF.Lower || "the special force")} "advisors" attached. `;
				}
			}
		}

		if (!input.active) {
			r += `<br>This unit has lost too many operatives `;
			if (jsDef(input.battlesFought)) {
				r += `in the ${input.battlesFought} it fought `;
			}
			r += `and can no longer be considered a unit at all.`;
		}
		return r;
	}
})();

App.SecExp.mercenaryAvgLoyalty = function() {
	return _.mean(V.SecExp.units.mercs.squads.filter((u) => u.active === 1).map((u) => u.loyalty));
};

App.SecExp.Manpower = {
	get totalMilitia() {
		return this.employedMilitia + this.freeMilitia;
	},

	get employedMilitia() {
		return V.SecExp.units.militia.squads.reduce((acc, cur) => acc + cur.troops, 0);
	},

	get freeMilitia() {
		return V.SecExp.units.militia.free;
	},

	get employedSlave() {
		return V.SecExp.units.slaves.squads.reduce((acc, cur) => acc + cur.troops, 0);
	},

	get totalMerc() {
		return this.employedMerc + this.freeMerc;
	},

	get employedMerc() {
		return V.SecExp.units.mercs.squads.reduce((acc, cur) => acc + cur.troops, 0);
	},

	get freeMerc() {
		return V.SecExp.units.mercs.free;
	},

	get employedOverall() {
		return this.employedMerc + this.employedMilitia + this.employedSlave;
	}
};
