'use strict';

App.Art.Engine = class {
	getVsSourceBg() {
		return  `#version 300 es
                precision highp float;

                in vec2 vertexPosition;
                out vec2 vertexPos;
            
                void main() {
                    vertexPos = vertexPosition;
                    gl_Position = vec4(vertexPosition, 0.0, 1.0);
                }`;
	}

	getFsSourceBg() {
		return  `#version 300 es
                precision highp float;
                precision highp sampler2D;

                uniform sampler2D textSampler;
                uniform vec4 backgroundColor;

                in vec2 vertexPos;
                out vec4 outputColor;

                void main() {
                    vec2 textureCoord = vec2(vertexPos.s, -vertexPos.t) * 0.5 + 0.5;
                    vec3 c = backgroundColor.rgb * texture(textSampler, textureCoord.st).rgb;
                    outputColor  = vec4(c.rgb * backgroundColor.a, backgroundColor.a);
                }`;
	}

	getVsSource() {
		return  `#version 300 es
                precision highp float;

                uniform mat4 matView;
                uniform mat4 matProj;
                uniform mat4 matRot;
                uniform mat4 matTrans;
                uniform mat4 matScale;

                in vec3 vertexNormal;
                in vec3 vertexPosition;
                in vec2 textureCoordinate;
                in vec3 vertexTangent;

                in vec3 vertexNormalMorph;
                in vec3 vertexPositionMorph;

                out vec2 textureCoord;
                out vec3 normal;
                out vec4 pos;
                out mat3 TBN;

                void main() {
                    gl_Position = matProj * matView * matTrans * matScale * matRot * vec4(vertexPosition + vertexPositionMorph, 1.0);
                    normal = normalize((matRot * vec4(vertexNormal + vertexNormalMorph, 1.0)).xyz);

                    vec3 T = normalize(vec3(matTrans * matRot * vec4(vertexTangent, 0.0)));
                    vec3 N = normalize(vec3(matTrans * matRot * vec4(vertexNormal + vertexNormalMorph, 0.0)));
                    T = normalize(T - dot(T, N) * N);
                    vec3 B = cross(N, T);
                    TBN = mat3(T, B, N);

                    textureCoord = textureCoordinate;
                    pos = matTrans * matScale * matRot * vec4(vertexPosition + vertexPositionMorph, 1.0);
                }`;
	}

	getFsSource(dl, pl) {
		return `#version 300 es
                precision highp float;
                precision highp sampler2D;
                
                uniform float lightInt[${dl}];
                uniform float lightAmb[${dl}];
                uniform vec3 lightColor[${dl}];
                uniform vec3 lightVect[${dl}];

                uniform float pointLightInt[${pl}];
                uniform float pointLightAmb[${pl}];
                uniform vec3 pointLightColor[${pl}];
                uniform vec3 pointLightPos[${pl}];

                uniform float whiteM;
                uniform float gammaY;

                uniform vec3 Ka;
                uniform vec3 Kd;
                uniform vec3 Ks;
                uniform float d;
                uniform float Ns;

                uniform float sNormals;
                uniform float sAmbient;
                uniform float sDiffuse;
                uniform float sSpecular;
                uniform float sAlpha;
                uniform float sGamma;
                uniform float sReinhard;
                uniform float sNormal;
                
                uniform vec3 lookDir;

                uniform sampler2D textSampler[6];

                in vec2 textureCoord;
                in vec3 normal;
                in vec4 pos;
                in mat3 TBN;

                out vec4 outputColor;

                void main() {
                    vec3 new_normal = normal;
                    vec3 map_Ka = vec3(0.0,0.0,0.0);
                    vec3 map_Kd = vec3(0.0,0.0,0.0);
                    vec3 map_Ks = vec3(0.0,0.0,0.0);
                    float map_Ns = 0.0;
                    float map_d = 1.0;

                    if (sNormal == 1.0) {
                        vec3 map_Kn = texture(textSampler[5], textureCoord.st).rgb *2.0-1.0;
                        if (map_Kn != vec3(-1.0,-1.0,-1.0))
                            new_normal = normalize(TBN * map_Kn);
                    }

                    if (sAmbient == 1.0)
                        map_Ka = Ka * texture(textSampler[0], textureCoord.st).rgb;

                    if (sDiffuse == 1.0)
                        map_Kd = Kd * texture(textSampler[1], textureCoord.st).rgb;

                    if (sSpecular == 1.0) {
                        map_Ks = Ks * texture(textSampler[2], textureCoord.st).rgb;
                        map_Ns = Ns * texture(textSampler[3], textureCoord.st).r;
                    }

                    if (sAlpha == 1.0)
                        map_d = d * texture(textSampler[4], textureCoord.st).r;

                    vec3 Ld = vec3(0.0,0.0,0.0);
                    vec3 Ls = vec3(0.0,0.0,0.0);
                    vec3 La = vec3(0.0,0.0,0.0);
                    for (int i = 0; i < ${dl}; i++) {
                        float angle = max(dot(-lightVect[i], new_normal),0.0);
                        vec3 reflection = reflect(-lightVect[i], new_normal);
                        float specular = pow(max(dot(reflection, lookDir),0.0), (0.0001+map_Ns));

                        Ld += map_Kd * lightInt[i] * angle * lightColor[i];
                        Ls += map_Ks * lightInt[i] * specular * lightColor[i];
                        La += map_Ka * lightInt[i] * lightAmb[i] * lightColor[i];
                    }

                    for (int i = 0; i < ${pl}; i++) {
                        vec3 pointLightDir = normalize(pos.xyz - pointLightPos[i]);
                        float angle = max(dot(-pointLightDir, new_normal),0.0);
                        vec3 reflection = reflect(-pointLightDir, new_normal);
                        float specular = pow(max(dot(reflection, lookDir),0.0), (0.0001+map_Ns));

                        Ld += map_Kd * pointLightInt[i] * angle * pointLightColor[i];
                        Ls += map_Ks * pointLightInt[i] * specular * pointLightColor[i];
                        La += map_Ka * pointLightInt[i] * pointLightAmb[i] * pointLightColor[i];
                    }

                    vec3 vLighting = Ld + Ls + La;
                    vec3 c = map_Kd * vLighting;

                    if (sReinhard == 1.0) {
                        float l_old = 0.2126*c.r+0.7152*c.g+0.0722*c.b;
                        float numerator = l_old * (1.0 + (l_old / (whiteM*whiteM)));
                        float l_new = numerator / (1.0 + l_old);
                        c = c * (l_new / l_old);
                    }

                    if (sGamma == 1.0) {
                        c.r = pow(c.r, (1.0/gammaY));
                        c.g = pow(c.g, (1.0/gammaY));
                        c.b = pow(c.b, (1.0/gammaY));
                    }

                    if (sNormals == 1.0) {
                        c = new_normal;
                    }

                    outputColor = vec4(c*map_d, map_d);
                }`;
	}

	initBuffers(sceneData) {
		// init buffer containers
		this.buffers = new class {};
		this.buffers.models = [];
		for (let m=0; m < sceneData.models.length; m++) {
			this.buffers.models[m] = new class {};
		}

		// init background buffers
		this.buffers.backgroundPositionBuffer = this.gl.createBuffer();
		this.gl.bindBuffer(this.gl.ARRAY_BUFFER, this.buffers.backgroundPositionBuffer);
		this.gl.bufferData(this.gl.ARRAY_BUFFER, new Float32Array([-1, -1, 1, -1, 1, 1, -1, 1]), this.gl.STATIC_DRAW);

		this.buffers.backgroundIndexBuffer = this.gl.createBuffer();
		this.gl.bindBuffer(this.gl.ELEMENT_ARRAY_BUFFER, this.buffers.backgroundIndexBuffer);
		this.gl.bufferData(this.gl.ELEMENT_ARRAY_BUFFER, new Uint16Array([0, 1, 2, 0, 2, 3]), this.gl.STATIC_DRAW);

		// init model buffers
		for (let m=0; m < this.buffers.models.length; m++) {
			let modelBuffers = this.buffers.models[m];
			let modelData = sceneData.models[m];

			modelBuffers.verticesPositionBuffer = [];
			modelBuffers.verticesNormalBuffer = [];
			modelBuffers.verticesTextureCoordBuffer = [];
			modelBuffers.verticesTangentBuffer = [];
			modelBuffers.vertexCount = [];
			modelBuffers.verticesMorphBuffer = [];
			modelBuffers.verticesNormalMorphBuffer = [];

			modelBuffers.verticesIndexBuffer = [];
			modelBuffers.indexSizes = [];
			for (let i=0, count=0; i < modelData.figures.length; i++) {
				modelBuffers.verticesPositionBuffer[i] = this.gl.createBuffer();
				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesPositionBuffer[i]);
				this.gl.bufferData(this.gl.ARRAY_BUFFER, this.base64ToFloat(modelData.figures[i].verts), this.gl.STATIC_DRAW);
				modelBuffers.vertexCount[i] = this.gl.getBufferParameter(this.gl.ARRAY_BUFFER, this.gl.BUFFER_SIZE)/4;

				modelBuffers.verticesNormalBuffer[i] = this.gl.createBuffer();
				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesNormalBuffer[i]);
				this.gl.bufferData(this.gl.ARRAY_BUFFER, this.base64ToFloat(modelData.figures[i].vertsn), this.gl.STATIC_DRAW);

				modelBuffers.verticesTextureCoordBuffer[i] = this.gl.createBuffer();
				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesTextureCoordBuffer[i]);
				this.gl.bufferData(this.gl.ARRAY_BUFFER, this.base64ToFloat(modelData.figures[i].texts), this.gl.STATIC_DRAW);

				modelBuffers.verticesTangentBuffer[i] = this.gl.createBuffer();
				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesTangentBuffer[i]);
				this.gl.bufferData(this.gl.ARRAY_BUFFER, this.base64ToFloat(modelData.figures[i].tans), this.gl.STATIC_DRAW);

				// return dummy morph
				modelBuffers.verticesMorphBuffer[i] = this.gl.createBuffer();
				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesMorphBuffer[i]);
				this.gl.bufferData(this.gl.ARRAY_BUFFER, new Float32Array(0), this.gl.STATIC_DRAW);

				modelBuffers.verticesNormalMorphBuffer[i] = this.gl.createBuffer();
				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesNormalMorphBuffer[i]);
				this.gl.bufferData(this.gl.ARRAY_BUFFER, new Float32Array(0), this.gl.STATIC_DRAW);

				for (let j=0; j < modelData.figures[i].surfaces.length; j++, count++) {
					modelBuffers.verticesIndexBuffer[count] = this.gl.createBuffer();
					this.gl.bindBuffer(this.gl.ELEMENT_ARRAY_BUFFER, modelBuffers.verticesIndexBuffer[count]);
					let intArray = this.base64ToInt(modelData.figures[i].surfaces[j].vertsi);
					this.gl.bufferData(this.gl.ELEMENT_ARRAY_BUFFER, intArray, this.gl.STATIC_DRAW);
					modelBuffers.indexSizes[count] = intArray.length;
				}
			}

			this.initMorphs(modelBuffers, modelData);
		}
	}

	initMorphs(modelBuffers, modelData) {
		window.sceneBlocks = {}; // automatically populated during loading of morphs

		let promisedMorphs = [];
		modelBuffers.vertexPositionMorphs = [];
		modelBuffers.vertexNormalMorphs = [];
		modelBuffers.vertexIndexMorphs = [];
		for (let f=0; f < modelData.figures.length; f++) {
			modelBuffers.vertexPositionMorphs[f] = [];
			modelBuffers.vertexNormalMorphs[f] = [];
			modelBuffers.vertexIndexMorphs[f] = [];
			for (let m=0; m < modelData.morphCount; m++) {
				modelBuffers.vertexPositionMorphs[f].push(new Float32Array(0));
				modelBuffers.vertexNormalMorphs[f].push(new Float32Array(0));
				modelBuffers.vertexIndexMorphs[f].push(new Int32Array(0));
			}

			// stream real morphs
			promisedMorphs[f] = this.loadMorph(modelBuffers, f, modelData.figures[f].morphs);
		}

		Promise.all(promisedMorphs).then((values) => {
			if (values.length > 2) { // promise triggers twice (?)
				window.sceneBlocks = null; // let garbage collector clean
				if (App.Art.engineReady) { // re-send loaded event after morphs finish streaming
					modelBuffers.oldMorphValues = null;
					let containers = document.getElementsByClassName("artContainer");
					for (let i = 0; i < containers.length; i++) {
						containers[i].dispatchEvent(new Event("engineLoaded"));
					}
				}
			}
		});
	}

	loadMorph(modelBuffers, m, path) {
		let engine = this;
		return new Promise(function(resolve, reject) {
			let script = document.createElement("script");
			script.onload = function() {
				let morph = window.sceneBlocks[path];

				for (let i=0; i < morph.length; i+=3) {
					modelBuffers.vertexPositionMorphs[m][i/3] = engine.base64ToFloat(morph[i+0]);
					modelBuffers.vertexNormalMorphs[m][i/3] = engine.base64ToFloat(morph[i+1]);
					// reconstruct compressed indices
					modelBuffers.vertexIndexMorphs[m][i/3] = engine.base64ToInt(morph[i+2]).map((sum => value => sum += value)(0));
				}
				resolve();
			};
			script.onerror = function(e) {
				reject(e, script);
			};
			script.src = path;
			document.head.appendChild(script);
		});
	}

	initTextures(sceneData) {
		// load model textures
		this.textures = [];
		let promisedTextures = [];
		for (let i=0; i < sceneData.textures.length; i++) {
			const {texture, promise} = this.loadTexture(this.gl, sceneData.textures[i]);
			this.textures[i] = texture;
			promisedTextures[i] = promise;
		}
		Promise.all(promisedTextures).then(() => {
			if (App.Art.engineReady) { // re-send loaded event after textures finish streaming
				let containers = document.getElementsByClassName("artContainer");
				for (let i = 0; i < containers.length; i++) {
					containers[i].dispatchEvent(new Event("engineLoaded"));
				}
			}
		});
	}

	loadTexture(gl, url) {
		// return dummy texture right now
		let texture = gl.createTexture();
		gl.bindTexture(gl.TEXTURE_2D, texture);
		gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, 1, 1, 0, gl.RGBA, gl.UNSIGNED_BYTE, new Uint8Array([255, 255, 255, 255]));

		// promise that real textures will stream in the future
		let promise = fetch(url)
			.then(response => response.blob())
			.then(blob => createImageBitmap(blob))
			.then(bitmap => {
				gl.bindTexture(gl.TEXTURE_2D, texture);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
				gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, bitmap);

				return texture;
			});

		return {texture, promise};
	}

	initShaders(sceneParams) {
		// compile shaders
		let vertexShader = this.gl.createShader(this.gl.VERTEX_SHADER);
		this.gl.shaderSource(vertexShader, this.getVsSource());
		this.gl.compileShader(vertexShader);

		let fragmentShader = this.gl.createShader(this.gl.FRAGMENT_SHADER);
		this.gl.shaderSource(fragmentShader, this.getFsSource(sceneParams.directionalLights.length, sceneParams.pointLights.length));
		this.gl.compileShader(fragmentShader);

		this.shaderProgram = this.gl.createProgram();
		this.gl.attachShader(this.shaderProgram, vertexShader);
		this.gl.attachShader(this.shaderProgram, fragmentShader);
		this.gl.linkProgram(this.shaderProgram);

		let vertexShaderBg = this.gl.createShader(this.gl.VERTEX_SHADER);
		this.gl.shaderSource(vertexShaderBg, this.getVsSourceBg());
		this.gl.compileShader(vertexShaderBg);

		let fragmentShaderBg = this.gl.createShader(this.gl.FRAGMENT_SHADER);
		this.gl.shaderSource(fragmentShaderBg, this.getFsSourceBg());
		this.gl.compileShader(fragmentShaderBg);

		this.shaderProgramBg = this.gl.createProgram();
		this.gl.attachShader(this.shaderProgramBg, vertexShaderBg);
		this.gl.attachShader(this.shaderProgramBg, fragmentShaderBg);
		this.gl.linkProgram(this.shaderProgramBg);

		this.gl.useProgram(this.shaderProgram);

		// enable vertex attributes
		this.backgroundPositionAttribute = this.gl.getAttribLocation(this.shaderProgramBg, "vertexPosition");
		this.gl.enableVertexAttribArray(this.backgroundPositionAttribute);

		this.vertexPositionAttribute = this.gl.getAttribLocation(this.shaderProgram, "vertexPosition");
		this.gl.enableVertexAttribArray(this.vertexPositionAttribute);

		this.textureCoordAttribute = this.gl.getAttribLocation(this.shaderProgram, "textureCoordinate");
		this.gl.enableVertexAttribArray(this.textureCoordAttribute);

		this.vertexNormalAttribute = this.gl.getAttribLocation(this.shaderProgram, "vertexNormal");
		this.gl.enableVertexAttribArray(this.vertexNormalAttribute);

		this.vertexTangentAttribute = this.gl.getAttribLocation(this.shaderProgram, "vertexTangent");
		this.gl.enableVertexAttribArray(this.vertexTangentAttribute);

		this.vertexNormalMorphAttribute = this.gl.getAttribLocation(this.shaderProgram, "vertexNormalMorph");
		this.gl.enableVertexAttribArray(this.vertexNormalMorphAttribute);

		this.vertexPositionMorphAttribute = this.gl.getAttribLocation(this.shaderProgram, "vertexPositionMorph");
		this.gl.enableVertexAttribArray(this.vertexPositionMorphAttribute);
	}

	bind(sceneData, sceneParams) {
		this.offscreenCanvas = document.createElement("canvas");
		this.gl = this.offscreenCanvas.getContext("webgl2", {alpha:true, premultipliedAlpha: true});

		this.gl.enable(this.gl.CULL_FACE);
		this.gl.cullFace(this.gl.BACK);
		this.gl.enable(this.gl.DEPTH_TEST);
		this.gl.depthFunc(this.gl.LEQUAL);
		this.gl.enable(this.gl.BLEND);
		this.gl.blendEquation( this.gl.FUNC_ADD );
		this.gl.blendFunc(this.gl.ONE, this.gl.ONE_MINUS_SRC_ALPHA);

		this.initBuffers(sceneData);
		this.initTextures(sceneData);
		this.initShaders(sceneParams);
	}

	render(sceneParams, canvas) {
		// set render resolution
		this.offscreenCanvas.width = sceneParams.settings.rwidth;
		this.offscreenCanvas.height = sceneParams.settings.rheight;
		this.gl.viewport(0, 0, this.gl.drawingBufferWidth, this.gl.drawingBufferHeight);

		// draw background
		this.gl.clearColor(0, 0, 0, 0);
		this.gl.clear(this.gl.COLOR_BUFFER_BIT | this.gl.DEPTH_BUFFER_BIT);
		this.gl.useProgram(this.shaderProgramBg);
		if (sceneParams.background.visible) {
			this.drawBackground(sceneParams);
		}

		// draw scene
		this.gl.clear(this.gl.DEPTH_BUFFER_BIT);
		this.gl.useProgram(this.shaderProgram);
		this.drawScene(sceneParams);

		// clone from offscreen to real canvas
		let ctx = canvas.getContext('2d', {alpha:true});
		ctx.clearRect(0, 0, canvas.width, canvas.height);
		ctx.drawImage(this.gl.canvas, 0, 0, canvas.width, canvas.height);
	}

	drawBackground(sceneParams) {
		this.gl.uniform1i(this.gl.getUniformLocation(this.shaderProgramBg, "textSampler"), 0);
		this.gl.uniform4fv(this.gl.getUniformLocation(this.shaderProgramBg, "backgroundColor"), sceneParams.background.color);

		this.gl.activeTexture(this.gl.TEXTURE0);
		this.gl.bindTexture(this.gl.TEXTURE_2D, this.textures[sceneParams.background.filename]);

		this.gl.bindBuffer(this.gl.ARRAY_BUFFER, this.buffers.backgroundPositionBuffer);
		this.gl.vertexAttribPointer(this.backgroundPositionAttribute, 2, this.gl.FLOAT, false, 0, 0);

		this.gl.bindBuffer(this.gl.ELEMENT_ARRAY_BUFFER, this.buffers.backgroundIndexBuffer);
		this.gl.drawElements(this.gl.TRIANGLES, 6, this.gl.UNSIGNED_SHORT, 0);
	}

	drawScene(sceneParams) {
		// create camera
		let camRotX = this.degreeToRad(-sceneParams.camera.xr);
		let camRotY = this.degreeToRad(-sceneParams.camera.yr);
		let camRotZ = this.degreeToRad(sceneParams.camera.zr);

		let up = [Math.sin(camRotZ), Math.cos(camRotZ), Math.sin(camRotZ)];
		let camera = [sceneParams.camera.x, sceneParams.camera.y, sceneParams.camera.z];

		let matCameraRot = this.matrixMulMatrix(this.matrixMakeRotationX(camRotX), this.matrixMakeRotationY(camRotY));
		let lookDir = this.matrixMulVector(matCameraRot, [0, 0, 1]);
		let target = this.vectorAdd(lookDir, camera);
		let matCamera = this.matrixPointAt(camera, target, up);

		// create scene transforms
		let matProj = this.matrixMakeProjection(sceneParams.camera.fov, sceneParams.settings.rheight/sceneParams.settings.rwidth, sceneParams.camera.fnear, sceneParams.camera.ffar);
		let matView = this.matrixInverse(matCamera);

		// set scene uniforms
		this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "sNormals"), sceneParams.settings.normals);
		this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "sAmbient"), sceneParams.settings.ambient);
		this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "sDiffuse"), sceneParams.settings.diffuse);
		this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "sSpecular"), sceneParams.settings.specular);
		this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "sNormal"), sceneParams.settings.normal);
		this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "sAlpha"), sceneParams.settings.alpha);
		this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "sReinhard"), sceneParams.settings.reinhard);
		this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "sGamma"), sceneParams.settings.gamma);
		this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "whiteM"), sceneParams.settings.whiteM);
		this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "gammaY"), sceneParams.settings.gammaY);

		for (let i = 0; i < sceneParams.directionalLights.length; i++) {
			let lightVect = this.polarToCart(this.degreeToRad(sceneParams.directionalLights[i].yr), this.degreeToRad(sceneParams.directionalLights[i].xr));
			this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "lightAmb[" + i + "]"), sceneParams.directionalLights[i].ambient);
			this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "lightInt[" + i + "]"), sceneParams.directionalLights[i].intensity);
			this.gl.uniform3fv(this.gl.getUniformLocation(this.shaderProgram, "lightColor[" + i + "]"), sceneParams.directionalLights[i].color);
			this.gl.uniform3fv(this.gl.getUniformLocation(this.shaderProgram, "lightVect[" + i + "]"), lightVect);
		}

		for (let i = 0; i < sceneParams.pointLights.length; i++) {
			this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "pointLightAmb[" + i + "]"), sceneParams.pointLights[i].ambient);
			this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "pointLightInt[" + i + "]"), sceneParams.pointLights[i].intensity);
			this.gl.uniform3fv(this.gl.getUniformLocation(this.shaderProgram, "pointLightColor[" + i + "]"), sceneParams.pointLights[i].color);
			this.gl.uniform3fv(this.gl.getUniformLocation(this.shaderProgram, "pointLightPos[" + i + "]"), [sceneParams.pointLights[i].x, sceneParams.pointLights[i].y, sceneParams.pointLights[i].z]);
		}

		this.gl.uniform3fv(this.gl.getUniformLocation(this.shaderProgram, "lookDir"), lookDir);

		this.gl.uniformMatrix4fv(this.gl.getUniformLocation(this.shaderProgram, "matProj"), false, new Float32Array(this.matrixFlatten(matProj)));
		this.gl.uniformMatrix4fv(this.gl.getUniformLocation(this.shaderProgram, "matView"), false, new Float32Array(this.matrixFlatten(matView)));

		// process each model in the scene
		for (let m=0; m < this.buffers.models.length; m++) {
			let modelBuffers = this.buffers.models[m];
			let modelParams = sceneParams.models[m];

			if(!modelParams.visible) {
				continue;
			}

			// create model transforms
			this.applyMorphs(modelParams, modelBuffers);

			let matRot = this.matrixMakeRotation(this.degreeToRad(modelParams.transform.xr), this.degreeToRad(modelParams.transform.yr), this.degreeToRad(modelParams.transform.zr));
			let matTrans = this.matrixMakeTranslation(modelParams.transform.x, modelParams.transform.y, modelParams.transform.z);
			let matScale = this.matrixMakeScaling( modelParams.transform.scale);

			// set model uniforms
			this.gl.uniformMatrix4fv(this.gl.getUniformLocation(this.shaderProgram, "matTrans"), false, new Float32Array(this.matrixFlatten(matTrans)));
			this.gl.uniformMatrix4fv(this.gl.getUniformLocation(this.shaderProgram, "matScale"), false, new Float32Array(this.matrixFlatten(matScale)));
			this.gl.uniformMatrix4fv(this.gl.getUniformLocation(this.shaderProgram, "matRot"), false, new Float32Array(this.matrixFlatten(matRot)));

			for (let i=0, count=0; i < modelParams.figures.length; i++) {
				if(!modelParams.figures[i].visible) {
					count += modelParams.figures[i].surfaces.length;
					continue;
				}

				// bind vertex buffers per figure
				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesPositionBuffer[i]);
				this.gl.vertexAttribPointer(this.vertexPositionAttribute, 3, this.gl.FLOAT, false, 0, 0);

				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesTextureCoordBuffer[i]);
				this.gl.vertexAttribPointer(this.textureCoordAttribute, 2, this.gl.FLOAT, false, 0, 0);

				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesNormalBuffer[i]);
				this.gl.vertexAttribPointer(this.vertexNormalAttribute, 3, this.gl.FLOAT, false, 0, 0);

				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesTangentBuffer[i]);
				this.gl.vertexAttribPointer(this.vertexTangentAttribute, 3, this.gl.FLOAT, false, 0, 0);

				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesMorphBuffer[i]);
				this.gl.vertexAttribPointer(this.vertexPositionMorphAttribute, 3, this.gl.FLOAT, false, 0, 0);

				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesNormalMorphBuffer[i]);
				this.gl.vertexAttribPointer(this.vertexNormalMorphAttribute, 3, this.gl.FLOAT, false, 0, 0);

				// bind materials per surface and set uniforms
				for (let j=0; j < modelParams.figures[i].surfaces.length; j++, count++) {
					let visible = modelParams.figures[i].surfaces[j].visible;

					for (let h=0; h < modelParams.figures[i].surfaces[j].matIds.length; h++) {
						let matId = modelParams.figures[i].surfaces[j].matIds[h];
						let matIdx = sceneParams.materials.map(e => e.matId).indexOf(matId);
						if (matIdx === -1) {
							continue;
						}
						let mat = sceneParams.materials[matIdx];

						if (mat.d > 0 && visible) {
							this.gl.activeTexture(this.gl.TEXTURE0);
							this.gl.bindTexture(this.gl.TEXTURE_2D, this.textures[parseInt(mat.map_Ka)]);
							this.gl.uniform1i(this.gl.getUniformLocation(this.shaderProgram, "textSampler[0]"), 0);

							this.gl.activeTexture(this.gl.TEXTURE1);
							this.gl.bindTexture(this.gl.TEXTURE_2D, this.textures[parseInt(mat.map_Kd)]);
							this.gl.uniform1i(this.gl.getUniformLocation(this.shaderProgram, "textSampler[1]"), 1);

							this.gl.activeTexture(this.gl.TEXTURE2);
							this.gl.bindTexture(this.gl.TEXTURE_2D, this.textures[parseInt(mat.map_Ks)]);
							this.gl.uniform1i(this.gl.getUniformLocation(this.shaderProgram, "textSampler[2]"), 2);

							this.gl.activeTexture(this.gl.TEXTURE3);
							this.gl.bindTexture(this.gl.TEXTURE_2D, this.textures[parseInt(mat.map_Ns)]);
							this.gl.uniform1i(this.gl.getUniformLocation(this.shaderProgram, "textSampler[3]"), 3);

							this.gl.activeTexture(this.gl.TEXTURE4);
							this.gl.bindTexture(this.gl.TEXTURE_2D, this.textures[parseInt(mat.map_D)]);
							this.gl.uniform1i(this.gl.getUniformLocation(this.shaderProgram, "textSampler[4]"), 4);

							this.gl.activeTexture(this.gl.TEXTURE5);
							this.gl.bindTexture(this.gl.TEXTURE_2D, this.textures[parseInt(mat.map_Kn)]);
							this.gl.uniform1i(this.gl.getUniformLocation(this.shaderProgram, "textSampler[5]"), 5);

							this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "d"), mat.d);
							this.gl.uniform3fv(this.gl.getUniformLocation(this.shaderProgram, "Ka"), mat.Ka);
							this.gl.uniform3fv(this.gl.getUniformLocation(this.shaderProgram, "Kd"), mat.Kd);
							this.gl.uniform3fv(this.gl.getUniformLocation(this.shaderProgram, "Ks"), mat.Ks);
							this.gl.uniform1f(this.gl.getUniformLocation(this.shaderProgram, "Ns"), mat.Ns);

							// draw materials
							this.gl.bindBuffer(this.gl.ELEMENT_ARRAY_BUFFER, modelBuffers.verticesIndexBuffer[count]);
							this.gl.drawElements(this.gl.TRIANGLES, modelBuffers.indexSizes[count], this.gl.UNSIGNED_INT, 0);
						}
					}
				}
			}
		}
	}

	applyMorphs(modelParams, modelBuffers) {
		if(modelBuffers.oldMorphValues !== JSON.stringify(modelParams.morphs) + JSON.stringify(modelParams.figures)) {
			for (let f=0; f < modelParams.figures.length; f++) {
				if(!modelParams.visible || !modelParams.figures[f].visible) {
					continue;
				}

				let vertexPositionMorph = new Float32Array(modelBuffers.vertexCount[f]);
				let vertexNormalMorph = new Float32Array(modelBuffers.vertexCount[f]);

				for(let m=0; m < modelParams.morphs.length; m++) {
					let morphValue = modelParams.morphs[m].value;

					if (morphValue !== 0) {
						let vp = modelBuffers.vertexPositionMorphs[f][m];
						let vn = modelBuffers.vertexNormalMorphs[f][m];
						let vi = modelBuffers.vertexIndexMorphs[f][m];

						if (morphValue === 1) {
							for(let j = 0; j < vi.length; j++) {
								vertexPositionMorph[vi[j]] += vp[j];
								vertexNormalMorph[vi[j]] += vn[j];
							}
						} else {
							for(let j=0; j < vi.length; j++) {
								vertexPositionMorph[vi[j]] += vp[j] * morphValue;
								vertexNormalMorph[vi[j]] += vn[j] * morphValue;
							}
						}
					}
				}

				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesMorphBuffer[f]);
				this.gl.bufferData(this.gl.ARRAY_BUFFER, vertexPositionMorph, this.gl.STATIC_DRAW);

				this.gl.bindBuffer(this.gl.ARRAY_BUFFER, modelBuffers.verticesNormalMorphBuffer[f]);
				this.gl.bufferData(this.gl.ARRAY_BUFFER, vertexNormalMorph, this.gl.STATIC_DRAW);
			}

			modelBuffers.oldMorphValues = JSON.stringify(modelParams.morphs) + JSON.stringify(modelParams.figures);
		}
	}

	base64ToFloat(array) {
		let b = window.atob(array);
		let fLen = b.length / (Float32Array.BYTES_PER_ELEMENT-1);
		let dView = new DataView(new ArrayBuffer(Float32Array.BYTES_PER_ELEMENT));
		let fAry = new Float32Array(fLen);
		let p = 0;

		for(let j=0; j < fLen; j++){
			p = j * 3;
			dView.setUint8(0, 0); // skip 1 precision byte
			dView.setUint8(1, b.charCodeAt(p+0));
			dView.setUint8(2, b.charCodeAt(p+1));
			dView.setUint8(3, b.charCodeAt(p+2));
			fAry[j] = dView.getFloat32(0, true);
		}
		return fAry;
	}

	base64ToInt(array) {
		let b = window.atob(array);
		let fLen = b.length / Int32Array.BYTES_PER_ELEMENT;
		let dView = new DataView(new ArrayBuffer(Int32Array.BYTES_PER_ELEMENT));
		let fAry = new Int32Array(fLen);
		let p = 0;

		for(let j=0; j < fLen; j++){
			p = j * 4;
			dView.setUint8(0, b.charCodeAt(p));
			dView.setUint8(1, b.charCodeAt(p+1));
			dView.setUint8(2, b.charCodeAt(p+2));
			dView.setUint8(3, b.charCodeAt(p+3));
			fAry[j] = dView.getInt32(0, true);
		}
		return fAry;
	}

	base64ToByte(array) {
		let b = window.atob(array);
		let fLen = b.length / Uint8Array.BYTES_PER_ELEMENT;
		let dView = new DataView(new ArrayBuffer(Uint8Array.BYTES_PER_ELEMENT));
		let fAry = new Uint8Array(fLen);

		for(let j=0; j < fLen; j++){
			dView.setUint8(0, b.charCodeAt(j));
			fAry[j] = dView.getUint8(0);
		}
		return fAry;
	}

	degreeToRad(d) { return d * (Math.PI / 180); }

	polarToCart(y, p) { return [Math.sin(p) * Math.cos(y), Math.sin(p) * Math.sin(y), Math.cos(p)]; }

	vectorAdd(v1, v2) { return [v1[0] + v2[0], v1[1] + v2[1], v1[2] + v2[2]]; }

	vectorSub(v1, v2) { return [v1[0] - v2[0], v1[1] - v2[1], v1[2] - v2[2]]; }

	vectorMul(v, k) { return [v[0] * k, v[1] * k, v[2] * k]; }

	vectorDotProduct(v1, v2) { return [v1[0] * v2[0] + v1[1] * v2[1] + v1[2] * v2[2]]; }

	vectorCrossProduct(v1, v2) { return [v1[1] * v2[2] - v1[2] * v2[1], v1[2] * v2[0] - v1[0] * v2[2], v1[0] * v2[1] - v1[1] * v2[0]]; }

	vectorNormalize(v) {
		let l = Math.sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
		return [v[0]/l, v[1]/l, v[2]/l];
	}

	matrixMakeProjection(fov, aspect, near, far) {
		return [[aspect * (1/Math.tan(fov*0.5/180*Math.PI)), 0, 0, 0],
			    [0, (1/Math.tan(fov*0.5/180*Math.PI)), 0, 0],
			    [0, 0, far/(far-near), 1],
			    [0, 0, (-far*near)/(far-near), 0]];
	}

	matrixPointAt(pos, target, up) {
		let newForward = this.vectorNormalize(this.vectorSub(target, pos));
		let a = this.vectorMul(newForward, this.vectorDotProduct(up, newForward));
		let newUp = this.vectorNormalize(this.vectorSub(up, a));
		let newRight = this.vectorCrossProduct(newUp, newForward);

		return [[newRight[0], newRight[1], newRight[2], 0],
			    [newUp[0], newUp[1], newUp[2], 0],
			    [newForward[0], newForward[1], newForward[2], 0],
			    [pos[0], pos[1], pos[2], 1]];
	}

	matrixMakeRotation(xr, yr, zr) {
		let cosA = Math.cos(xr);
		let cosB = Math.cos(yr);
		let cosC = Math.cos(zr);
		let sinA = Math.sin(xr);
		let sinB = Math.sin(yr);
		let sinC = Math.sin(zr);

		return([[cosB*cosC, -cosB*sinC, sinB, 0],
			[sinA*sinB*cosC+cosA*sinC, -sinA*sinB*sinC+cosA*cosC, -sinA*cosB, 0],
			[-cosA*sinB*cosC+sinA*sinC, cosA*sinB*sinC+sinA*cosC, cosA*cosB, 0],
			[0, 0, 0, 1]]);
	}

	matrixMakeRotationX(r) {
		return [[1, 0, 0],
			[0, Math.cos(r), Math.sin(r)],
			[0, -Math.sin(r), Math.cos(r)]];
	}

	matrixMakeRotationY(r) {
		return [[Math.cos(r), 0, Math.sin(r)],
			    [0, 1, 0],
			    [-Math.sin(r), 0, Math.cos(r)]];
	}

	matrixMakeRotationZ(r) {
		return [[Math.cos(r), Math.sin(r), 0],
			    [-Math.sin(r), Math.cos(r), 0],
			    [0, 0, 1]];
	}

	matrixMakeTranslation(x, y, z) {
		return [[1, 0, 0, 0],
			    [0, 1, 0, 0],
			    [0, 0, 1, 0],
			    [x, y, z, 1]];
	}

	matrixMakeScaling(s) {
		return [[s, 0, 0, 0],
			    [0, s, 0, 0],
			    [0, 0, s, 0],
			    [0, 0, 0, 1]];
	}

	matrixMulMatrix(m1, m2) {
		return [[m1[0][0] * m2[0][0] + m1[0][1] * m2[1][0] + m1[0][2] * m2[2][0],
			    m1[0][0] * m2[0][1] + m1[0][1] * m2[1][1] + m1[0][2] * m2[2][1],
			    m1[0][0] * m2[0][2] + m1[0][1] * m2[1][2] + m1[0][2] * m2[2][2]],
		        [m1[1][0] * m2[0][0] + m1[1][1] * m2[1][0] + m1[1][2] * m2[2][0],
			    m1[1][0] * m2[0][1] + m1[1][1] * m2[1][1] + m1[1][2] * m2[2][1],
			    m1[1][0] * m2[0][2] + m1[1][1] * m2[1][2] + m1[1][2] * m2[2][2]],
		        [m1[2][0] * m2[0][0] + m1[2][1] * m2[1][0] + m1[2][2] * m2[2][0],
			    m1[2][0] * m2[0][1] + m1[2][1] * m2[1][1] + m1[2][2] * m2[2][1],
			    m1[2][0] * m2[0][2] + m1[2][1] * m2[1][2] + m1[2][2] * m2[2][2]]];
	}

	matrixMulVector(m, v) {
		return [v[0] * m[0][0] + v[1] * m[1][0] + v[2] * m[2][0],
			    v[0] * m[0][1] + v[1] * m[1][1] + v[2] * m[2][1],
			    v[0] * m[0][2] + v[1] * m[1][2] + v[2] * m[2][2]];
	}

	matrixInverse(m) {
		return [[m[0][0], m[1][0], m[2][0], 0],
			    [m[0][1], m[1][1], m[2][1], 0],
			    [m[0][2], m[1][2], m[2][2], 0],
			    [-(m[3][0]*m[0][0]+m[3][1]*m[0][1]+m[3][2]*m[0][2]), -(m[3][0]*m[1][0]+m[3][1]*m[1][1]+m[3][2]*m[1][2]), -(m[3][0]*m[2][0]+m[3][1]*m[2][1]+m[3][2]*m[2][2]), 1]];
	}

	matrixFlatten(m) {
		return [m[0][0], m[0][1], m[0][2], m[0][3],
			    m[1][0], m[1][1], m[1][2], m[1][3],
			    m[2][0], m[2][1], m[2][2], m[2][3],
			    m[3][0], m[3][1], m[3][2], m[3][3]];
	}
};

