App.Art.isDraggingCanvas = false;

App.Art.createWebglUI = function(container, slave, artSize, scene) {
	let lockViewDisabled = "resources/webgl/ui/lockViewDisabled.png";
	let lockViewEnabled = "resources/webgl/ui/lockViewEnabled.png";
	let faceViewDisabled = "resources/webgl/ui/faceViewDisabled.png";
	let faceViewEnabled = "resources/webgl/ui/faceViewEnabled.png";
	let resetViewDisabled = "resources/webgl/ui/resetViewDisabled.png";
	let resetViewEnabled = "resources/webgl/ui/resetViewEnabled.png";
	let inspectViewDisabled = "resources/webgl/ui/inspectViewDisabled.png";
	let inspectViewEnabled = "resources/webgl/ui/inspectViewEnabled.png";

	let uicontainer = document.createElement("div");
	uicontainer.setAttribute("style", "left: 82.5%; top: 5%; position: absolute; width: 15%; border: 0px; padding: 0px;");

	// canvas
	let cvs = document.createElement("canvas");
	cvs.setAttribute("style", "position: absolute;");

	// btnLockView
	let btnLockView = document.createElement("input");
	btnLockView.setAttribute("style", "display: flex; width: 100%; position: relative; border: 0px; padding: 0px; background-color: transparent;");
	btnLockView.setAttribute("type", "image");
	btnLockView.setAttribute("src", scene.lockView ? lockViewDisabled : lockViewEnabled);

	// btnFaceView
	let btnFaceView = document.createElement("input");
	btnFaceView.setAttribute("style", "display: flex; width: 100%; position: relative; border: 0px; padding: 0px; background-color: transparent;");
	btnFaceView.setAttribute("type", "image");
	btnFaceView.setAttribute("src", scene.faceView ? faceViewDisabled : faceViewEnabled);

	// btnResetView
	let btnResetView = document.createElement("input");
	btnResetView.setAttribute("style", "display: flex; width: 100%; position: relative; border: 0px; padding: 0px; background-color: transparent;");
	btnResetView.setAttribute("type", "image");
	btnResetView.setAttribute("src", scene.resetView ? resetViewEnabled : resetViewDisabled);

	// btnInspectView
	let btnInspectView = document.createElement("input");
	btnInspectView.setAttribute("style", "display: flex; width: 100%; position: relative; border: 0px; padding: 0px; background-color: transparent;");
	btnInspectView.setAttribute("type", "image");
	btnInspectView.setAttribute("src", scene.inspectView ? inspectViewDisabled : inspectViewEnabled);

	// events
	btnInspectView.onclick = function(e){
		scene.inspectView = true;
		scene.resetView = true;
		scene.faceView = false;
		btnFaceView.src = faceViewEnabled;
		btnResetView.src = resetViewEnabled;
		btnInspectView.src = inspectViewDisabled;

		scene.models[0].transform.yr = 180;
		App.Art.applyMorphs(slave, scene);
		App.Art.Frame(slave, scene);
		App.Art.engine.render(scene, cvs);
	};

	btnLockView.onclick = function(e){
		scene.lockView = !scene.lockView;
		btnLockView.src = scene.lockView ? lockViewDisabled : lockViewEnabled;
	};

	btnFaceView.onclick = function(e){
		scene.resetView = true;
		scene.inspectView = false;
		scene.faceView = true;
		btnFaceView.src = faceViewDisabled;
		btnResetView.src = resetViewEnabled;
		btnInspectView.src = inspectViewEnabled;

		scene.camera.y = slave.height-5;
		scene.models[0].transform.yr = 0;
		scene.camera.xr = -6;
		scene.camera.z = -slave.height/3.85;
		App.Art.applyMorphs(slave, scene);
		App.Art.engine.render(scene, cvs);
	};

	btnResetView.onclick = function(e){
		scene.resetView = false;
		scene.faceView = false;
		scene.inspectView = false;
		btnResetView.src = resetViewDisabled;
		btnFaceView.src = faceViewEnabled;
		btnInspectView.src = inspectViewEnabled;

		scene.models[0].transform.yr = App.Art.defaultScene.models[0].transform.yr;
		App.Art.applyMorphs(slave, scene);
		App.Art.Frame(slave, scene);
		App.Art.engine.render(scene, cvs);
	};

	cvs.onmousemove = function(e){
		if(scene.lockView){ return; }
		if(!App.Art.isDraggingCanvas){ return; }
		e.preventDefault();
		e.stopPropagation();

		scene.resetView = true;
		scene.faceView = false;
		btnResetView.src = resetViewEnabled;
		btnFaceView.src = faceViewEnabled;

		scene.camera.y = scene.camera.y + e.movementY/7;
		scene.models[0].transform.yr = scene.models[0].transform.yr + e.movementX*5;
		App.Art.engine.render(scene, cvs);
	};

	cvs.onmousedown = function(e){
		if(scene.lockView){ return; }
		e.preventDefault();
		e.stopPropagation();
		App.Art.isDraggingCanvas=true;
	};

	cvs.onmouseup = function(e){
		if(scene.lockView){ return; }
		if(!App.Art.isDraggingCanvas){ return; }
		e.preventDefault();
		e.stopPropagation();
		App.Art.isDraggingCanvas=false;
	};

	cvs.onmouseout = function(e){
		if(scene.lockView){ return; }
		if(!App.Art.isDraggingCanvas){ return; }
		e.preventDefault();
		e.stopPropagation();
		App.Art.isDraggingCanvas=false;
	};

	cvs.onwheel = function(e){
		if(scene.lockView){ return; }

		scene.resetView = true;
		scene.faceView = false;
		btnResetView.src = resetViewEnabled;
		btnFaceView.src = faceViewEnabled;

		// zoom speed based on distance from origin, and along direction of camera
		let zOld = scene.camera.z;
		let magnitude = e.deltaY/(10/V.setZoomSpeed) * (-scene.camera.z/50 + 0.2);
		let zDistance = Math.cos(-scene.camera.xr * (Math.PI/180)) * magnitude;
		scene.camera.z -= zDistance;
		scene.camera.z = Math.clamp(scene.camera.z, -900, -10);
		scene.camera.y += Math.sin(-scene.camera.xr * (Math.PI/180)) * magnitude * -(scene.camera.z - zOld)/zDistance;

		App.Art.engine.render(scene, cvs);
		return false;
	};

	container.appendChild(cvs);
	uicontainer.appendChild(btnLockView);
	uicontainer.appendChild(btnFaceView);
	uicontainer.appendChild(btnInspectView);
	uicontainer.appendChild(btnResetView);
	container.appendChild(uicontainer);

	// calculate canvas resolution
	if (artSize) {
		let sz;
		switch (artSize) {
			case 3:
				sz = [300, 530];
				break;
			case 2:
				sz = [300, 300];
				break;
			case 1:
				sz = [150, 150];
				break;
			default:
				sz = [120, 120];
				break;
		}

		cvs.width = sz[0];
		cvs.height = sz[1];
		container.setAttribute("style", "position: relative; width: " + sz[0] + "px; height: " + sz[1] + "px;");
	}

	if(typeof V.setSuperSampling === "undefined") {
		V.setSuperSampling = 2;
	}

	scene.settings.rwidth = cvs.width * V.setSuperSampling;
	scene.settings.rheight = cvs.height * V.setSuperSampling;

	// render based on active view
	if (scene.faceView) {
		btnFaceView.click();
	} else if (scene.inspectView) {
		btnInspectView.click();
	} else {
		btnResetView.click();
	}
};

App.Art.Frame = function(slave, scene) {
	if (slave.height > 185) {
		App.Art.AutoFrame(scene, slave.height, 127);
	} else {
		App.Art.FixedFrame(scene);
	}
};

App.Art.AutoFrame = function(scene, slaveHeight, cameraHeight) {
	// auto-frame based on camera height and FoV
	let n = Math.max(slaveHeight * 1.05 - cameraHeight, 1);
	let m = cameraHeight * 1.065;
	let fov = scene.camera.fov;

	let a = fov * (Math.PI/180);
	let r = m/n;
	let h = 0;

	// solve for distance
	if (a !== Math.PI/2) {
		if (a > Math.PI/2) {
			h = n/((-(r + 1) - ((r+1)**2 + 4*r*Math.tan(a)**2)**(1/2))/(2*Math.tan(a)*r)); // take negative discriminant
		} else {
			h = n/((-(r + 1) +  ((r+1)**2 + 4*r*Math.tan(a)**2)**(1/2))/(2*Math.tan(a)*r)); // take positive discriminant
		}
	} else {
		h = (m+n)/2 * Math.sin(Math.acos(((m+n)/2-n)/((m+n)/2))); // edge case
	}

	// solve for rotation
	let rot = fov/2 - Math.atan(n/h) * (180/Math.PI);

	scene.camera.z = -h;
	scene.camera.y = cameraHeight;
	scene.camera.xr = -rot;
};

App.Art.FixedFrame = function(scene) {
	scene.camera.z = -275;
	scene.camera.y = 127;
	scene.camera.xr = -6;
};
