// Scheduled/nonrandom events are run sequentially, in order.
// They should always return to Nonrandom Event to ensure that none are skipped.

/** get a list of possible scheduled events
 * @returns {Array<App.Events.BaseEvent>}
 */
App.Events.getNonrandomEvents = function() {
	return [
		// instantiate all possible scheduled/nonrandom events here
		// ORDER MATTERS - if multiple events from this list trigger in a single week, they are executed in this order

		new App.Events.TwineEvent().wrapPassage([
			() => V.rivalOwner === -1
		], "P rival initiation"),
		new App.Events.TwineEvent().wrapPassage([
			() => V.secExpEnabled > 0,
			() => V.foughtThisWeek === 0,
			() => (V.slaveRebellion === 1 || V.citizenRebellion === 1)
		], "rebellionOptions"),
		new App.Events.attackOptions(),
		new App.Events.TwineEvent().wrapPassage([
			() => V.PC.labor === 1
		], "SE Player Birth"),
		new App.Events.SEpcBirthday(),
		new App.Events.TwineEvent().wrapPassage([
			() => V.independenceDay === 1,
			() => (V.week-23) % 52 === 0
		], "SE independence day"),
		new App.Events.SEWedding(),
		new App.Events.TwineEvent().wrapPassage([
			() => V.playerBred === 1,
			() => (V.PC.preg === 0 || V.PC.preg === -1),
			() => V.PC.pregWeek === 0,
			() => V.PC.vagina >= 1
		], "P insemination"),
		new App.Events.SERetire(),
		new App.Events.SEExpiration(),
		new App.Events.SEBurst(),
		new App.Events.SEDeath(),
		new App.Events.SEBirth(),
		new App.Events.SEfctv(),
		new App.Events.TimeGatedPlotEvent(),
		new App.Events.assistantAwakens(),
		new App.Events.assistantSP(),
		new App.Events.assistantFS(),
		new App.Events.assistantName(),
		new App.Events.assistantMarket(),
		new App.Events.assistantBody(),
		new App.Events.pBadCuratives(),
		new App.Events.pBadBreasts(),
		new App.Events.pAidInvitation(),
		new App.Events.TwineEvent().wrapPassage([
			() => V.RecruiterID !== 0,
			() => V.recruiterProgress >= (13 + (V.recruiterEugenics === 1 ? policies.countEugenicsSMRs() * 6 : 0))
		], "SE recruiter success"),
		new App.Events.SEcustomSlaveDelivery(),
		new App.Events.TwineEvent().wrapPassage([
			() => V.JFC.order === 1,
			() => V.JFC.reorder !== 1
		], "JobFulfillmentCenterDelivery"),
		new App.Events.TwineEvent().wrapPassage([
			() => V.huskSlaveOrdered === 1
		], "SE husk slave delivery"),
		new App.Events.TwineEvent().wrapPassage([
			() => V.LurcherID !== 0,
			() => V.policies.coursingAssociation === 1,
			() => (Math.trunc(V.week/4) === (V.week/4)),
			() => V.coursed === 0
		], "SE coursing"),
		new App.Events.SERaiding(),
		new App.Events.SEPitFight(),
		new App.Events.TwineEvent().wrapPassage([
			() => V.bioreactorPerfectedID !== 0,
			() => (V.bioreactorsAnnounced !== 1)
		], "P bioreactor perfected"),
		new App.Events.TwineEvent().wrapPassage([
			() => !!App.Utils.schoolFailure()
		], "RES Failure"),
		new App.Events.TwineEvent().wrapPassage([
			() => V.TFS.schoolPresent === 1,
			() => V.organFarmUpgrade !== 0,
			() => V.TFS.farmUpgrade === 0
		], "TFS Farm Upgrade"),
		new App.Events.TwineEvent().wrapPassage([
			() => V.nicaea.held !== 1,
			() => V.arcologies[0].FSChattelReligionist !== "unset",
			() => V.arcologies[0].FSChattelReligionist >= V.FSLockinLevel,
			() => V.nicaea.announced !== 1,
			() => V.nicaea.eventWeek !== V.week
		], "SE nicaea announcement"),
		new App.Events.TwineEvent().wrapPassage([
			() => V.nicaea.held !== 1,
			() => V.arcologies[0].FSChattelReligionist !== "unset",
			() => V.nicaea.preparation === 1,
			() => V.nicaea.eventWeek !== V.week
		], "SE nicaea preparation"),
		new App.Events.TwineEvent().wrapPassage([
			() => V.nicaea.held !== 1,
			() => V.arcologies[0].FSChattelReligionist !== "unset",
			() => V.nicaea.involvement >= 0,
			() => V.nicaea.eventWeek !== V.week
		], "SE nicaea council"),
		new App.Events.MurderAttempt(),
		new App.Events.MurderAttemptFollowup(),
		new App.Events.pRaped(),
		new App.Events.pAbducted(),
	];
};

/** get the next nonrandom event which should fire
 * @returns {App.Events.BaseEvent}
 */
App.Events.getNextNonrandomEvent = function() {
	return App.Events.getNonrandomEvents()
		.find(e => App.Events.canExecute(e));
};

/** get all the nonrandom events which should fire this week
 * @returns {Array<App.Events.BaseEvent>}
 */
App.Events.getWeekNonrandomEvents = function() {
	return App.Events.getNonrandomEvents()
		.filter(e => App.Events.canExecute(e));
};

/** get the next queued event which should fire, and remove it from the queue
 * @returns {App.Events.BaseEvent}
 */
App.Events.dequeueNextQueuedEvent = function() {
	const event = (V.eventQueue[0] || [])
		.find(e => App.Events.canExecute(e));
	if (event) {
		V.eventQueue[0].delete(event);
	}
	return event;
};

/** get all the queued events which should fire this week
 * @returns {Array<App.Events.BaseEvent>}
 */
App.Events.getWeekQueuedEvents = function() {
	return (V.eventQueue[0] || [])
		.filter(e => App.Events.canExecute(e));
};

App.Events.playNonrandomEvent = function() {
	const d = document.createElement("div");
	V.nextLink = passage();
	V.nextButton = "Continue";

	const clearEvent = () => { V.event = null; };

	if (V.event instanceof App.Events.BaseEvent) {
		// we've deserialized a saved game with an event active, or a player has picked one, so just play it immediately
		V.event.execute(d);
		V.passageSwitchHandler = clearEvent;
	} else {
		if (V.debugMode > 0 && V.debugModeEventSelection > 0) {
			V.nextButton = "Refresh";
			// show all the scheduled, nonrandom, and queued events, and allow the player to play them in any order and skip the remainder
			App.UI.DOM.appendNewElement("h2", d, "Scheduled and Nonrandom Events");
			App.UI.DOM.appendNewElement("div", d, "These scheduled and nonrandom events still need to play this week, in this order.");
			App.UI.DOM.appendNewElement("div", d, "WARNING: playing certain scheduled events out of order, or skipping them, can break your game! Be careful!", ["note", "warning"]);
			const events = App.Events.getWeekNonrandomEvents();
			const linkList = App.UI.DOM.appendNewElement("div", d, '', "event-section");
			for (const event of events) {
				App.UI.DOM.appendNewElement("div", linkList, App.UI.DOM.passageLink(event.eventName, passage(), () => { V.event = event; }));
			}
			const queuedEvents = App.Events.getWeekQueuedEvents();
			for (const event of queuedEvents) {
				App.UI.DOM.appendNewElement("div", linkList, App.UI.DOM.passageLink(event.eventName, passage(), () => { V.event = event; V.eventQueue[0].delete(event); }));
			}
			if (events.length + queuedEvents.length > 0) {
				App.UI.DOM.appendNewElement("div", d, App.UI.DOM.passageLink("SKIP remaining events and proceed", "Nonrandom Event"));
			} else {
				App.UI.DOM.appendNewElement("div", d, App.UI.DOM.passageLink("No more events. Proceed.", "Nonrandom Event"));
			}
			d.append(App.Events.renderEventDebugger());
		} else {
			// pick the next scheduled, nonrandom, or queued event, if there is one
			const event = App.Events.getNextNonrandomEvent() || App.Events.dequeueNextQueuedEvent();
			if (event) {
				// record the chosen event in 'current' (pre-play!) history as well as current state so that it will serialize out correctly if saved from this passage
				// WARNING: THIS IS ***NOT*** THE ACTIVE STATE PAGE!
				// @ts-ignore - under-defined object
				State.current.variables.event = V.event = event;
				event.execute(d);
				V.passageSwitchHandler = clearEvent;
			} else {
				// no more events for this week, move on to random events (or ordinary Nonrandom for now, until we get rid of globalThis.nonRandomEvent)
				setTimeout(() => Engine.play("Nonrandom Event"), Engine.minDomActionDelay);
			}
		}
	}
	return d;
};

/** @deprecated - use instance list in getNextNonrandomEvent instead */
globalThis.nonRandomEvent = function() {
	V.activeSlave = 0;
	V.eventSlave = 0;
	const effectiveWeek = App.Events.effectiveWeek();
	if (V.plot) {
		if (effectiveWeek >= 5 && V.FCTV.receiver === -1) {
			setTimeout(() => Engine.play("SE FCTV Install"), Engine.minDomActionDelay);
		} else if (V.projectN.status === 1) {
			V.projectN.status = 2;
			V.projectN.phase1 = effectiveWeek;
			setTimeout(() => Engine.play("SE projectNInitialized"), Engine.minDomActionDelay);
		} else if ((V.bodyPuristRiot === 1) && (V.puristRiotDone === 0) && (effectiveWeek >= V.projectN.phase1 + 1) && (V.projectN.status !== 9)) {
			setTimeout(() => Engine.play("SE bodypuristprotest"), Engine.minDomActionDelay);
			V.puristRiotDone = 1;
		} else if ((V.projectN.status === 2) && (V.projectN.public === 0) && (effectiveWeek >= V.projectN.phase1 + 3)) {
			setTimeout(() => Engine.play("SE projectNmomoney"), Engine.minDomActionDelay);
			V.projectN.phase2 = effectiveWeek;
		} else if ((V.projectN.status === 2) && (V.projectN.public === 1) && (effectiveWeek >= V.projectN.phase1 + 5)) {
			setTimeout(() => Engine.play("SE projectNmomoney"), Engine.minDomActionDelay);
			V.projectN.phase2 = effectiveWeek;
		} else if ((V.projectN.status === 3) && (V.projectN.wellFunded === 1) && (effectiveWeek >= V.projectN.phase2 + 4)) {
			setTimeout(() => Engine.play("SE projectNbubbles"), Engine.minDomActionDelay);
			V.projectN.status = 4;
			V.projectN.phase3 = effectiveWeek;
		} else if ((V.projectN.status === 3) && (V.projectN.wellFunded !== 1) && (effectiveWeek >= V.projectN.phase2 + 6)) {
			setTimeout(() => Engine.play("SE projectNbubbles"), Engine.minDomActionDelay);
			V.projectN.status = 4;
			V.projectN.phase3 = effectiveWeek;
		} else if ((V.projectN.status === 4) && (V.projectN.poorlyFunded !== 1) && (effectiveWeek >= V.projectN.phase3 + 4)) {
			setTimeout(() => Engine.play("SE projectNsaboteur"), Engine.minDomActionDelay);
			V.projectN.phase4 = effectiveWeek;
		} else if ((V.projectN.status === 4) && (V.projectN.public === 1) && (V.projectN.poorlyFunded === 1) && (effectiveWeek >= V.projectN.phase3 + 6)) {
			setTimeout(() => Engine.play("SE projectNsaboteur"), Engine.minDomActionDelay);
			V.projectN.phase4 = effectiveWeek;
		} else if ((V.projectN.status === 4) && (V.projectN.public === 0) && (V.projectN.poorlyFunded === 1) && (effectiveWeek >= V.projectN.phase3 + 6)) {
			setTimeout(() => Engine.play("SE projectNblowingthelid"), Engine.minDomActionDelay);
			V.projectN.phase4 = effectiveWeek;
		} else if ((V.puristsFurious === 1) && (V.puristRiotDone === 0) && (effectiveWeek >= V.projectN.phase4 + 1 && (V.projectN.status !== 9))) {
			setTimeout(() => Engine.play("SE bodypuristriot"), Engine.minDomActionDelay);
			V.puristRiotDone = 1;
		} else if ((V.projectN.status === 5) && (effectiveWeek >= V.projectN.phase4 + 4)) {
			setTimeout(() => Engine.play("SE projectNcomplete"), Engine.minDomActionDelay);
		} else if ((V.projectN.status >= 6) && (V.projectN.status !== 9) && V.projectN.decisionMade !== 1 && (effectiveWeek >= V.projectN.phase4 + 5)) {
			setTimeout(() => Engine.play("SE projectNtechrelease"), Engine.minDomActionDelay);
		} else if ((V.projectN.status === 7) && (V.growingNewCat === 0)) {
			setTimeout(() => Engine.play("SE vatcatgirl"), Engine.minDomActionDelay);
		} else if (V.projectN.status === 8 && V.growingNewCat === 0) {
			setTimeout(() => Engine.play("SE vatcatboy"), Engine.minDomActionDelay);
		} else if ((effectiveWeek >= 15) && (V.arcologies[0].FSNeoImperialistLaw1 === 1) && V.assholeKnight !== 1) {
			V.assholeKnight = 1;
			V.imperialEventWeek = effectiveWeek;
			setTimeout(() => Engine.play("SE assholeknight"), Engine.minDomActionDelay);
		} else if ((effectiveWeek >= 44) && (V.mercenaries > 0) && V.mercRomeo !== 1) {
			const valid = V.slaves.find(function(s) { return (["serve the public", "serve in the club", "whore", "work in the brothel"].includes(s.assignment) || s.counter.publicUse >= 50) && s.fetish !== "mindbroken" && s.fuckdoll === 0; });
			V.mercRomeo = 1;
			if (valid) {
				setTimeout(() => Engine.play("P mercenary romeo"), Engine.minDomActionDelay);
			} else {
				setTimeout(() => Engine.play("Nonrandom Event"), Engine.minDomActionDelay);
			}
		} else if (effectiveWeek === 48 && V.experimental.food === 1) {
			V.foodCrisis = 1;
			setTimeout(() => Engine.play("P food crisis"), Engine.minDomActionDelay);
		} else if (effectiveWeek === 50 && V.rations > 0) {
			V.foodCrisis = 2;
			setTimeout(() => Engine.play("P food crisis"), Engine.minDomActionDelay);
		} else if (effectiveWeek === 54 && (V.peacekeepers) && V.peacekeepers.attitude >= 0) {
			setTimeout(() => Engine.play("P peacekeepers deficit"), Engine.minDomActionDelay);
		} else if (effectiveWeek === 60 && V.rations > 0) {
			V.foodCrisis = 3;
			setTimeout(() => Engine.play("P food crisis"), Engine.minDomActionDelay);
		} else if (V.SF.Toggle && V.SF.Active === -1 && effectiveWeek >= 72) {
			setTimeout(() => Engine.play("Security Force Proposal"), Engine.minDomActionDelay);
		} else if (V.arcologies[0].FSRestart !== "unset" && V.failedElite > 300 && V.eugenicsFullControl !== 1) {
			setTimeout(() => Engine.play("eliteTakeOver"), Engine.minDomActionDelay);
		} else if (effectiveWeek > 5 && V.rep > 3000 && V.FSAnnounced === 0) {
			setTimeout(() => Engine.play("P FS Announcement"), Engine.minDomActionDelay);
		} else if (effectiveWeek > 45 && V.bodyswapAnnounced === 0 && V.surgeryUpgrade === 1) {
			setTimeout(() => Engine.play("P Bodyswap Reveal"), Engine.minDomActionDelay);
		} else if (effectiveWeek > 48 && V.invasionVictory > 0 && V.peacekeepers === 0 && V.peacekeepersGone !== 1) {
			setTimeout(() => Engine.play("P peacekeepers intro"), Engine.minDomActionDelay);
		} else if (V.arcologies[0].prosperity > 80 && App.Utils.schoolCounter() === 0 && V.schoolSuggestion === 0) {
			setTimeout(() => Engine.play("P school suggestion"), Engine.minDomActionDelay);
		} else if (effectiveWeek > 70 && V.corp.Incorporated > 0 && V.rivalOwnerEnslaved > 0 && V.mercenaries >= 3 && V.mercenariesHelpCorp === 0 && V.corp.DivExtra > 0) {
			setTimeout(() => Engine.play("P Mercs Help Corp"), Engine.minDomActionDelay);
		} else if (effectiveWeek > 75 && (V.peacekeepers) && V.peacekeepers.strength < 50 && V.rivalOwner === 0 && V.peacekeepersFate !== 1) {
			setTimeout(() => Engine.play("P peacekeepers independence"), Engine.minDomActionDelay);
		} else if ((V.peacekeepers) && V.peacekeepers.strength >= 50 && V.peacekeepers.influenceAnnounced === 0) {
			setTimeout(() => Engine.play("P peacekeepers influence"), Engine.minDomActionDelay);
		} else if (V.cash > 120000 && V.rep > 4000 && V.corp.Announced === 0) {
			setTimeout(() => Engine.play("P Corp Announcement"), Engine.minDomActionDelay);
		} else if (V.rivalOwner > 0) {
			if (V.hostageAnnounced === 0 && V.rivalSet !== 0) {
				setTimeout(() => Engine.play("P rivalry hostage"), Engine.minDomActionDelay);
			} else if ((V.rivalOwner - V.rivalryPower + 10) / V.arcologies[0].prosperity < 0.5) {
				setTimeout(() => Engine.play("P rivalry victory"), Engine.minDomActionDelay);
			} else if (V.peacekeepers && V.peacekeepers.attitude > 5 && V.rivalryDuration > 1) {
				setTimeout(() => Engine.play("P rivalry peacekeepers"), Engine.minDomActionDelay);
			} else {
				setTimeout(() => Engine.play("P rivalry actions"), Engine.minDomActionDelay);
			}
		} else if ((V.arcologies[0].FSPaternalistDecoration === 100) && (V.swanSong === 2) && (V.week - V.swanSongWeek >= 3)) {
			setTimeout(() => Engine.play("RE The Siren Strikes Back"), Engine.minDomActionDelay);
		} else if (V.eventResults.snatch === 1) {
			setTimeout(() => Engine.play("P snatch and grab result"), Engine.minDomActionDelay);
		} else if (V.eventResults.raid === 1) {
			setTimeout(() => Engine.play("P raid result"), Engine.minDomActionDelay);
		} else if (V.eventResults.slaveMedic > 0) {
			setTimeout(() => Engine.play("P slave medic"), Engine.minDomActionDelay);
		} else if (V.eventResults.pit === 1 && V.BodyguardID !== 0) {
			setTimeout(() => Engine.play("PE pit fight"), Engine.minDomActionDelay);
		} else if ((effectiveWeek >= 37) && (V.arcologies[0].FSNeoImperialistLaw1 === 1) && (V.arcologies[0].FSNeoImperialistLaw2 === 1) && V.poorKnight !== 1 && effectiveWeek >= V.imperialEventWeek + 3) {
			V.poorKnight = 1;
			V.imperialEventWeek = effectiveWeek;
			setTimeout(() => Engine.play("SE poorknight"), Engine.minDomActionDelay);
		} else if ((effectiveWeek >= 40) && (V.arcologies[0].prosperity > 80) && (V.arcologies[0].FSNeoImperialistLaw2 === 1) && V.newBaron !== 1 && effectiveWeek >= V.imperialEventWeek + 3) {
			V.newBaron = 1;
			V.imperialEventWeek = effectiveWeek;
			setTimeout(() => Engine.play("SE newBaron"), Engine.minDomActionDelay);
		} else if (V.rivalOwner === 0 && V.secExpEnabled > 0 && (effectiveWeek >= 74 && V.SecExp.smilingMan.progress === 0 || effectiveWeek >= 77 && V.SecExp.smilingMan.progress === 1 || effectiveWeek >= 82 && V.SecExp.smilingMan.progress === 2 || V.SecExp.smilingMan.progress === 3)) {
			setTimeout(() => Engine.play("secExpSmilingMan"), Engine.minDomActionDelay);
		} else if (V.rivalOwner === 0 && V.seeFCNN === 1 && V.FCNNstation === 0 && V.week > 95 && V.cash > 200000 && V.rep > 7500) {
			setTimeout(() => Engine.play("SE FCNN Station"), Engine.minDomActionDelay);
		} else {
			if (random(1, 100) > effectiveWeek + 25) {
				setTimeout(() => Engine.play("RIE Eligibility Check"), Engine.minDomActionDelay);
			} else {
				setTimeout(() => Engine.play("Random Nonindividual Event"), Engine.minDomActionDelay);
			}
		}
	} else {
		if (random(1, 200) > effectiveWeek + 100 || V.hostageRescued === 1) {
			setTimeout(() => Engine.play("RIE Eligibility Check"), Engine.minDomActionDelay);
		} else {
			setTimeout(() => Engine.play("Random Nonindividual Event"), Engine.minDomActionDelay);
		}
	}
	// never reached, just for typing
	return new DocumentFragment();
};
