/** This class wraps a Twine passage as a Javascript nonrandom event with a predicate list.
 * Casting must be performed manually (in Twine) if required, so this mechanism is unsuitable for most random events.
 * Not guaranteed safe for arbitrary passages, TEST BEFORE USING */
App.Events.TwineEvent = class TwineEvent extends App.Events.BaseEvent {
	constructor(actors, params) {
		super(actors, params);
		if (!params) {
			this.params.initialized = false;
		}
	}

	/** Initialize this TwineEvent Wrapper for the given twine passage
	 * @param {Array<eventPredicate>} prereqs
	 * @param {string} passage
	 * @returns {App.Events.TwineEvent}
	 */
	wrapPassage(prereqs, passage) {
		this.params.passage = passage;
		this.params.prereqs = prereqs;
		this.params.initialized = true;
		return this;
	}

	eventPrerequisites() {
		return this.params.prereqs;
	}

	actorPrerequisites() {
		return []; // automatic casting is not supported for twine events
	}

	get eventName() {
		return this.params.passage + " (tw)";
	}

	execute(node) {
		if (!this.params.initialized) {
			throw new Error("Uninitialized Twine Event");
		}
		node.append(App.UI.DOM.renderPassage(this.params.passage));
	}
};
