App.Facilities.Farmyard.farmyard = function() {
	const frag = new DocumentFragment();

	const introDiv = App.UI.DOM.makeElement("div", null, ['farmyard-intro']);
	const expandDiv = App.UI.DOM.makeElement("div", null, ['farmyard-expand']);
	const menialsDiv = App.UI.DOM.makeElement("div", null, ['farmyard-menials']);
	const rulesDiv = App.UI.DOM.makeElement("div", null, ['farmyard-rules']);
	const upgradesDiv = App.UI.DOM.makeElement("div", null, ['farmyard-upgrades']);
	const kennelsDiv = App.UI.DOM.makeElement("div", null, ['farmyard-kennels']);
	const stablesDiv = App.UI.DOM.makeElement("div", null, ['farmyard-stables']);
	const cagesDiv = App.UI.DOM.makeElement("div", null, ['farmyard-cages']);
	const removeHousingDiv = App.UI.DOM.makeElement("div", null, ['farmyard-remove']);
	const renameDiv = App.UI.DOM.makeElement("div", null, ['farmyard-rename']);
	const slavesDiv = App.UI.DOM.makeElement("div", null, ['farmyard-slaves']);

	let farmyardNameCaps = capFirstChar(V.farmyardName);

	const count = App.Entity.facilities.farmyard.totalEmployeesCount;

	V.nextButton = "Back to Main";
	V.nextLink = "Main";
	V.returnTo = "Farmyard";
	V.encyclopedia = "Farmyard";

	frag.append(
		intro(),
		expand(),
		menials(),
		rules(),
		upgrades(),
		kennels(),
		stables(),
		cages(),
		removeHousing(),
		rename(),
		slaves(),
	);

	return frag;

	function intro() {
		const text = [];

		text.push(`${farmyardNameCaps} is an oasis of growth in the midst of the jungle of steel and concrete that is ${V.arcologies[0].name}. Animals are kept in pens, tended to by your slaves, while ${V.farmyardUpgrades.hydroponics
			? `rows of hydroponics equipment`
			: `makeshift fields`} grow crops. `);

		switch (V.farmyardDecoration) {
			case "Roman Revivalist":
				text.push(`Its red tiles and white stone walls are the very picture of a Roman farm villa's construction, as are the marble statues and reliefs. Saturn and Ceres look over the prosperity of the fields${V.seeBestiality ? `. Mercury watches over the health of the animals, and Feronia ensures strong litters in your slaves.` : `, and Mercury watches over the health of the animals.`} The slaves here are all looked after well, as they have one of the most important jobs in ${V.arcologies[0].name}.`);
				break;
			case "Neo-Imperialist":
				text.push(`Its high-tech, sleek black design invocates an embracement of the future, tempered by the hanging banners displaying your family crest as the rightful lord and master of these farms. Serf-like peasants work tirelessly in the fields, both to grow crops and oversee the slaves beneath them. Despite the harsh nature of the fieldwork, the slaves here are all looked after well, as they have one of the most important jobs in ${V.arcologies[0].name}.`);
				break;
			case "Aztec Revivalist":
				text.push(`It can't completely recreate the floating farms in the ancient Aztec fashion, but it comes as close as it can, shallow pseudo-canals dividing each field into multiple sections. Smooth stone and colorful murals cover the walls, depicting bloody stories of gods and mortals alike.`);
				break;
			case "Egyptian Revivalist":
				text.push(`It does its best to capture the wide open nature of ancient Egyptian farms, including mimicking the irrigation systems fed by the Nile. The stone walls are decorated with murals detailing its construction and your prowess in general, ${V.seeBestiality ? `with animal-bloated slaves featured prominently.` : `hieroglyphs spelling out volumes of praise.`}`);
				break;
			case "Edo Revivalist":
				text.push(`It does its best to mimic the rice patties and thatch roofed buildings of the Edo period despite the wide variety of crops tended by various slaves. Not every crop can thrive in flooded fields, but the ones that can take advantage of your attention to detail.`);
				break;
			case "Arabian Revivalist":
				text.push(`Large plots of olive trees and date palms line the outer edges of the main crop area, while a combination of wheat, flax, and barley occupies the interior space. Irrigation canals snake through the area, ensuring every inch of cropland is well-watered.`);
				break;
			case "Chinese Revivalist":
				text.push(`It does its best to capture the terraces that covered the ancient Chinese hills and mountains, turning every floor into ribbons of fields following a slight incline. Slaves wade through crops that can handle flooding and splash through the irrigation of the others when they aren't tending to${V.seeBestiality ? ` or breeding with` : ``} your animals.`);
				break;
			case "Chattel Religionist":
				text.push(`It runs like a well oiled machine, slaves bent in humble service as they tend crops grown on the Prophet's command, or see to the animals' needs. Their clothing is tucked up and out of the way as they see to their tasks, keeping them clean as they work ${V.seeBestiality ? `around animal-bloated bellies ` : ``}as divine will dictates.`);
				break;
			case "Degradationist":
				text.push(`It is constructed less as a converted warehouse and more as something to visit, allowing guests to enjoy the spectacle of slaves ${V.seeBestiality ? `being pounded by eager animals` : `elbow deep in scrubbing animal waste`} to their satisfaction.`);
				break;
			case "Repopulationist":
				text.push(`It teems with life, both in the belly of every animal and the belly of every slave, though the latter makes tending the fields difficult. They're ordered to take care, as they carry the future ${V.seeBestiality ? `of this farm` : `of the arcology`} in their bellies.`);
				break;
			case "Eugenics":
				text.push(`It holds a wide variety of crops and animals, but the best of the best is easy to find. They're set apart from the others, given only the best care and supplies${V.seeBestiality ? ` and bred with only the highest quality slaves` : ``}, while the sub-par stock is neglected off to the side.`);
				break;
			case "Asset Expansionist":
				text.push(`It is not easy to look after animals and till fields with such enormous body parts, but your slaves are diligent regardless, working hard to provide food and livestock for the arcology.`);
				break;
			case "Transformation Fetishist":
				// text.push(`TODO:`);
				break;
			case "Gender Radicalist":
				// text.push(`TODO:`);
				break;
			case "Gender Fundamentalist":
				// text.push(`TODO:`);
				break;
			case "Physical Idealist":
				text.push(`Its animals are in exceptional shape, their coats unable to hide how muscular they are, requiring your slaves to be equally toned to control them. There's plenty of space for their exercise as well${V.seeBestiality ? ` and an abundance of curatives for the slaves full of their fierce, kicking offspring` : ``}.`);
				break;
			case "Supremacist":
				text.push(`It is a clean and orderly operation, stables and cages mucked by a multitude of inferior slaves, along with grooming your animals and harvesting your crops.`);
				break;
			case "Subjugationist":
				text.push(`It is a clean and orderly operation, stables and cages mucked by a multitude of ${V.arcologies[0].FSSubjugationistRace} slaves, while the others are tasked with grooming your animals and harvesting your crops.`);
				break;
			case "Paternalist":
				text.push(`It's full of healthy animals, crops, and slaves, the former's every need diligently looked after by the latter. The fields flourish to capacity under such care, and the animals give the distinct impression of happiness${V.seeBestiality ? ` — some more than others if the growing bellies of your slaves are anything to go by, the only indication that such rutting takes place` : ``}.`);
				break;
			case "Pastoralist":
				// text.push(`TODO:`);
				break;
			case "Maturity Preferentialist":
				// text.push(`TODO:`);
				break;
			case "Youth Preferentialist":
				// text.push(`TODO:`);
				break;
			case "Body Purist":
				// text.push(`TODO:`);
				break;
			case "Slimness Enthusiast":
				text.push(`It features trim animals and slaves alike, not a pound of excess among them. The feed for both livestock and crops are carefully maintained to ensure optimal growth without waste, letting them flourish without being weighed down.`);
				break;
			case "Hedonistic":
				text.push(`It features wider gates and stalls, for both the humans visiting or tending the occupants, and the animals starting to mimic their handlers${V.seeBestiality ? ` and company` : ``}, with plenty of seats along the way.`);
				break;
			case "Slave Professionalism":
				// text.push(`TODO:`);
				break;
			case "Intellectual Dependency":
				// text.push(`TODO:`);
				break;
			default:
				text.push(`It is very much a converted warehouse still, sectioned off in various 'departments'${V.farmyardUpgrades.machinery ? ` with machinery placed where it can be` : V.farmyardUpgrades.hydroponics ? ` and plumbing for the hydroponics system running every which way` : ``}.`);
				break;
		}

		if (count > 2) {
			text.push(`${farmyardNameCaps} is bustling with activity. Farmhands are hurrying about, on their way to feed animals and maintain farming equipment.`);
		} else if (count) {
			text.push(`${farmyardNameCaps} is working steadily. Farmhands are moving about, looking after the animals and crops.`);
		} else if (S.Farmer) {
			text.push(`${S.Farmer.slaveName} is alone in ${V.farmyardName}, and has nothing to do but look after the animals and crops.`);
		} else {
			text.push(`${farmyardNameCaps} is empty and quiet.`);
		}

		App.UI.DOM.appendNewElement("div", introDiv, text.join(' '), ['scene-intro']);

		if (count === 0) {
			App.UI.DOM.appendNewElement("div", introDiv, App.UI.DOM.passageLink(`Decommission ${V.farmyardName}`, "Main", () => {
				if (V.farmMenials) {
					V.menials += V.farmMenials;
					V.farmMenials = 0;
				}

				V.farmyardName = "the Farmyard";
				V.farmyard = 0;
				V.farmyardDecoration = "standard";

				V.farmMenials = 0;
				V.farmMenialsSpace = 0;

				V.farmyardShows = 0;
				V.farmyardBreeding = 0;
				V.farmyardCrops = 0;

				V.farmyardKennels = 0;
				V.farmyardStables = 0;
				V.farmyardCages = 0;

				if (V.pit) {
					V.pit.animal = null;
				}

				V.farmyardUpgrades = {
					pump: 0,
					fertilizer: 0,
					hydroponics: 0,
					machinery: 0,
					seeds: 0
				};

				clearAnimalsPurchased();
				App.Arcology.cellUpgrade(V.building, App.Arcology.Cell.Manufacturing, "Farmyard", "Manufacturing");
			}), ['indent']);
		}

		return introDiv;
	}

	function expand() {
		const cost = Math.trunc(V.farmyard * 1000 * V.upgradeMultiplierArcology);

		expandDiv.append(`It can support ${num(V.farmyard)} farmhands. There ${count === 1 ? `is currently ${count} farmhand` : `are currently ${count} farmhands`} in ${V.farmyardName}.`);

		App.UI.DOM.appendNewElement("div", expandDiv, App.UI.DOM.link(`Expand ${V.farmyardName}`, () => {
			cashX(forceNeg(cost), "capEx");
			V.farmyard += 5;
			V.PC.skill.engineering += .1;

			App.UI.DOM.replace(expandDiv, expand);
		},
		[], '', `Costs ${cashFormat(cost)} and increases the capacity of ${V.farmyardName} by 5.`), ['indent']);

		if (count > 0) {
			App.UI.DOM.appendNewElement("div", expandDiv, removeFacilityWorkers("farmyard"), 'indent');
		}

		return expandDiv;
	}

	function menials() {
		menialsDiv.append(transferMenials(), buyMenials(), houseMenials());

		return menialsDiv;
	}

	function transferMenials() {
		const frag = new DocumentFragment();

		const links = [];

		if (V.farmMenials) {
			frag.append(`Assigned to ${V.farmyardName} ${V.farmMenials === 1 ? `is` : `are`} ${V.farmMenials} menial ${V.farmMenials === 1 ? `slave` : `slaves`}, working to produce as much food for your arcology as they can. `);
		}

		if (V.farmMenialsSpace) {
			frag.append(`You ${V.menials ? `own ${num(V.menials)}` : `don't own any`} free menial slaves. ${farmyardNameCaps} can house ${V.farmMenialsSpace} menial slaves total, with ${V.farmMenialsSpace - V.farmMenials} free spots. `);
		}

		if (V.farmMenialsSpace && V.farmMenials < V.farmMenialsSpace) {
			if (V.menials) {
				links.push(App.UI.DOM.link("Transfer in a menial slave", () => {
					V.menials--;
					V.farmMenials++;

					App.UI.DOM.replace(menialsDiv, menials);
				}));
			}

			if (V.menials >= 10 && V.farmMenials <= V.farmMenialsSpace - 10) {
				links.push(App.UI.DOM.link("Transfer in 10 menial slaves", () => {
					V.menials -= 10;
					V.farmMenials += 10;

					App.UI.DOM.replace(menialsDiv, menials);
				}));
			}

			if (V.menials >= 100 && V.farmMenials <= V.farmMenialsSpace - 100) {
				links.push(App.UI.DOM.link("Transfer in 100 menial slaves", () => {
					V.menials -= 100;
					V.farmMenials += 100;

					App.UI.DOM.replace(menialsDiv, menials);
				}));
			}

			if (V.menials) {
				links.push(App.UI.DOM.link("Transfer in all free menial slaves", () => {
					if (V.menials > V.farmMenialsSpace - V.farmMenials) {
						V.menials -= V.farmMenialsSpace - V.farmMenials;
						V.farmMenials = V.farmMenialsSpace;
					} else {
						V.farmMenials += V.menials;
						V.menials = 0;
					}

					App.UI.DOM.replace(menialsDiv, menials);
				}));
			}
		} else if (!V.farmMenialsSpace) {
			frag.append(`${farmyardNameCaps} cannot currently house any menial slaves. `);
		} else {
			frag.append(`${farmyardNameCaps} has all the menial slaves it can currently house assigned to it. `);
		}

		if (V.farmMenials) {
			links.push(App.UI.DOM.link("Transfer out all menial slaves", () => {
				V.menials += V.farmMenials;
				V.farmMenials = 0;

				App.UI.DOM.replace(menialsDiv, menials);
			}));
		}

		App.UI.DOM.appendNewElement("div", frag, App.UI.DOM.generateLinksStrip(links), ['indent']);

		return frag;
	}

	function buyMenials() {
		const frag = new DocumentFragment();

		const links = [];

		const popCap = menialPopCap();
		const bulkMax = popCap.value - V.menials - V.fuckdolls - V.menialBioreactors;

		const menialPrice = Math.trunc(menialSlaveCost());
		const maxMenials = Math.trunc(Math.clamp(V.cash / menialPrice, 0, bulkMax));

		if (V.farmMenialsSpace) {
			if (bulkMax > 0 || V.menials + V.fuckdolls + V.menialBioreactors === 0) {
				links.push(App.UI.DOM.link(`Buy ${num(1)}`, () => {
					V.menials++;
					V.menialSupplyFactor--;
					cashX(forceNeg(menialPrice), "farmyard");

					App.UI.DOM.replace(menialsDiv, menials);
				}));

				links.push(App.UI.DOM.link(`Buy ${num(10)}`, () => {
					V.menials += 10;
					V.menialSupplyFactor -= 10;
					cashX(forceNeg(menialPrice * 10), "farmyard");

					App.UI.DOM.replace(menialsDiv, menials);
				}));

				links.push(App.UI.DOM.link(`Buy ${num(100)}`, () => {
					V.menials += 100;
					V.menialSupplyFactor -= 100;
					cashX(forceNeg(menialPrice * 100), "farmyard");

					App.UI.DOM.replace(menialsDiv, menials);
				}));

				links.push(App.UI.DOM.link(`Buy maximum`, () => {
					V.menials += maxMenials;
					V.menialSupplyFactor -= maxMenials;
					cashX(forceNeg(maxMenials * menialPrice), "farmyard");

					App.UI.DOM.replace(menialsDiv, menials);
				}));
			}
		}

		if (V.farmMenials) {
			App.UI.DOM.appendNewElement("div", frag, App.UI.DOM.generateLinksStrip(links), ['indent']);
		}

		return frag;
	}

	function houseMenials() {
		const frag = new DocumentFragment();

		const unitCost = Math.trunc(1000 * V.upgradeMultiplierArcology);

		if (V.farmMenialsSpace < 500) {
			App.UI.DOM.appendNewElement("div", frag, `There is enough room in ${V.farmyardName} to build housing, enough to give ${V.farmMenialsSpace + 100} menial slaves a place to sleep and relax.`);

			App.UI.DOM.appendNewElement("div", frag, App.UI.DOM.link(`Build a new housing unit`, () => {
				cashX(forceNeg(unitCost), "farmyard");
				V.farmMenialsSpace += 100;
			},
			[], '', `Costs ${cashFormat(unitCost)} and increases housing by 100.`), ['indent']);
		}

		return frag;
	}

	function rules() {
		if (App.Entity.facilities.farmyard.employeesIDs().size > 0 && (V.farmyardKennels || V.farmyardStables || V.farmyardCages)) {	// TODO: redo this with V.farmyardShowgirls
			const options = new App.UI.OptionsGroup();

			options.addOption(`Slaves ${V.farmyardShows ? `are` : `are not`} putting on shows.`, "farmyardShows")
				.addValue(`Begin shows`, 1)
				.addValue(`End shows`, 0);

			if (V.farmyardShows && (V.canine || V.hooved || V.feline)) {
				if (V.seeBestiality) {
					options.addOption(`Slaves ${V.farmyardBreeding ? `are` : `are not`} being bred with animals.`, "farmyardBreeding")
						.addValue(`Begin breeding`, 1)
						.addValue(`End breeding`, 0);

					if (V.farmyardBreeding) {
						options.addOption(`${V.farmyardRestraints ? `All of the slaves` : `Only disobedient slaves`} are being restrained.`, "farmyardRestraints")
							.addValue(`Restrain all slaves`, 1)
							.addValue(`Restrain only disobedient slaves`, 0);
					}
				}
			}

			rulesDiv.append(options.render());
		}

		return rulesDiv;
	}

	function upgrades() {
		const farmyardUpgrades = V.farmyardUpgrades;

		const pumpCost = Math.trunc(5000 * V.upgradeMultiplierArcology);
		const fertilizerCost = Math.trunc(10000 * V.upgradeMultiplierArcology);
		const hydroponicsCost = Math.trunc(20000 * V.upgradeMultiplierArcology);
		const seedsCost = Math.trunc(25000 * V.upgradeMultiplierArcology);
		const machineryCost = Math.trunc(50000 * V.upgradeMultiplierArcology);

		if (!farmyardUpgrades.pump) {
			App.UI.DOM.appendNewElement("div", upgradesDiv, `${farmyardNameCaps} is currently using the basic water pump that it came with.`);

			upgradesDiv.append(createUpgrade(
				`Upgrade the water pump`,
				pumpCost,
				`slightly decreases upkeep costs`,
				"pump"
			));
		} else {
			App.UI.DOM.appendNewElement("div", upgradesDiv, `The water pump in ${V.farmyardName} is a more efficient model, slightly improving the amount of crops it produces.`);

			if (!farmyardUpgrades.fertilizer) {
				upgradesDiv.append(createUpgrade(
					`Use a higher-quality fertilizer`,
					fertilizerCost,
					`moderately increases crop yield and slightly increases upkeep costs`,
					"fertilizer"
				));
			} else {
				App.UI.DOM.appendNewElement("div", upgradesDiv, `${farmyardNameCaps} is using a higher-quality fertilizer, moderately increasing the amount of crops it produces and slightly raising upkeep costs.`);

				if (!farmyardUpgrades.hydroponics) {
					upgradesDiv.append(createUpgrade(
						`Purchase an advanced hydroponics system`,
						hydroponicsCost,
						`moderately decreases upkeep costs`,
						"hydroponics"
					));
				} else {
					App.UI.DOM.appendNewElement("div", upgradesDiv, `${farmyardNameCaps} is outfitted with an advanced hydroponics system, reducing the amount of water your crops consume and thus moderately reducing upkeep costs.`);

					if (!farmyardUpgrades.seeds) {
						upgradesDiv.append(createUpgrade(
							`Purchase genetically modified seeds`,
							seedsCost,
							`moderately increases crop yield and slightly increases upkeep costs`,
							"seeds"
						));
					} else {
						App.UI.DOM.appendNewElement("div", upgradesDiv, `${farmyardNameCaps} is using genetically modified seeds, significantly increasing the amount of crops it produces and moderately increasing upkeep costs.`);

						if (!farmyardUpgrades.machinery) {
							upgradesDiv.append(createUpgrade(
								`Upgrade the machinery`,
								machineryCost,
								`moderately increases crop yield and slightly increases upkeep costs`,
								"machinery"
							));
						} else {
							App.UI.DOM.appendNewElement("div", upgradesDiv, `The machinery in ${V.farmyardName} has been upgraded and is more efficient, significantly increasing crop yields and significantly decreasing upkeep costs.`);
						}
					}
				}
			}
		}

		/**
		 * @param {string} linkText The text to display.
		 * @param {number} price The price of the upgrade.
		 * @param {string} effect The change the upgrade causes.
		 * @param {"pump"|"fertilizer"|"hydroponics"|"machinery"|"seeds"} type The variable name of the upgrade.
		 */
		function createUpgrade(linkText, price, effect, type) {
			const frag = new DocumentFragment();

			App.UI.DOM.appendNewElement("div", frag, App.UI.DOM.link(linkText, () => {
				cashX(forceNeg(price), "farmyard");
				farmyardUpgrades[type] = 1;

				App.UI.DOM.replace(upgradesDiv, upgrades);
			},
			[], '', `Costs ${cashFormat(price)} and ${effect}.`), ['indent']);

			return frag;
		}

		return upgradesDiv;
	}

	function kennels() {
		const cost = Math.trunc(5000 * V.upgradeMultiplierArcology);

		const CL = V.canine.length;

		const dogs = CL === 1
			? `${V.canine[0]}s`
			: CL < 3
				? `several different breeds of dogs`
				: `all kinds of dogs`;
		const canines = CL === 1
			? V.canine[0].species === "dog"
				? V.canine[0].breed
				: V.canine[0].speciesPlural
			: CL < 3
				? `several different ${V.canine.every(c => c.species === "dog")
					? `breeds of dogs`
					: `species of canines`}`
				: `all kinds of canines`;

		if (V.farmyardKennels === 0) {
			App.UI.DOM.appendNewElement("div", kennelsDiv, App.UI.DOM.link(`Add kennels`, () => {
				cashX(forceNeg(cost), "farmyard");
				V.farmyardKennels = 1;
				V.PC.skill.engineering += .1;

				refresh();
			}, [], '', `Costs ${cashFormat(cost)}, will incur upkeep costs, and unlocks domestic canines.`), ['indent']);
		} else if (V.farmyardKennels === 1) {
			kennelsDiv.append(App.UI.DOM.passageLink("Kennels", "Farmyard Animals"), ` have been built in one corner of ${V.farmyardName}, and are currently ${CL < 1 ? `empty` : `occupied by ${dogs}`}.`);

			if (V.rep > 10000) {
				App.UI.DOM.appendNewElement("div", kennelsDiv, App.UI.DOM.link("Upgrade kennels", () => {
					cashX(forceNeg(cost * 2), "farmyard");
					V.farmyardKennels = 2;
					V.PC.skill.engineering += .1;

					refresh();
				}, [], '', `Costs ${cashFormat(cost * 2)} will incur additional upkeep costs, and unlocks exotic canines.`), ['indent']);
			} else {
				App.UI.DOM.appendNewElement("div", stablesDiv, App.UI.DOM.disabledLink("Upgrade kennels",
					[`You must be more reputable to be able to house exotic canines.`]),
				['indent']);
			}
		} else if (V.farmyardKennels === 2) {
			kennelsDiv.append(App.UI.DOM.passageLink("Large kennels", "Farmyard Animals"), ` have been built in one corner of ${V.farmyardName}, and are currently ${CL < 1 ? `empty` : `occupied by ${canines}`}.`);
		}

		return kennelsDiv;
	}

	function stables() {
		const cost = Math.trunc(5000 * V.upgradeMultiplierArcology);

		const HL = V.hooved.length;

		const hooved = HL === 1
			? `${V.hooved[0]}s` : HL < 3
				? `several different types of hooved animals`
				: `all kinds of hooved animals`;

		if (V.farmyardStables === 0) {
			App.UI.DOM.appendNewElement("div", stablesDiv, App.UI.DOM.link("Add stables", () => {
				cashX(forceNeg(cost), "farmyard");
				V.farmyardStables = 1;
				V.PC.skill.engineering += .1;

				refresh();
			}, [], '', `Costs ${cashFormat(cost)}, will incur upkeep costs, and unlocks domestic hooved animals.`), ['indent']);
		} else if (V.farmyardStables === 1) {
			stablesDiv.append(App.UI.DOM.passageLink("Stables", "Farmyard Animals"), ` have been built in one corner of ${V.farmyardName}, and are currently ${HL < 1 ? `empty` : `occupied by ${hooved}`}.`);

			if (V.rep > 10000) {
				App.UI.DOM.appendNewElement("div", stablesDiv, App.UI.DOM.link("Upgrade stables", () => {
					cashX(forceNeg(cost * 2), "farmyard");
					V.farmyardStables = 2;
					V.PC.skill.engineering += .1;

					refresh();
				}, [], '', `Costs ${cashFormat(cost * 2)}, will incur additional upkeep costs, and unlocks exotic hooved animals.`), ['indent']);
			} else {
				App.UI.DOM.appendNewElement("div", stablesDiv, App.UI.DOM.disabledLink("Upgrade stables",
					[`You must be more reputable to be able to house exotic hooved animals.`]),
				['indent']);
			}
		} else if (V.farmyardStables === 2) {
			stablesDiv.append(App.UI.DOM.passageLink("Large stables", "Farmyard Animals"), ` have been built in one corner of ${V.farmyardName}, and are currently ${HL < 1 ? `empty` : `occupied by ${hooved}`}.`);
		}

		return stablesDiv;
	}

	function cages() {
		const cost = Math.trunc(5000 * V.upgradeMultiplierArcology);

		const FL = V.feline.length;

		const cats = FL === 1
			? `${V.feline[0]}s`
			: FL < 3
				? `several different breeds of cats`
				: `all kinds of cats`;
		const felines = FL === 1
			? V.feline[0].species === "cat"
				? V.feline[0].breed
				: V.feline[0].speciesPlural
			: FL < 3
				? `several different ${V.feline.every(c => c.species === "cat")
					? `breeds of cats`
					: `species of felines`}`
				: `all kinds of felines`;

		if (V.farmyardCages === 0) {
			App.UI.DOM.appendNewElement("div", cagesDiv, App.UI.DOM.link("Add cages", () => {
				cashX(forceNeg(cost), "farmyard");
				V.farmyardCages = 1;
				V.PC.skill.engineering += .1;

				refresh();
			}, [], '', `Costs ${cashFormat(cost)}, will incur upkeep costs, and unlocks domestic felines.`), ['indent']);
		} else if (V.farmyardCages === 1) {
			cagesDiv.append(App.UI.DOM.passageLink("Cages", "Farmyard Animals"), ` have been built in one corner of ${V.farmyardName}, and are currently ${FL < 1 ? `empty` : `occupied by ${cats}`}.`);

			if (V.rep > 10000) {
				App.UI.DOM.appendNewElement("div", cagesDiv, App.UI.DOM.link("Upgrade cages", () => {
					cashX(forceNeg(cost * 2), "farmyard");
					V.farmyardCages = 2;
					V.PC.skill.engineering += .1;

					refresh();
				}, [], '', `Costs ${cashFormat(cost * 2)}, will increase upkeep costs, and unlocks exotic felines.`), ['indent']);
			} else {
				App.UI.DOM.appendNewElement("div", cagesDiv, App.UI.DOM.disabledLink("Upgrade cages",
					[`You must be more reputable to be able to house exotic felines.`]),
				['indent']);
			}
		} else if (V.farmyardCages === 2) {
			cagesDiv.append(App.UI.DOM.passageLink("Large cages", "Farmyard Animals"), ` have been built in one corner of ${V.farmyardName}, and are currently ${FL < 1 ? `empty` : `occupied by ${felines}`}.`);
		}

		return cagesDiv;
	}

	function removeHousing() {
		const cost = ((V.farmyardKennels + V.farmyardStables + V.farmyardCages) * 5000) * V.upgradeMultiplierArcology;

		if (V.farmyardKennels || V.farmyardStables || V.farmyardCages) {
			App.UI.DOM.appendNewElement("div", removeHousingDiv, App.UI.DOM.link("Remove the animal housing", () => {
				V.farmyardKennels = 0;
				V.farmyardStables = 0;
				V.farmyardCages = 0;

				V.farmyardShows = 0;
				V.farmyardBreeding = 0;
				V.farmyardRestraints = 0;

				clearAnimalsPurchased();
				cashX(forceNeg(cost), "farmyard");

				refresh();
			},
			[], '', `Will cost ${cashFormat(cost)}.`), ['indent']);
		}

		return removeHousingDiv;
	}

	function clearAnimalsPurchased() {
		V.canine = [];
		V.hooved = [];
		V.feline = [];

		V.active.canine = null;
		V.active.hooved = null;
		V.active.feline = null;
	}

	function rename() {
		renameDiv.append(frag.appendChild(App.Facilities.rename(App.Entity.facilities.farmyard, () => {
			farmyardNameCaps = capFirstChar(V.farmyardName);

			refresh();
		})));

		return renameDiv;
	}

	function slaves() {
		slavesDiv.append(App.UI.DOM.appendNewElement("div", frag, App.UI.SlaveList.stdFacilityPage(App.Entity.facilities.farmyard), 'farmyard-slaves'));

		return slavesDiv;
	}

	function refresh() {
		App.UI.DOM.replace(introDiv, intro);
		App.UI.DOM.replace(expandDiv, expand);
		App.UI.DOM.replace(menialsDiv, menials);
		App.UI.DOM.replace(rulesDiv, rules);
		App.UI.DOM.replace(upgradesDiv, upgrades);
		App.UI.DOM.replace(kennelsDiv, kennels);
		App.UI.DOM.replace(stablesDiv, stables);
		App.UI.DOM.replace(cagesDiv, cages);
		App.UI.DOM.replace(removeHousingDiv, removeHousing);
		App.UI.DOM.replace(renameDiv, rename);
		App.UI.DOM.replace(slavesDiv, slaves);
	}
};
