/**
 * @param {App.Entity.SlaveState|FC.FetusGenetics} slave
 * @param {boolean} fetus
 * @returns {DocumentFragment}
 */
App.UI.SlaveInteract.geneticQuirks = function(slave, fetus) {
	const el = new DocumentFragment();
	const options = new App.UI.OptionsGroup();
	for (const [key, obj] of App.Data.geneticQuirks) {
		if (obj.hasOwnProperty("requirements") && !obj.requirements) {
			continue;
		}
		const option = options.addOption(capFirstChar(obj.title), key, slave.geneticQuirks)
			.addComment(capFirstChar(obj.description))
			.addValue("off", 0).off()
			.addValue("carrier", 1);
		if (key === "heterochromia") {
			option.pulldown();
			for (const color of App.Medicine.Modification.eyeColor.map(color => color.value)) {
				option.addValue(capFirstChar(color), color);
			}
		} else {
			if (fetus) {
				option.addValue("active", obj.pubertyActivated ? 3 : 2).on();
			} else {
				option.addValue("active", 2).on();
				if (obj.pubertyActivated) {
					option.addValue("inactive", 3);
					option.comment += `. "inactive" is for dominant genes that are not yet taking effect (age, hormones, etc)`;
				}
			}
			if (key === "albinism" && "albinismOverride" in slave) {
				option.addCallbackToEach((val) => induceAlbinism(slave, val));
			}
		}
	}
	el.append(options.render());
	return el;
};
