/**
 * UI for performing surgery. Refreshes without refreshing the passage.
 * @param {App.Entity.SlaveState} slave
 * @param {boolean} [cheat=false]
 * @returns {HTMLElement}
 */

App.UI.surgeryPassageExotic = function(slave, cheat = false) {
	const container = document.createElement("span");
	container.append(content());
	return container;

	function content() {
		const frag = new DocumentFragment();
		const {
			His, He,
			his, him
		} = getPronouns(slave);

		frag.append(race());
		if (V.geneticMappingUpgrade >= 1) {
			App.UI.DOM.appendNewElement("h3", frag, `Retro-virus treatments:`);
			frag.append(geneTherapy());
		}
		frag.append(bodySwap());

		return frag;

		function race() {
			const el = new DocumentFragment();
			const linkArray = [];
			App.UI.DOM.appendNewElement("div", el, `${He} is ${slave.race}${(slave.race !== slave.origRace) ? `, but was originally ${slave.origRace}` : ``}. Surgically alter ${him} to look more:`);
			if (slave.indentureRestrictions > 1) {
				App.UI.DOM.appendNewElement("div", el, `${His} indenture forbids elective surgery`, ["choices", "note"]);
			} else {
				for (const [race, capRace] of App.Data.misc.filterRaces) {
					if (slave.race === race) {
						continue;
					}
					linkArray.push(
						makeLink(
							capRace,
							"race",
							() => {
								slave.race = race;
								slave.skin = randomRaceSkin(slave.race);
								slave.hColor = randomRaceHair(slave.race);
								setEyeColor(slave, randomRaceEye(slave.race));
								surgeryDamage(slave, 20);
							}
						)
					);
				}
			}

			App.UI.DOM.appendNewElement("div", el, App.UI.DOM.generateLinksStrip(linkArray), "choices");
			return el;
		}

		function geneTherapy() {
			const el = new DocumentFragment();
			const r = [];
			const linkArray = [];
			if (slave.indentureRestrictions >= 1) {
				App.UI.DOM.appendNewElement("div", el, `${His} indenture forbids elective surgery`, ["choices", "note"]);
			} else if (slave.health.health < 0) {
				App.UI.DOM.appendNewElement("div", el, `${He}'s too unhealthy to undergo gene therapy`, ["choices", "note"]);
			} else {
				if (V.arcologies[0].childhoodFertilityInducedNCSResearch === 1) {
					if (slave.geneMods.NCS === 0) {
						linkArray.push(makeLink(
							"Induced NCS treatment",
							"retrograde virus injection NCS",
							() => {
								slave.geneMods.NCS = 1;
								slave.chem += 40;
								surgeryDamage(slave, 80);
							},
							4,
							`This will induce NCS in ${his} genetic code`
						));
					} else {
						App.UI.DOM.appendNewElement("div", el, `${He} already has Induced NCS`, ["choices", "note"]);
					}
				}

				if (V.RapidCellGrowthFormula === 1) {
					if (slave.geneMods.rapidCellGrowth === 0) {
						linkArray.push(makeLink(
							"Increased elasticity treatment",
							"elasticity treatment",
							() => {
								slave.geneMods.rapidCellGrowth = 1;
								slave.chem += 100;
								surgeryDamage(slave, 40);
							},
							4,
							`This will alter ${his} genetic code to encourage ${his} body to stretch`
						));
					} else {
						App.UI.DOM.appendNewElement("div", el, `${He} already has received the plasticity increasing elasticity treatment`, ["choices", "note"]);
					}
				}

				if (V.immortalityFormula === 1) {
					if (slave.geneMods.immortality === 0) {
						linkArray.push(makeLink(
							"Immortality",
							"immortality treatment",
							() => {
								slave.geneMods.immortality = 1;
								slave.chem += 1000;
								surgeryDamage(slave, 40);
							},
							4,
							`This will alter ${his} genetic code to reverse and prevent aging, effectively thwarting the rigors of old age`
						));
					} else {
						App.UI.DOM.appendNewElement("div", el, `${He} already is already immortal`, ["choices", "note"]);
					}
				}
			}
			App.Events.addNode(el, r, "div");
			App.UI.DOM.appendNewElement("div", el, App.UI.DOM.generateLinksStrip(linkArray), "choices");
			el.append(retroVirus());
			return el;

			function retroVirus() {
				const el = new DocumentFragment();
				const slaveGeneList = App.UI.DOM.appendNewElement("ul", el);
				const select = App.UI.DOM.appendNewElement("select", el);
				select.classList.add("rajs-list");
				const description = App.UI.DOM.appendNewElement("div", el, null);
				for (const gene in slave.geneticQuirks) {
					const geneData = App.Data.geneticQuirks.get(gene);

					// Continue if player settings do not allow them to even see it described
					if (geneData.hasOwnProperty("requirements") && !geneData.requirements) {
						continue;
					}

					if (slave.geneticQuirks[gene] === 2 || (slave.geneticQuirks[gene] === 1 && V.geneticMappingUpgrade >= 2)) {
						const strength = (slave.geneticQuirks[gene] === 1) ? "Carrier for " : "Activated ";
						App.UI.DOM.appendNewElement("li", slaveGeneList, strength)
							.append(App.UI.DOM.makeElement("span", geneData.title, "orange"));
					}

					// Some of these can be described, but not tweaked in certain ways and circumstances
					if (
						(gene === "pFace" && slave.geneticQuirks.pFace !== 2) ||
						(gene === "uFace" && slave.geneticQuirks.uFace !== 2) ||
						(["heterochromia", "girlsOnly"].includes(gene))
					) {
						continue;
					}
					const choice = App.UI.DOM.appendNewElement("option", select, capFirstChar(geneData.title));
					choice.value = gene;
					select.append(choice);
				}
				select.selectedIndex = -1;
				select.onchange = () => {
					// selectedGene = select.options[select.selectedIndex];
					jQuery(description).empty().append(describeGene(select.value));
				};

				return el;

				function describeGene(selectedGene) {
					const el = new DocumentFragment();
					const r = [];
					const linkArray = [];
					const geneData = App.Data.geneticQuirks.get(selectedGene);
					r.push(`Selected gene is`);
					r.push(App.UI.DOM.makeElement("span", `${geneData.title}:`, "orange"));
					r.push(`${geneData.description}.`);
					App.Events.addNode(el, r, "div");

					const warning = "Applying a retro-virus treatment radically increases carcinogen buildup.";
					if (["pFace", "uFace"].includes(selectedGene)) {
						linkArray.push(makeLink(
							`Prevent passing of ${geneData.title}s`,
							"gene treatment",
							() => {
								slave.geneticQuirks[selectedGene] = 0;
								slave.chem += 100;
								surgeryDamage(slave, 40);
							},
							4,
							warning
						));
					} else if (slave.geneticQuirks[selectedGene] === 2) {
						linkArray.push(makeLink(
							`Correct ${geneData.title}`,
							"gene treatment",
							() => {
								if (selectedGene === "albinism") {
									induceAlbinism(slave, 0);
								} else {
									slave.geneticQuirks[selectedGene] = 0;
								}
								slave.chem += 100;
								surgeryDamage(slave, 40);
							},
							4,
							warning
						));
					} else if (slave.geneticQuirks[selectedGene] === 1 && V.geneticMappingUpgrade >= 2) {
						linkArray.push(makeLink(
							`${capFirstChar(geneData.title)} activation treatment`,
							"gene treatment",
							() => {
								if (selectedGene === "albinism") {
									induceAlbinism(slave, 2);
								} else {
									slave.geneticQuirks[selectedGene] = 2;
								}
								slave.chem += 100;
								surgeryDamage(slave, 40);
							},
							4,
							warning
						));
						linkArray.push(makeLink(
							`${capFirstChar(geneData.title)} carrier corrective treatment`,
							"gene treatment",
							() => {
								if (selectedGene === "albinism") {
									induceAlbinism(slave, 0);
								} else {
									slave.geneticQuirks[selectedGene] = 0;
								}
								slave.chem += 100;
								surgeryDamage(slave, 40);
							},
							4,
							warning
						));
					} else if (V.geneticFlawLibrary === 1) {
						linkArray.push(makeLink(
							`Induced ${geneData.title} treatment`,
							"gene treatment",
							() => {
								if (selectedGene === "albinism") {
									induceAlbinism(slave, 2);
								} else {
									slave.geneticQuirks[selectedGene] = 2;
								}
								slave.chem += 100;
								surgeryDamage(slave, 40);
							},
							10,
							`This will induce ${geneData.title} in ${his} genetic code. ${warning}`
						));
					}
					App.UI.DOM.appendNewElement("div", el, App.UI.DOM.generateLinksStrip(linkArray), "choices");
					return el;
				}
			}
		}

		function bodySwap() {
			const el = new DocumentFragment();
			const r = [];
			const linkArray = [];
			if (V.bodyswapAnnounced === 1 && slave.indenture < 0) {
				if (slave.bodySwap === 0) {
					r.push(`${He} is in ${his} native body.`);
				} else if (slave.origBodyOwner !== "") {
					r.push(`${He} currently occupies ${slave.origBodyOwner}'s body.`);
				} else {
					r.push(`${He} is no longer in ${his} native body.`);
				}
				if (slave.indenture === -1) {
					linkArray.push(App.UI.DOM.passageLink(
						`Swap ${his} body with another of your stock`,
						"Slave Slave Swap Workaround",
					));
				} else {
					App.UI.DOM.appendNewElement("div", el, `Indentured servants must remain in their own bodies.`, ["choices", "note"]);
				}
			} else if (V.cheatMode === 1) {
				linkArray.push(App.UI.DOM.link(
					`Force enable bodyswapping`,
					() => {
						V.bodyswapAnnounced = 1;
					},
					[],
					"Remote Surgery",
				));
			}
			App.Events.addNode(el, r, "div");
			App.UI.DOM.appendNewElement("div", el, App.UI.DOM.generateLinksStrip(linkArray), "choices");
			return el;
		}

		/**
		 *
		 * @param {string} title
		 * @param {string} surgeryType
		 * @param {function(void):void} [func]
		 * @param {number} [costMult=1]
		 * @param {string} [note]
		 * @returns {HTMLAnchorElement}
		 */
		function makeLink(title, surgeryType, func, costMult = 1, note = "") {
			const cost = Math.trunc(V.surgeryCost * costMult);
			return App.UI.DOM.link(
				title,
				() => {
					if (typeof func === "function") {
						func();
					}
					if (cheat) {
						jQuery(container).empty().append(content());
						App.Art.refreshSlaveArt(slave, 3, "art-frame");
					} else {
						V.surgeryType = surgeryType;
						// TODO: pass if it affected health or not?
						cashX(forceNeg(cost), "slaveSurgery", slave);
						Engine.play("Surgery Degradation");
					}
				},
				[],
				"",
				`Costs ${cashFormat(cost)}${note ? ` ${note}` : ""}`
			);
		}
	}
};
