/**
 * UI for performing surgery. Refreshes without refreshing the passage.
 * @param {App.Entity.SlaveState} slave
 * @param {boolean} [cheat=false]
 * @returns {HTMLElement}
 */

App.UI.surgeryPassageStructural = function(slave, cheat = false) {
	const container = document.createElement("span");
	container.append(content());
	return container;

	function content() {
		const frag = new DocumentFragment();
		const {
			His, He,
			he, him
		} = getPronouns(slave);

		frag.append(shoulders(), hips());
		if (hasAllNaturalLimbs(slave)) {
			frag.append(limbLength(), tendons());
		}
		App.UI.DOM.appendNewElement("h3", frag, `Amputation`);
		frag.append(amputate());
		App.UI.DOM.appendNewElement("h3", frag, `Prosthetics`);
		frag.append(prostheticInterface(), tail(), App.Medicine.OrganFarm.fullMenu(slave));

		return frag;

		function shoulders() {
			const el = new DocumentFragment();
			const r = [];
			const linkArray = [];

			if (slave.shoulders < -1) {
				r.push(`${He} has very narrow shoulders.`);
			} else if (slave.shoulders < 0) {
				r.push(`${He} has narrow shoulders.`);
			} else if (slave.shoulders < 1) {
				r.push(`${He} has average shoulders.`);
			} else if (slave.shoulders < 2) {
				r.push(`${He} has broad shoulders.`);
			} else if (slave.shoulders === 2) {
				r.push(`${He} has very broad shoulders.`);
			}
			if (slave.shouldersImplant === 0) {
				r.push(`${He} has a natural shoulder structure.`);
			} else if (slave.shouldersImplant > 3) {
				r.push(`${His} shoulders have been completely rebuilt to be massively wider.`);
			} else if (slave.shouldersImplant > 2) {
				r.push(`${His} shoulders have been heavily restructured to broaden them.`);
			} else if (slave.shouldersImplant > 1) {
				r.push(`${His} shoulders have been repeatedly restructured to broaden them.`);
			} else if (slave.shouldersImplant > 0) {
				r.push(`${His} shoulders have been restructured to broaden them.`);
			} else if (slave.shouldersImplant < -3) {
				r.push(`${His} shoulders have been completely rebuilt to be massively narrower.`);
			} else if (slave.shouldersImplant < -2) {
				r.push(`${His} shoulders have been heavily restructured to narrow them.`);
			} else if (slave.shouldersImplant < -1) {
				r.push(`${His} shoulders have been repeatedly restructured to narrow them.`);
			} else if (slave.shouldersImplant < 0) {
				r.push(`${His} shoulders have been restructured to narrow them.`);
			}
			if (slave.indentureRestrictions > 0) {
				App.UI.DOM.appendNewElement("div", el, `${His} indenture forbids elective surgery`, ["choices", "note"]);
			} else {
				if (slave.shouldersImplant === 0 || V.surgeryUpgrade === 1) {
					const broaden = slave.shouldersImplant === 0 ? "Restructure shoulders more broadly" : "Advanced shoulder broadening surgery";
					const narrow = slave.shouldersImplant === 0 ? "Restructure shoulders more narrowly" : "Advanced shoulder narrowing surgery";
					if (slave.shoulders < 2) {
						linkArray.push(makeLink(
							broaden,
							"shoulders",
							() => {
								slave.shouldersImplant++;
								slave.shoulders++;
								surgeryDamage(slave, 40);
							}
						));
					}
					if (slave.shoulders > -2) {
						linkArray.push(makeLink(
							narrow,
							"shoulders",
							() => {
								slave.shouldersImplant--;
								slave.shoulders--;
								surgeryDamage(slave, 40);
							}
						));
					}
				}
			}
			App.Events.addNode(el, r, "div");
			App.UI.DOM.appendNewElement("div", el, App.UI.DOM.generateLinksStrip(linkArray), "choices");
			return el;
		}

		function hips() {
			const el = new DocumentFragment();
			const r = [];
			const linkArray = [];
			if (slave.hips < -1) {
				r.push(`${He} has very narrow hips.`);
			} else if (slave.hips < 0) {
				r.push(`${He} has narrow hips.`);
			} else if (slave.hips < 1) {
				r.push(`${He} has average hips.`);
			} else if (slave.hips < 2) {
				r.push(`${He} has broad hips.`);
			} else if (slave.hips < 3) {
				r.push(`${He} has very broad hips.`);
			} else if (slave.hips === 3) {
				r.push(`${He} has doorway-jamming hips.`);
			}
			if (slave.hipsImplant === 0) {
				r.push(`They have not been altered.`);
			} else if (slave.hipsImplant > 3) {
				r.push(`${His} pelvis has been completely rebuilt to be massively wider.`);
			} else if (slave.hipsImplant > 2) {
				r.push(`${His} pelvis has been heavily broadened.`);
			} else if (slave.hipsImplant > 1) {
				r.push(`${His} pelvis has been repeatedly broadened.`);
			} else if (slave.hipsImplant > 0) {
				r.push(`${His} pelvis has been broadened.`);
			} else if (slave.hipsImplant < -3) {
				r.push(`${His} pelvis has been completely rebuilt to be massively narrower.`);
			} else if (slave.hipsImplant < -2) {
				r.push(`${His} pelvis has heavily narrowed.`);
			} else if (slave.hipsImplant < -1) {
				r.push(`${His} pelvis has been repeatedly narrowed.`);
			} else if (slave.hipsImplant < 0) {
				r.push(`${His} pelvis has been narrowed.`);
			}
			if (slave.indentureRestrictions > 0) {
				App.UI.DOM.appendNewElement("div", el, `${His} indenture forbids elective surgery`, ["choices", "note"]);
			} else {
				if (slave.hipsImplant === 0) {
					if (slave.hips < 2 || (slave.hips < 3 && V.surgeryUpgrade === 1)) {
						linkArray.push(makeLink(
							"Broaden pelvis",
							"hips",
							() => {
								slave.hipsImplant++;
								slave.hips++;
								surgeryDamage(slave, 40);
							}
						));
					}
					if (slave.hips > -2) {
						linkArray.push(makeLink(
							"Narrow pelvis",
							"hips",
							() => {
								slave.hipsImplant--;
								slave.hips--;
								surgeryDamage(slave, 40);
							}
						));
					}
				} else if (V.surgeryUpgrade === 1) {
					if (slave.hips < 3) {
						linkArray.push(makeLink(
							"Advanced pelvis broadening",
							"hips",
							() => {
								slave.hipsImplant++;
								slave.hips++;
								surgeryDamage(slave, 40);
							}
						));
					}
					if (slave.hips > -1) {
						linkArray.push(makeLink(
							"Advanced pelvis narrowing",
							"hips",
							() => {
								slave.hipsImplant--;
								slave.hips--;
								surgeryDamage(slave, 40);
							}
						));
					}
				}
			}

			App.Events.addNode(el, r, "div");
			App.UI.DOM.appendNewElement("div", el, App.UI.DOM.generateLinksStrip(linkArray), "choices");
			return el;
		}

		function limbLength() {
			const el = new DocumentFragment();
			const r = [];
			const linkArray = [];
			const increaseHeight = function() {
				slave.heightImplant += 1;
				slave.height += 10;
				surgeryDamage(slave, 40);
			};
			const decreaseHeight = function() {
				slave.heightImplant -= 1;
				slave.height -= 10;
				surgeryDamage(slave, 40);
			};

			if (slave.heightImplant === 0) {
				r.push(`${He} has normal femurs and humeri.`);
			} else if (slave.heightImplant > 0) {
				r.push(`${His} femurs, humeri, and other major bones have been `);
				if (slave.heightImplant >= 1) {
					r.push(`considerably `);
				}
				r.push(`lengthened.`);
			} else if (slave.heightImplant < 0) {
				r.push(`${His} femurs, humeri, and other major bones have been `);
				if (slave.heightImplant <= -1) {
					r.push(`considerably `);
				}
				r.push(`shortened.`);
			}
			if (slave.indentureRestrictions < 1) {
				if (slave.heightImplant === 0) {
					linkArray.push(makeLink(
						"Lengthen major bones",
						"height",
						increaseHeight
					));
					linkArray.push(makeLink(
						"Shorten major bones",
						"height",
						decreaseHeight
					));
				} else if (slave.heightImplant >= 1) {
					if (slave.height < (Height.mean(slave) + 15) && V.surgeryUpgrade === 1) {
						linkArray.push(makeLink(
							"Advanced height gain surgery",
							"height",
							increaseHeight
						));
					}
					if (slave.heightImplant === 1) {
						linkArray.push(makeLink(
							"Reverse existing height surgery",
							"height",
							decreaseHeight
						));
					} else if (V.surgeryUpgrade === 1) {
						linkArray.push(makeLink(
							"Revert a stage of existing height surgery",
							"height",
							decreaseHeight
						));
					}
				} else if (slave.heightImplant <= -1) {
					if (slave.height >= (Height.mean(slave) - 15) && V.surgeryUpgrade === 1) {
						linkArray.push(makeLink(
							"Advanced height reduction surgery",
							"height",
							decreaseHeight
						));
					}
					if (slave.heightImplant === -1) {
						linkArray.push(makeLink(
							"Reverse existing height surgery",
							"height",
							increaseHeight
						));
					} else if (V.surgeryUpgrade === 1) {
						linkArray.push(makeLink(
							"Revert a stage of existing height surgery",
							"height",
							increaseHeight
						));
					}
				}
			}

			App.Events.addNode(el, r, "div");
			App.UI.DOM.appendNewElement("div", el, App.UI.DOM.generateLinksStrip(linkArray), "choices");
			return el;
		}

		function tendons() {
			const el = new DocumentFragment();
			const r = [];
			const linkArray = [];
			if (slave.heels === 0 && hasAnyNaturalLegs(slave)) {
				r.push(`${He} has healthy calves.`);
			} else if (slave.heels === 1 && hasAnyNaturalLegs(slave)) {
				r.push(`${His} calves have been altered so that ${he} cannot walk in anything but very high heels.`);
			}
			if (slave.heels === 0 && hasAnyNaturalLegs(slave) && V.seeExtreme === 1) {
				if (slave.indentureRestrictions < 1) {
					linkArray.push(makeLink(
						"Shorten tendons",
						"heels",
						() => {
							slave.heels = 1;
							slave.shoes = "heels";
							surgeryDamage(slave, 20);
						},
						1,
						`Prevents ${him} from walking in anything but very high heels`
					));
				}
			}
			if (slave.heels === 1) {
				linkArray.push(makeLink(
					"Replace tendons",
					"heelsRestoration",
					() => {
						slave.heels = 0;
						slave.shoes = "none";
						surgeryDamage(slave, 10);
					}
				));
			}
			App.Events.addNode(el, r, "div");
			App.UI.DOM.appendNewElement("div", el, App.UI.DOM.generateLinksStrip(linkArray), "choices");
			return el;
		}

		function amputate() {
			const el = new DocumentFragment();
			const r = [];
			r.push(App.Desc.limbs(slave));
			if (isAmputee(slave)) {
				if (slave.PLimb === 1) {
					r.push(`${He} has been implanted with a basic PLimb interface.`);
				} else if (slave.PLimb === 2) {
					r.push(`${He} has been implanted with an advanced PLimb interface.`);
				}
			}
			App.Events.addNode(el, r, "div");

			if (V.seeExtreme === 1 && slave.indentureRestrictions < 1 && hasAnyNaturalLimbs(slave)) {
				el.append(amputateSelector());
			}
			return el;
		}

		/**
		 * @returns {HTMLDivElement}
		 */
		function amputateSelector() {
			const outerDiv = document.createElement("div");
			outerDiv.classList.add("choices");
			const linkContainer = document.createElement("div");

			const allLimbs = makeCheckbox("all-limbs");
			/**
			 * @type {Array<HTMLInputElement>}
			 */
			const limbs = [];
			if (getLeftArmID(slave) === 1) {
				limbs.push(makeCheckbox("left-arm"));
			}
			if (getRightArmID(slave) === 1) {
				limbs.push(makeCheckbox("right-arm"));
			}
			if (getLeftLegID(slave) === 1) {
				limbs.push(makeCheckbox("left-leg"));
			}
			if (getRightLegID(slave) === 1) {
				limbs.push(makeCheckbox("right-leg"));
			}

			allLimbs.onchange = () => {
				const checked = allLimbs.checked;
				limbs.forEach(l => l.checked = checked);
				App.UI.DOM.replace(linkContainer, link(limbs.length));
			};

			const checkAllBox = function() {
				let checkedCount = 0;
				limbs.forEach(l => {
					if (l.checked) {
						checkedCount++;
					}
				});

				if (checkedCount === 0) {
					allLimbs.checked = false;
					allLimbs.indeterminate = false;
				} else if (checkedCount === limbs.length) {
					allLimbs.checked = true;
					allLimbs.indeterminate = false;
				} else {
					allLimbs.checked = false;
					allLimbs.indeterminate = true;
				}
				App.UI.DOM.replace(linkContainer, link(checkedCount));
			};

			limbs.forEach(l => l.onchange = checkAllBox);

			const idToNames = new Map([
				["left-arm", {displayName: "Left arm", surgeryName: "left arm"}],
				["right-arm", {displayName: "Right arm", surgeryName: "right arm"}],
				["left-leg", {displayName: "Left leg", surgeryName: "left leg"}],
				["right-leg", {displayName: "Right leg", surgeryName: "right leg"}]
			]);

			appendCheckBox(outerDiv, allLimbs, "All limbs");
			const div = document.createElement("div");
			div.classList.add("choices");
			limbs.forEach(l => appendCheckBox(div, l, idToNames.get(l.id).displayName));
			outerDiv.append(div);

			linkContainer.append(link(0));
			outerDiv.append(linkContainer);

			/**
			 * @param {number} checkedCount
			 * @returns {HTMLAnchorElement|HTMLSpanElement}
			 */
			function link(checkedCount) {
				if (checkedCount) {
					return App.UI.DOM.link(
						(checkedCount === 1) ? "Amputate limb" : "Amputate limbs",
						() => {
							const oldLimbs = App.Medicine.Limbs.currentLimbs(slave);
							let count = 0;
							limbs.forEach(l => {
								if (l.checked) {
									removeLimbs(slave, idToNames.get(l.id).surgeryName);
									count++;
								}
							});
							if (count > 0) {
								V.oldLimbs = oldLimbs;
								surgeryDamage(slave, count * 10);
								cashX(-count * V.surgeryCost, "slaveSurgery", slave);
								if (!hasAnyArms(slave)) {
									slave.rules.release.masturbation = 0;
								}
								V.surgeryType = "amp";
								Engine.play("Surgery Degradation");
							}
						},
						[],
						"",
						` This will greatly restrict ${him}.`
					);
				} else {
					return App.UI.DOM.disabledLink("Amputate limb(s)", ["No limb selected"]);
				}
			}

			/**
			 * @param {string} id must be unique
			 * @returns {HTMLInputElement}
			 */
			function makeCheckbox(id) {
				const checkbox = document.createElement("input");
				checkbox.id = id;
				checkbox.setAttribute("type", "checkbox");
				checkbox.checked = false;
				return checkbox;
			}

			/**
			 * @param {HTMLElement} container
			 * @param {HTMLInputElement} box
			 * @param {string} name
			 */
			function appendCheckBox(container, box, name) {
				const label = document.createElement("label");
				label.append(box, " ", name);
				App.UI.DOM.appendNewElement("div", container, label);
			}

			return outerDiv;
		}

		function prostheticInterface() {
			const el = new DocumentFragment();
			const linkArray = [];
			if (!hasAllNaturalLimbs(slave) && slave.PLimb === 0) {
				if (isProstheticAvailable(slave, "interfaceP1")) {
					linkArray.push(makeLink(
						"Install basic prosthetic interface",
						"PLimb interface",
						() => {
							V.oldLimbs = App.Medicine.Limbs.currentLimbs(slave);
							slave.PLimb = 1;
							surgeryDamage(slave, 20);
						}
					));
				}
				if (isProstheticAvailable(slave, "interfaceP2")) {
					linkArray.push(makeLink(
						"Install advanced prosthetic interface",
						"PLimb interface",
						() => {
							V.oldLimbs = App.Medicine.Limbs.currentLimbs(slave);
							slave.PLimb = 2;
							surgeryDamage(slave, 20);
						}
					));
				}
			} else if (slave.PLimb === 1 && isProstheticAvailable(slave, "interfaceP2")) {
				linkArray.push(makeLink(
					"Upgrade to advanced prosthetic interface",
					"PLimb interface",
					() => {
						V.oldLimbs = App.Medicine.Limbs.currentLimbs(slave);
						slave.PLimb = 2;
						surgeryDamage(slave, 5);
					}
				));
			}
			App.UI.DOM.appendNewElement("div", el, App.UI.DOM.generateLinksStrip(linkArray), "choices");
			return el;
		}

		function tail() {
			const el = new DocumentFragment();
			const r = [];
			const linkArray = [];
			if (slave.PTail === 1) {
				r.push(`${He} has a neural interface allowing attachment of tails.`);
			} else if (isProstheticAvailable(slave, "interfaceTail")) {
				r.push(`${He} lacks a neural interface allowing attachment of tails.`);
				linkArray.push(makeLink(
					"Implant interface",
					"tailInterface",
					() => {
						slave.PTail = 1;
						slave.tail = "none";
						slave.tailColor = "none";
						surgeryDamage(slave, 10);
					}
				));
			} else {
				r.push(`${He} lacks a neural interface allowing attachment of tails and you have none ready for ${him}.`);
			}
			App.Events.addNode(el, r, "div");
			App.UI.DOM.appendNewElement("div", el, App.UI.DOM.generateLinksStrip(linkArray), "choices");
			return el;
		}
	}

	/**
	 *
	 * @param {string} title
	 * @param {string} surgeryType
	 * @param {function(void):void} [func]
	 * @param {number} [costMult=1]
	 * @param {string} [note]
	 * @returns {HTMLAnchorElement}
	 */
	function makeLink(title, surgeryType, func, costMult = 1, note = "") {
		const cost = Math.trunc(V.surgeryCost * costMult);
		return App.UI.DOM.link(
			title,
			() => {
				if (typeof func === "function") {
					func();
				}
				if (cheat) {
					jQuery(container).empty().append(content());
					App.Art.refreshSlaveArt(slave, 3, "art-frame");
				} else {
					V.surgeryType = surgeryType;
					// TODO: pass if it affected health or not?
					cashX(forceNeg(cost), "slaveSurgery", slave);
					Engine.play("Surgery Degradation");
				}
			},
			[],
			"",
			`Costs ${cashFormat(cost)}${note ? ` ${note}` : ""}`
		);
	}
};
