/**
 * @param {App.Entity.SlaveState} slave
 */
App.UI.prostheticsConfig = function(slave) {
	/* get all prosthetics that are ready for this slave */
	if (V.adjustProstheticsCompleted > 0) {
		V.adjustProsthetics = V.adjustProsthetics.filter(function(p) {
			if (p.workLeft <= 0 && p.slaveID === slave.ID) {
				addProsthetic(slave, p.id);
				V.adjustProstheticsCompleted--;
				return false;
			}
			return true;
		});
	}

	const {He, his, him} = getPronouns(slave);

	const fragment = document.createDocumentFragment();

	App.UI.DOM.appendNewElement("h1", fragment, "Prosthetic Configuration");
	const introP = App.UI.DOM.makeElement("p", "This room is lined with shelves and cabinets; it could be easily mistaken for a storage room if it were not for the examination table in its center.", "scene-intro");

	if (hasBothLegs(slave)) {
		App.UI.DOM.appendNewElement("div", introP, `${slave.slaveName} is obediently waiting for your instructions.`);
	} else {
		App.UI.DOM.appendNewElement("div", introP, `${slave.slaveName} is lying on the table, waiting for your instructions.`);
	}

	fragment.append(introP);

	fragment.append(eyes());
	fragment.append(ears());
	fragment.append(voice());
	fragment.append(limbs());
	fragment.append(tail());

	fragment.append(buyScreen());

	return fragment;

	/**
	 * @returns {DocumentFragment}
	 */
	function eyes() {
		const f = document.createDocumentFragment();

		if (hasAnyCyberneticEyes(slave)) {
			App.UI.DOM.appendNewElement("h2", f, "Eyes");
			const p = document.createElement("p");

			p.append(`${He} has ${hasBothCyberneticEyes(slave) ? "ocular implants" : "an ocular implant"} installed. You can change ${hasBothCyberneticEyes(slave) ? "their" : "its"} settings:`);

			const eyeContainer = document.createElement("div");
			eyeContainer.classList.add("eyeContainer", "choices");

			let _on = 0;
			let _blur = 0;
			let _off = 0;
			if (getLeftEyeType(slave) === 3) {
				App.UI.DOM.appendNewElement("div", eyeContainer, "Left:");
				let div = document.createElement("div");
				if (getLeftEyeVision(slave) !== 2) {
					_on++;
					div.append(App.UI.DOM.passageLink("[ON]", "Prosthetics Configuration", () => eyeSurgery(slave, "left", "fix")));
				} else {
					div.append("[ON]");
				}
				eyeContainer.append(div);
				div = document.createElement("div");

				if (getLeftEyeVision(slave) !== 1) {
					_blur++;
					div.append(App.UI.DOM.passageLink("[BLUR]", "Prosthetics Configuration", () => eyeSurgery(slave, "left", "blur")));
				} else {
					div.append("[BLUR]");
				}
				eyeContainer.append(div);
				div = document.createElement("div");
				if (getLeftEyeVision(slave) !== 0) {
					_off++;
					div.append(App.UI.DOM.passageLink("[OFF]", "Prosthetics Configuration", () => eyeSurgery(slave, "left", "blind")));
				} else {
					div.append("[OFF]");
				}
				eyeContainer.append(div);
			}
			if (getRightEyeType(slave) === 3) {
				App.UI.DOM.appendNewElement("div", eyeContainer, "Right:");
				let div = document.createElement("div");
				if (getRightEyeVision(slave) !== 2) {
					_on++;
					div.append(App.UI.DOM.passageLink("[ON]", "Prosthetics Configuration", () => eyeSurgery(slave, "right", "fix")));
				} else {
					div.append("[ON]");
				}
				eyeContainer.append(div);
				div = document.createElement("div");

				if (getRightEyeVision(slave) !== 1) {
					_blur++;
					div.append(App.UI.DOM.passageLink("[BLUR]", "Prosthetics Configuration", () => eyeSurgery(slave, "right", "blur")));
				} else {
					div.append("[BLUR]");
				}
				eyeContainer.append(div);
				div = document.createElement("div");
				if (getRightEyeVision(slave) !== 0) {
					_off++;
					div.append(App.UI.DOM.passageLink("[OFF]", "Prosthetics Configuration", () => eyeSurgery(slave, "right", "blind")));
				} else {
					div.append("[OFF]");
				}
				eyeContainer.append(div);
			}
			if (hasBothCyberneticEyes(slave)) {
				App.UI.DOM.appendNewElement("div", eyeContainer, "Both:");
				let div = document.createElement("div");
				if (_on > 0) {
					div.append(App.UI.DOM.passageLink("[ON]", "Prosthetics Configuration", () => eyeSurgery(slave, "both", "fix")));
				} else {
					div.append("[ON]");
				}
				eyeContainer.append(div);
				div = document.createElement("div");

				if (_blur > 0) {
					div.append(App.UI.DOM.passageLink("[BLUR]", "Prosthetics Configuration", () => eyeSurgery(slave, "both", "blur")));
				} else {
					div.append("[BLUR]");
				}
				eyeContainer.append(div);
				div = document.createElement("div");
				if (_off > 0) {
					div.append(App.UI.DOM.passageLink("[OFF]", "Prosthetics Configuration", () => eyeSurgery(slave, "both", "blind")));
				} else {
					div.append("[OFF]");
				}
				eyeContainer.append(div);
			}
			p.append(eyeContainer);

			const colorDiv = document.createElement("div");
			colorDiv.append(`${He} has ${App.Desc.eyesColor(slave)}. To change ${his} eye color visit the `, App.UI.DOM.passageLink("auto salon", "Salon"), ".");
			p.append(colorDiv);

			f.append(p);
		}
		return f;
	}

	/**
	 * @returns {DocumentFragment}
	 */
	function ears() {
		const f = document.createDocumentFragment();
		if (slave.earImplant === 1) {
			App.UI.DOM.appendNewElement("h2", f, "Ears");

			const p = document.createElement("p");
			p.append(`${He} has cochlear implants installed.`);
			if (slave.hears === 0) {
				p.append("They are operating normally.");
			} else if (slave.hears === -1) {
				p.append(`They are set to muffle ${his} hearing.`);
			} else {
				p.append("They are turned off.");
			}

			const links = [];
			if (slave.hears !== 0) {
				links.push(App.UI.DOM.passageLink("Restore hearing", "Prosthetics Configuration", () => {
					slave.hears = 0;
					V.prostheticsConfig = "hearing";
				}));
			}
			if (slave.hears !== -1) {
				links.push(App.UI.DOM.passageLink("Muffle hearing", "Prosthetics Configuration", () => {
					slave.hears = -1;
					V.prostheticsConfig = "hearing";
				}));
			}
			if (slave.hears !== -2) {
				links.push(App.UI.DOM.passageLink("Disable", "Prosthetics Configuration", () => {
					slave.hears = -2;
					V.prostheticsConfig = "hearing";
				}));
			}
			App.UI.DOM.appendNewElement("div", p, linkStrip(links), "choices");

			f.append(p);
		}
		return f;
	}

	/**
	 * @returns {DocumentFragment}
	 */
	function voice() {
		const f = document.createDocumentFragment();

		if (slave.electrolarynx === 1) {
			App.UI.DOM.appendNewElement("h2", f, "Voice");
			const p = document.createElement("p");
			p.append(`${He} has an electrolarynx installed.`);
			if (slave.voice === 0) {
				p.append("It is turned off.");
			} else if (slave.voice === 1) {
				p.append(`It is set to its "deep voice" setting.`);
			} else if (slave.voice === 2) {
				p.append(`It is set to its "normal voice" setting.`);
			} else if (slave.voice === 3) {
				p.append(`It is set to its "high voice" setting.`);
			}

			const links = [];
			if (slave.voice !== 0) {
				links.push(App.UI.DOM.passageLink("Disable", "Prosthetics Configuration", () => {
					slave.voice = 0;
					V.prostheticsConfig = "voice";
				}));
			}
			if (slave.voice !== 1) {
				links.push(App.UI.DOM.passageLink("Deep voice setting", "Prosthetics Configuration", () => {
					slave.voice = 1;
					V.prostheticsConfig = "voice";
				}));
			}
			if (slave.voice !== 2) {
				links.push(App.UI.DOM.passageLink("Standard voice setting", "Prosthetics Configuration", () => {
					slave.voice = 2;
					V.prostheticsConfig = "voice";
				}));
			}
			if (slave.voice !== 3) {
				links.push(App.UI.DOM.passageLink("High voice setting", "Prosthetics Configuration", () => {
					slave.voice = 3;
					V.prostheticsConfig = "voice";
				}));
			}
			App.UI.DOM.appendNewElement("div", p, linkStrip(links), "choices");

			f.append(p);
		}
		return f;
	}

	/**
	 * @returns {DocumentFragment}
	 */
	function limbs() {
		const f = document.createDocumentFragment();
		App.UI.DOM.appendNewElement("h2", f, "Limbs");

		App.UI.DOM.appendNewElement("p", f, App.Medicine.Limbs.selector(slave, App.Medicine.Limbs.currentLimbs(slave)));

		return f;
	}

	/**
	 * @returns {DocumentFragment}
	 */
	function tail() {
		const f = document.createDocumentFragment();
		App.UI.DOM.appendNewElement("h2", f, "Tail");

		const p = document.createElement("p");

		if (slave.PTail === 1) {
			App.UI.DOM.appendNewElement("div", p, `${He} has a neural tail interface installed. You can assign and adjust ${his} tail here.`);

			if (slave.tail !== "none") {
				App.UI.DOM.appendNewElement("div", p, `${He} currently has a tail attached, if you wish to change it you first need to detach it.`);
				App.UI.DOM.appendNewElement("div", p,
					App.UI.DOM.passageLink("Detach", "Prosthetics Configuration", () => {
						V.prostheticsConfig = "detachTail";
						V.nextButton = "Continue";
						V.nextLink = "Prosthetics Configuration";
					}), "choices");
			} else {
				if (isProstheticAvailable(slave, "modT")) {
					App.UI.DOM.appendNewElement("div", p, "Attach a modular tail designed to look like a:");

					const links = [];
					App.Data.modTails.forEach((value, key) => {
						links.push(App.UI.DOM.passageLink(`${value.animal}'s Tail`, "Prosthetics Configuration",
							() => {
								V.prostheticsConfig = "attachTail";
								slave.tail = "mod";
								slave.tailShape = key;
								slave.tailColor = slave.hColor;
							}
						));
					});
					App.UI.DOM.appendNewElement("div", p, linkStrip(links), "choices");
				}
				const links = [];
				if (isProstheticAvailable(slave, "combatT")) {
					links.push(App.UI.DOM.passageLink("Attach Combat Tail", "Prosthetics Configuration", () => {
						V.prostheticsConfig = "attachTail";
						slave.tail = "combat";
						slave.tailColor = "jet black";
					}));
				}
				if (isProstheticAvailable(slave, "sexT")) {
					links.push(App.UI.DOM.passageLink("Attach Pleasure Tail", "Prosthetics Configuration", () => {
						V.prostheticsConfig = "attachTail";
						slave.tail = "sex";
						slave.tailColor = "pink";
					}));
				}
				App.UI.DOM.appendNewElement("div", p, linkStrip(links), "choices");
			}

			if (slave.tail === "mod") {
				App.UI.DOM.appendNewElement("div", p,
					`${He} currently has a modular tail, styled to look like ${App.Data.modTails.get(slave.tailShape).desc}. Modify ${his} tail's appearance:`);

				const links = [];
				App.Data.modTails.forEach((value, key) => {
					links.push(App.UI.DOM.passageLink(value.animal, "Prosthetics Configuration", () => {
						slave.tailShape = key;
						cashX(forceNeg(V.modCost), "slaveMod", slave);
					}));
				});
				App.UI.DOM.appendNewElement("div", p, linkStrip(links), "choices");
			}
		} else {
			App.UI.DOM.appendNewElement("span", p, `${He} does not have a neural tail interface installed so you cannot attach a tail.`, "note");
		}
		f.append(p);
		return f;
	}

	/**
	 * @returns {DocumentFragment}
	 */
	function buyScreen() {
		const f = document.createDocumentFragment();
		App.UI.DOM.appendNewElement("h2", f, "Prosthetics");

		const p = document.createElement("p");
		App.UI.DOM.appendNewElement("span", p, `Fit prosthetics to ${him}:`, "note");

		const gridDiv = document.createElement("div");
		gridDiv.classList.add("buy-prosthetics");

		gridDiv.append(document.createElement("div"));

		App.UI.DOM.appendNewElement("div", gridDiv, "Buy and fit");

		let tooltip;
		if (V.researchLab.level > 0) {
			tooltip = `Depending on lab speed, it might be faster than fitting an existing prosthetic, but should almost always be faster than first building and then fitting it to ${him}.`;
		} else {
			tooltip = "With a lab you could both increase speed and decrease cost.";
		}
		App.UI.DOM.appendNewElement("div", gridDiv, App.UI.DOM.spanWithTooltip("Construct in lab", tooltip));

		tooltip = "";
		if (V.researchLab.speed >= 300) { /* max speed */
			tooltip = "Your lab is so fast that fitting prosthetics to your slave can be done instantly, though you will sacrifice some efficiency.";
		} else if (V.researchLab.level > 0) {
			tooltip = "Your lab is not fast enough to fit prosthetics instantly.";
		}

		const fastDiv = document.createElement("div");
		if (tooltip!=="") {
			fastDiv.append(App.UI.DOM.spanWithTooltip("Fast assembly", tooltip));
			fastDiv.style.textAlign = "right";
		}
		gridDiv.append(fastDiv);

		for (const prostheticID of App.Data.prostheticIDs) {
			// TODO comment erectile out in data directly
			if (prostheticID !== "erectile") { /* exclude erectile implant */
				addBuyRow(prostheticID, gridDiv);
			}
		}
		p.append(gridDiv);
		f.append(p);

		return f;
	}

	/**
	 * @param {string} prosthetic
	 * @param {HTMLDivElement} container
	 */
	function addBuyRow(prosthetic, container) {
		App.UI.DOM.appendNewElement("div", container, capFirstChar(App.Data.prosthetics[prosthetic].name));

		if (V.adjustProsthetics.findIndex(function(p) { return p.id === prosthetic && p.slaveID === slave.ID; }) !== -1
			|| V.researchLab.tasks.findIndex(function(p) { return p.type === "craftFit" && p.id === prosthetic && p.slaveID === slave.ID; }) !== -1) {
			App.UI.DOM.appendNewElement("div", container, `Currently being fitted to ${him}.`, ["full", "note"]);
		} else if (App.Data.prosthetics[prosthetic].level > V.prostheticsUpgrade) {
			App.UI.DOM.appendNewElement("div", container, `Better contracts are needed to buy these.`, ["full", "note"]);
		} else if (isProstheticAvailable(slave, prosthetic)) {
			App.UI.DOM.appendNewElement("div", container, `Completed.`, ["full", "note"]);
		} else {
			if (V.prosthetics[prosthetic].amount > 0) {
				App.UI.DOM.appendNewElement("div", container,
					App.UI.DOM.passageLink("From storage", "Prosthetics Configuration", () => {
						V.adjustProsthetics.push({
							id: prosthetic,
							workLeft: App.Data.prosthetics[prosthetic].adjust,
							slaveID: slave.ID
						});
						V.prosthetics[prosthetic].amount -= 1;
					}));
			} else {
				App.UI.DOM.appendNewElement("div", container,
					App.UI.DOM.passageLink(cashFormat(App.Data.prosthetics[prosthetic].costs), "Prosthetics Configuration", () => {
						V.adjustProsthetics.push({
							id: prosthetic,
							workLeft: App.Data.prosthetics[prosthetic].adjust,
							slaveID: slave.ID
						});
						cashX(forceNeg(App.Data.prosthetics[prosthetic].costs), "slaveMod", slave);
					}));
			}

			if (V.prosthetics[prosthetic].research > 0) {
				const craftDiv = document.createElement("div");
				craftDiv.style.textAlign = "center";
				if (V.researchLab.level > 0 && V.prosthetics[prosthetic].research > 0) {
					craftDiv.append(App.UI.DOM.passageLink("Construct", "Prosthetics Configuration", () => {
						V.researchLab.tasks.push({
							type: "craftFit",
							id: prosthetic,
							/* 1.5: longer than adjust, but faster than adjust+craft. */
							workLeft: (App.Data.prosthetics[prosthetic].adjust + App.Data.prosthetics[prosthetic].craft) / 1.5,
							slaveID: slave.ID
						});
					}));
				}
				container.append(craftDiv);

				const instantDiv = document.createElement("div");
				instantDiv.style.textAlign = "right";
				if (V.researchLab.speed >= 300 && V.prosthetics[prosthetic].research > 0) { /* max speed */
					if (V.prosthetics[prosthetic].amount > 0) {
						const cost = App.Data.prosthetics[prosthetic].adjust * 50;
						instantDiv.append(App.UI.DOM.passageLink(`From storage: ${cashFormat(cost)}`,
							"Prosthetics Configuration", () => {
								cashX(-cost, "slaveMod", slave);
								addProsthetic(slave, prosthetic);
							}));
					} else {
						const cost = App.Data.prosthetics[prosthetic].costs * 2 + App.Data.prosthetics[prosthetic].adjust * 50;
						instantDiv.append(App.UI.DOM.passageLink(cashFormat(cost), "Prosthetics Configuration", () => {
							cashX(-cost, "slaveMod", slave);
							addProsthetic(slave, prosthetic);
						}));
					}
				}
				container.append(instantDiv);
			} else if (V.researchLab.level > 0) {
				App.UI.DOM.appendNewElement("div", container, "Not researched.", ["research", "note"]);
			} else {
				App.UI.DOM.appendNewElement("div", container, "You need to construct a lab first.", ["research", "note"]);
			}
		}
	}

	/**
	 * @param {Array<Element>} links
	 * @returns {DocumentFragment}
	 */
	function linkStrip(links) {
		const f = document.createDocumentFragment();
		if (links.length > 0) {
			f.append(links[0]);
			for (let i = 1; i < links.length; i++) {
				f.append(" | ", links[i]);
			}
		}
		return f;
	}
};
