/**
 *
 * @param {FC.SlaveMarketName} slaveMarket
 * @param {object} [param1]
 * @param {string} [param1.sTitleSingular]
 * @param {string} [param1.sTitlePlural]
 * @param {number} [param1.costMod]
 */
App.Markets.purchaseFramework = function(slaveMarket, {sTitleSingular = "slave", sTitlePlural = "slaves", costMod = 1} = {}) {
	const el = new DocumentFragment();
	const {slave, text} = generateMarketSlave(slaveMarket, (V.market.numArcology || 1));
	const cost = getCost();
	let prisonCrime = "";
	if (slaveMarket === V.prisonCircuit[V.prisonCircuitIndex]) {
		prisonCrime = pronounsForSlaveProp(slave, text);
	} else {
		$(el).append(` ${text}`);
	}

	App.UI.DOM.appendNewElement("p",
		el,
		`The offered price is ${cashFormat(cost)}. ${(V.slavesSeen > V.slaveMarketLimit) ? `You have cast such a wide net for slaves this week that it is becoming more expensive to find more for sale. Your reputation helps determine your reach within the slave market.` : ``}`
	);

	el.append(choices());
	return el;

	function getCost() {
		let cost = slaveCost(slave, false, !App.Data.misc.lawlessMarkets.includes(slaveMarket));
		if (V.slavesSeen > V.slaveMarketLimit) {
			cost += cost * ((V.slavesSeen - V.slaveMarketLimit) * 0.1);
		}
		if (costMod > 0 && costMod <= 1) {
			cost = cost * costMod;
		} else {
			cost = cost + costMod;
		}
		console.log("CostMod: ", costMod);
		cost = 500 * Math.trunc(cost / 500);
		return cost;
	}

	function choices() {
		const {him, his} = getPronouns(slave);
		const el = document.createElement("p");
		let title = {};
		V.slavesSeen += 1;
		if (sTitleSingular === "prisoner") {
			title = {
				decline: `Inspect a different prisoner`,
				buyAndKeepShopping: `Buy ${him} and check out other available ${sTitlePlural}`,
				buyJustHer: `Enslave ${him}`,
				buyHerAndFinish: `Enslave ${him} and finish your inspection`,
				finish: `Finish your enslavement of prisoners`
			};
		} else {
			title = {
				decline: `Decline to purchase ${him} and check out another ${sTitleSingular}`,
				buyAndKeepShopping: `Buy ${him} and check out other ${sTitlePlural} to order`,
				buyJustHer: `Buy ${his} slave contract`,
				buyHerAndFinish: `Buy ${him} and finish your order of slaves`,
				finish: `Finish your order of slaves`
			};
		}

		App.UI.DOM.appendNewElement(
			"div",
			el,
			App.UI.DOM.link(
				title.decline,
				() => {
					jQuery("#slave-markets").empty().append(App.Markets[slaveMarket]);
				},
			)
		);
		if (V.cash >= cost) {
			App.UI.DOM.appendNewElement(
				"div",
				el,
				App.UI.DOM.link(
					title.buyAndKeepShopping,
					() => {
						cashX(forceNeg(cost), "slaveTransfer", slave);
						V.market.totalCost += cost;
						V.market.newSlaves.push(slave);
						V.market.introType = "multi";
						student();
						jQuery("#slave-markets").empty().append(App.Markets[slaveMarket]);
						V.nextLink = "Bulk Slave Intro";
						V.nextButton = "Continue";
						App.Utils.updateUserButton();
					},
				)
			);
			if (V.market.newSlaves.length === 0) {
				App.UI.DOM.appendNewElement(
					"div",
					el,
					App.UI.DOM.link(
						title.buyJustHer,
						() => {
							cashX(forceNeg(cost), "slaveTransfer", slave);
							V.market.totalCost += cost;
							V.market.newSlaves.push(slave);
							V.nextButton = "Continue";
							V.returnTo = "Main";
							student();
							jQuery("#slave-markets").empty().append(App.UI.newSlaveIntro(slave));
						},
					)
				);
			} else {
				App.UI.DOM.appendNewElement(
					"div",
					el,
					App.UI.DOM.link(
						title.buyHerAndFinish,
						() => {
							student();
							cashX(forceNeg(cost), "slaveTransfer", slave);
							V.market.totalCost += cost;
							V.market.newSlaves.push(slave);
						},
						[],
						"Bulk Slave Intro"
					)
				);
			}
		} else {
			App.UI.DOM.appendNewElement("span", el, `You lack the necessary funds to buy this slave.`, "note");
		}
		if (V.market.newSlaves.length > 0) {
			App.UI.DOM.appendNewElement(
				"div",
				el,
				App.UI.DOM.passageLink(
					title.finish,
					"Bulk Slave Intro",
				)
			);
		}

		el.append(App.Desc.longSlave(slave, {market: slaveMarket, prisonCrime: prisonCrime}));
		return el;

		function student() {
			if (App.Data.misc.schools.has(slaveMarket)) {
				V[slaveMarket].schoolSale = 0;
				V[slaveMarket].studentsBought += 1;
			}
		}
	}
};
/**
 * @typedef {object} marketGlobal
 * @property {FC.SlaveMarketName} slaveMarket
 * @property {string} introType
 * @property {Array<FC.GingeredSlave>} newSlaves
 * @property {number} newSlaveIndex
 * @property {number} newSlavesDone
 * @property {number} numSlaves
 * @property {number} numArcology
 * @property {number} totalCost
 */

/** @this {marketGlobal} */
App.Markets.GlobalVariable = function() {
	/** @type {FC.SlaveMarketName} */
	this.slaveMarket = "kidnappers";
	this.introType = "";
	this.newSlaves = [];
	this.newSlaveIndex = 0;
	this.newSlavesDone = 0;
	this.numSlaves = 0;
	this.numArcology = 0;
	this.totalCost = 0;
	return this;
};

/**
 * User facing names for the markets
 * @param {*} market
 * @param {*} arcIndex
 */
App.Markets.marketName = function(market = "kidnappers", arcIndex = 1) {
	if (App.Data.misc.schools.has(market)) {
		return App.Data.misc.schools.get(market).title;
	} else {
		switch (market) {
			case "corporate":
				return `your corporation`;
			case "neighbor":
				return `${V.arcologies[arcIndex].name}`;
			case "kidnappers":
				return `the Kidnappers' Market`;
			case "indentures":
				return `the Indentures Market`;
			case "hunters":
				return `the Runaway Hunters' Market`;
			case "raiders":
				return `the Raiders' Market`;
			case "underage raiders":
				return `the Raiders' Black Market`;
			case "heap":
				return `the Flesh Heap as alive as when you purchased them.`;
			case "wetware":
				return `the Wetware CPU market`;
			case "trainers":
				return `the Trainers' Market`;
			case "low tier criminals":
			case "gangs and smugglers":
			case "white collar":
			case "military prison":
				return `the prisoner sale`;
			default:
				return `Someone messed up. ${market} is not known.`;
		}
	}
};
