App.Medicine.Limbs = {};

/**
 * Generates an object usable with the standard limb check functions.
 * @param {App.Entity.SlaveState} slave
 * @returns {FC.LimbsState}
 */
App.Medicine.Limbs.currentLimbs = function(slave) {
	let s = {arm: {left: {type: 1}, right: {type: 1}}, leg: {left: {type: 1}, right: {type: 1}}, PLimb: 0};
	if (hasLeftArm(slave)) {
		s.arm.left.type = getLeftArmID(slave);
	} else {
		s.arm.left = null;
	}
	if (hasRightArm(slave)) {
		s.arm.right.type = getRightArmID(slave);
	} else {
		s.arm.right = null;
	}
	if (hasLeftLeg(slave)) {
		s.leg.left.type = getLeftLegID(slave);
	} else {
		s.leg.left = null;
	}
	if (hasRightLeg(slave)) {
		s.leg.right.type = getRightLegID(slave);
	} else {
		s.leg.right = null;
	}
	s.PLimb = slave.PLimb;
	return s;
};

/**
 * @param {App.Entity.SlaveState} slave
 * @param {FC.LimbsState} oldLimbs
 * @param {string} returnTo
 * @returns {Element|DocumentFragment}
 */
App.Medicine.Limbs.amputate = function(slave, oldLimbs, returnTo) {
	const {his} = getPronouns(slave);

	const r = [];

	let implant = false;
	if (slave.PLimb < 1 && isProstheticAvailable(slave, "interfaceP1")) {
		implant = true;
		r.push(App.UI.DOM.makeElement("div", App.UI.DOM.link("Install basic interface", () => install(1))));
	}
	if (slave.PLimb < 2 && isProstheticAvailable(slave, "interfaceP2")) {
		implant = true;
		r.push(App.UI.DOM.makeElement("div", App.UI.DOM.link("Install advanced interface", () => install(2))));
	}

	// check if we can install a limb interface and if yes, give player the option to do so.
	if (implant) {
		const outerDiv = document.createElement("div");
		outerDiv.id = "amputate";
		outerDiv.append(
			App.UI.DOM.makeElement("div", `Since you already have a prosthetic interface prepared for this slave, you can install it during the operation. The procedure will put additional strain on ${his} health but less so than if you were to perform the procedures separately.`),
			...r,
			App.UI.DOM.makeElement("div", App.UI.DOM.link("Do not install", noInstall)));
		return outerDiv;
	}

	/**
	 * @param {number} id
	 */
	function install(id) {
		slave.PLimb = id;
		surgeryDamage(slave, 10);
		App.UI.DOM.replace("#amputate", App.Medicine.Limbs.prosthetic(slave, oldLimbs, returnTo));
	}

	function noInstall() {
		App.UI.DOM.replace("#amputate", App.Medicine.Limbs.reaction(slave, oldLimbs, returnTo));
	}

	// check if there is a limb interface installed already, if there is show limb selection screen
	if (slave.PLimb > 0) {
		return App.Medicine.Limbs.prosthetic(slave, oldLimbs, returnTo);
	}
	return App.Medicine.Limbs.reaction(slave, oldLimbs, returnTo);
};

/**
 * @param {App.Entity.SlaveState} slave
 * @param {FC.LimbsState} oldLimbs
 * @param {string} returnTo
 * @returns {DocumentFragment|HTMLDivElement}
 */
App.Medicine.Limbs.prosthetic = function(slave, oldLimbs, returnTo) {
	if (!(isProstheticAvailable(slave, "basicL") || isProstheticAvailable(slave, "sexL")
		|| isProstheticAvailable(slave, "beautyL") || isProstheticAvailable(slave, "combatL")
		|| (isProstheticAvailable(slave, "cyberneticL") && slave.PLimb > 1))) {
		return App.Medicine.Limbs.reaction(slave, oldLimbs, returnTo);
	}
	const {him} = getPronouns(slave);

	const div = document.createElement("div");
	div.id = "selector";

	App.UI.DOM.appendNewElement("div", div, `Since you already have limbs prepared for ${him} you might as well attach them while you are working on ${him}:`);
	div.append(App.Medicine.Limbs.selector(slave, oldLimbs, returnTo));

	return div;
};

/**
 * Displays a selector for prosthetic limbs of getSlave(V.AS)
 * @param {App.Entity.SlaveState} slave
 * @param {FC.LimbsState} oldLimbs
 * @param {string} [returnTo=""]
 * @returns {HTMLSpanElement|DocumentFragment}
 */
App.Medicine.Limbs.selector = function(slave, oldLimbs, returnTo = "") {
	const {her} = getPronouns(slave);
	if (hasAllNaturalLimbs(slave)) {
		return App.UI.DOM.makeElement("span", `You must amputate ${her} limbs before you can attach prosthetics.`, "detail");
	}
	if (slave.PLimb < 1) {
		return App.UI.DOM.makeElement("span", `You must surgically install a prosthetic interface before you can attach prosthetics.`, "detail");
	}

	const {his} = getPronouns(slave);
	const newState = currentState(slave);

	const f = document.createDocumentFragment();

	const limbSelector = document.createElement("div");
	limbSelector.classList.add("limb-selector");

	App.UI.DOM.appendNewElement("div", limbSelector, "");
	App.UI.DOM.appendNewElement("div", limbSelector, "Left Arm");
	App.UI.DOM.appendNewElement("div", limbSelector, "Right Arm");
	App.UI.DOM.appendNewElement("div", limbSelector, "Left Leg");
	App.UI.DOM.appendNewElement("div", limbSelector, "Right Leg");

	limbSelector.append(row("None", 0));
	App.Data.prostheticLimbs.forEach((limb, key) => {
		if (isProstheticAvailable(slave, limb.prostheticKey)) {
			if (limb.minimumInterface <= slave.PLimb) {
				limbSelector.append(row(capFirstChar(limb.short), key));
			} else {
				App.UI.DOM.appendNewElement("div", limbSelector,
					`You need to upgrade ${his} prosthetic interface to attach ${limb.short} limbs.`, ["full", "detail"]);
			}
		}
	});

	f.append(limbSelector);
	f.append(apply());

	return f;

	function apply() {
		V.AS = slave.ID;

		return App.UI.DOM.link("Apply", () => {
			applySelector(slave, newState);
			if (returnTo) {
				App.UI.DOM.replace("#selector", App.Medicine.Limbs.reaction(slave, oldLimbs, returnTo));
			} else {
				V.prostheticsConfig = "limbs";
				V.oldLimbs = oldLimbs;
				Engine.play("Prosthetics Configuration");
			}
		});
	}

	/**
	 * Generates an array with the current limbs of a slave.
	 * @param {App.Entity.SlaveState} slave
	 * @returns {number[]}
	 */
	function currentState(slave) {
		return [getLeftArmID(slave), getRightArmID(slave), getLeftLegID(slave), getRightLegID(slave)];
	}

	/**
	 * @param {number} limb
	 * @param {number} id
	 * @returns {HTMLDivElement}
	 */
	function radio(limb, id) {
		const div = document.createElement("div");

		if (newState[limb] !== 1) {
			const radio = document.createElement("input");
			radio.type = "radio";
			radio.name = "limb" + limb;
			if (newState[limb] === id) {
				radio.checked = true;
			}
			radio.onclick = () => {
				newState[limb] = id;
			};
			div.append(radio);
		}

		return div;
	}

	/**
	 * @param {string} title
	 * @param {number} id
	 * @returns {DocumentFragment}
	 */
	function row(title, id) {
		const f = document.createDocumentFragment();

		App.UI.DOM.appendNewElement("div", f, title);

		for (let i = 0; i < newState.length; i++) {
			f.append(radio(i, id));
		}

		return f;
	}

	/**
	 *
	 * @param {App.Entity.SlaveState} slave
	 * @param {number[]} newState
	 */
	function applySelector(slave, newState) {
		if (getLeftArmID(slave) !== newState[0]) {
			if (getLeftArmID(slave) > 1) {
				removeLimbs(slave, "left arm");
			}
			if (newState[0] > 1) {
				attachLimbs(slave, "left arm", newState[0]);
			}
		}
		if (getRightArmID(slave) !== newState[1]) {
			if (getRightArmID(slave) > 1) {
				removeLimbs(slave, "right arm");
			}
			if (newState[1] > 1) {
				attachLimbs(slave, "right arm", newState[1]);
			}
		}
		if (getLeftLegID(slave) !== newState[2]) {
			if (getLeftLegID(slave) > 1) {
				removeLimbs(slave, "left leg");
			}
			if (newState[2] > 1) {
				attachLimbs(slave, "left leg", newState[2]);
			}
		}
		if (getRightLegID(slave) !== newState[3]) {
			if (getRightLegID(slave) > 1) {
				removeLimbs(slave, "right leg");
			}
			if (newState[3] > 1) {
				attachLimbs(slave, "right leg", newState[3]);
			}
		}
	}
};

/**
 * @param {App.Entity.SlaveState} slave
 * @param {FC.LimbsState} oldLimbs
 * @param {string} returnTo
 * @returns {DocumentFragment}
 */
App.Medicine.Limbs.reaction = function(slave, oldLimbs, returnTo = "") {
	let r = "";
	if (oldLimbs.PLimb !== slave.PLimb) {
		r += `Prosthetic interface was ${oldLimbs.PLimb === 0 ? "none" : "basic"} and is now ${slave.PLimb === 1 ? "basic" : "advanced"}. `;
	}
	if (getLeftArmID(oldLimbs) !== getLeftArmID(slave)) {
		r += `Left arm was ${idToDescription(getLeftArmID(oldLimbs))} and is now ${idToDescription(getLeftArmID(slave))}. `;
	}
	if (getRightArmID(oldLimbs) !== getRightArmID(slave)) {
		r += `Right arm was ${idToDescription(getRightArmID(oldLimbs))} and is now ${idToDescription(getRightArmID(slave))}. `;
	}
	if (getLeftLegID(oldLimbs) !== getLeftLegID(slave)) {
		r += `Left leg was ${idToDescription(getLeftLegID(oldLimbs))} and is now ${idToDescription(getLeftLegID(slave))}. `;
	}
	if (getRightLegID(oldLimbs) !== getRightLegID(slave)) {
		r += `Right leg was ${idToDescription(getRightLegID(oldLimbs))} and is now ${idToDescription(getRightLegID(slave))}. `;
	}

	const f = document.createDocumentFragment();

	App.UI.DOM.appendNewElement("p", f, r,);
	App.UI.DOM.appendNewElement("p", f, "Slave's reaction", "note");

	if (returnTo) {
		f.append(App.UI.DOM.passageLink("Continue", returnTo));
	}

	return f;
};
